package com.bocloud.ams.booter.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.boot.config.PagerArgumentResolver;
import com.bocloud.cmp.boot.interceptor.AccessInterceptor;
import com.bocloud.cmp.boot.interceptor.LogInterceptor;
import com.megatron.framework.core.InterceptorBridge;
import com.megatron.framework.core.config.InterceptorConfiguration;
import com.megatron.framework.core.domain.InterceptorEntity;

import lombok.extern.slf4j.Slf4j;

/**
 * 拦截器配置类
 *
 * @author dmw
 */
@Slf4j
@Component
public class WebInterceptorConfiguration implements InterceptorConfiguration {

    private static final String[] ACCESS_EXCLUDES = new String[] {"/status", "/status/db", "/status/**",
        "/v1/status/**", "/login", "/sso", "/v1/logview", "/v1/logzip", "/v1/config/**", "/v1/plugins/**", "/*/*.css",
        "/*/*.js", "/*/*.png", "/*/*.jpg", "/*/*.jpeg", "/*.html", "/*/*.html", "/swagger-resources/**", "/favicon.ico",
        "/webjars/**", "/v3/**", "/swagger-ui.html/**", "**/api-docs", "/error", "/favicon.ico"};
    private static final String[] LOG_EXCLUDES = new String[] {"/*/*.css", "/*/*.js", "/*/*.png", "/*/*.jpg",
        "/*/*.jpeg", "/*.html", "/*/*.html", "/swagger-resources/**", "/favicon.ico", "/webjars/**", "/v3/**",
        "/swagger-ui.html/**", "**/api-docs", "/error", "/favicon.ico"};
    private static final String[] INCLUDES = new String[] {"/**"};
    private final LogInterceptor logInterceptor;
    private final AccessInterceptor accessInterceptor;

    @Autowired
    public WebInterceptorConfiguration(AccessInterceptor accessInterceptor, LogInterceptor logInterceptor) {
        this.logInterceptor = logInterceptor;
        this.accessInterceptor = accessInterceptor;
    }

    @Override
    public InterceptorBridge buildBridge() {
        log.info("start to build interceptors...");
        InterceptorBridge bridge = new InterceptorBridge();
        InterceptorEntity accessEntity = InterceptorEntity.builder().interceptor(accessInterceptor).includes(INCLUDES)
            .excludes(ACCESS_EXCLUDES).order(0).build();
        InterceptorEntity auditLogEntity = InterceptorEntity.builder().interceptor(logInterceptor).includes(INCLUDES)
            .excludes(LOG_EXCLUDES).order(1).build();
        List<InterceptorEntity> interceptors = new ArrayList<>();
        interceptors.add(accessEntity);
        interceptors.add(auditLogEntity);
        bridge.setInterceptors(interceptors);
        bridge.setResolvers(Collections.singletonList(new PagerArgumentResolver()));
        bridge.setMessageConverters(new ArrayList<>(0));
        log.info("build interceptors success!!!");
        return bridge;
    }
}
