package com.bocloud.ams.booter.controller.collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.collection.CollectionComponent;
import com.bocloud.ams.entity.collection.CollectionTemplate;
import com.bocloud.ams.service.collection.CollectionTemplateService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/collection/templates")
@Tag(name = "采集插件管理")
public class CollectionTemplateController {

    @Autowired
    private CollectionTemplateService collectionTemplateService;

    /**
     * 采集模板组件列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "采集插件组件列表")
    @GetMapping(value = "/components")
    public GeneralResult<GridBean<CollectionComponent>> components(Pager pager) {
        return collectionTemplateService.components(pager);
    }

    /**
     * 采集插件列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "采集插件列表")
    @GetMapping
    public GeneralResult<GridBean<CollectionTemplate>> list(Pager pager) {
        return collectionTemplateService.list(pager);
    }

    /**
     * 新增采集插件
     *
     * @param template
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "采集插件新增")
    @PostMapping
    public GeneralResult<CollectionTemplate> save(@RequestBody CollectionTemplate template,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        template.setCreatorId(context.getTarget());
        return collectionTemplateService.save(template, context.getTarget());
    }

    /**
     * 采集插件修改
     *
     * @param template
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集插件修改")
    public GeneralResult<CollectionTemplate> modify(@RequestBody CollectionTemplate template,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        template.setMenderId(context.getTarget());
        return collectionTemplateService.modify(template, context.getTarget());
    }

    /**
     * 采集插件删除
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集插件删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTemplateService.remove(id, context.getTarget());
    }

    /**
     * 采集插件详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集插件详情")
    public GeneralResult<CollectionTemplate> detail(@PathVariable(value = Common.ID) Long id) {
        return collectionTemplateService.detail(id);
    }
}
