package com.bocloud.ams.booter.controller.config;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.ams.entity.config.CmdbConfig;
import com.bocloud.ams.service.config.CmdbConfigService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/cmdb/configs")
@Tag(name = "配置中心配置管理")
public class CmdbConfigController {

    @Autowired
    private CmdbConfigService cmdbConfigService;

    /**
     * 更新CMDB配置
     *
     * @param configs
     * @param context
     * @return
     */
    @PutMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "配置中心配置修改")
    public Result modify(@RequestBody List<CmdbConfig> configs, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cmdbConfigService.batchUpdate(configs, context);
    }

    /**
     * 查询CMDB配置
     *
     * @param category
     * @param codes
     * @return
     */
    @GetMapping
    @Operation(summary = "配置中心配置批量查询")
    public GeneralResult<Map<String, String>> getByCodes(
        @RequestParam(value = "category", required = false) String category,
        @RequestParam(value = "codes", required = false) ArrayList<String> codes) {
        return cmdbConfigService.listConfigs(category, codes);
    }

    /**
     * 查询CMDB配置类型树
     *
     * @param category
     * @return
     */
    @GetMapping("/tree")
    @Operation(summary = "配置中心配置类型树")
    public GeneralResult<JSONArray> listCategoryTree(@RequestParam(value = "category") String category) {
        return cmdbConfigService.listCategoryTree(category);
    }

}
