package com.bocloud.ams.booter.controller.module;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.module.PropertyPool;
import com.bocloud.ams.service.module.PropertyPoolService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@Tag(name = "属性池管理")
@RestController
@RequestMapping("/v1/property/pool")
public class PropertyPoolController {

    @Autowired
    private PropertyPoolService propertyPoolService;

    /**
     * 查询属性池记录
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "属性池列表")
    public GeneralResult<GridBean<PropertyPool>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return propertyPoolService.list(pager);
    }

    /**
     * 新增属性池
     *
     * @param record
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "属性池新增")
    public GeneralResult<Void> save(@RequestBody PropertyPool record,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        return propertyPoolService.create(record, context.getTarget());
    }

    /**
     * 修改属性池
     *
     * @param record
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "属性池修改" )
    public GeneralResult modify(@RequestBody PropertyPool record, @PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }

        record.setId(id);
        return propertyPoolService.modify(record, context.getTarget());

    }

    /**
     * 移除属性池
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "属性池删除")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return propertyPoolService.remove(id, context.getTarget());

    }

    private boolean paramsCheck(PropertyPool record) {
        // 参数校验
        if (null == record || StringUtils.isEmpty(record.getName()) || StringUtils.isEmpty(record.getCode())
            || StringUtils.isEmpty(record.getType())) {
            return false;
        }
        // 下拉选择类型中，属性池配置项必须有值
        if (PropertyConstant.PropertyType.SINGLESELECT.equals(record.getType())
            || PropertyConstant.PropertyType.MULTISELECT.equals(record.getType())
            || PropertyConstant.PropertyType.RADIO.equals(record.getType())
            || PropertyConstant.PropertyType.CHECKBOX.equals(record.getType())) {
            if (CollectionUtils.isEmpty(record.getItemList())) {
                return false;
            }
        }

        if (PropertyConstant.PropertyType.MULTILINETABLE.equals(record.getType())
            || PropertyConstant.PropertyType.SINGLELINETABLE.equals(record.getType())) {

            if (PropertyConstant.TABLE_TYPE_ONE.equals(record.getTableType())) {
                if (null == record.getLinkModuleId() || 0 == record.getLinkModuleId()
                    || StringUtils.isEmpty(record.getPropertyIds())) {
                    return false;
                }
            } else {
                if (CollectionUtils.isEmpty(record.getItemList())) {
                    return false;
                }
            }
        }
        return true;
    }
}
