package com.bocloud.ams.booter.controller.module;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.model.ColumnAndSearchItemModel;
import com.bocloud.ams.entity.model.InstanceTopo;
import com.bocloud.ams.entity.model.PathModel;
import com.bocloud.ams.entity.model.ResponseInstanceTopologyModel;
import com.bocloud.ams.entity.module.TopologyConfig;
import com.bocloud.ams.service.module.TopologyConfigService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;

@Tag(name =  "实例拓扑管理")
@RestController
@RequestMapping("/v1/instance/topology")
public class TopologyConfigController {

    @Autowired
    private TopologyConfigService topologyConfigService;

    /**
     * 拓扑实例列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary ="实例拓扑列表")
    public GeneralResult<GridBean<TopologyConfig>> list(Pager pager,
                                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return topologyConfigService.list(pager);
    }

    /**
     * 新增模型
     *
     * @param config
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC"}, summary ="实例拓扑新增")
    public GeneralResult<Void> save(@RequestBody TopologyConfig config,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return topologyConfigService.save(config, context.getTarget());
    }

    /**
     * 更新实例图谱
     *
     * @param config
     * @param context
     * @return
     */
    @PutMapping
    @Operation(tags = {"CMC"}, summary ="实例拓扑修改")
    public GeneralResult<Void> modify(@RequestBody TopologyConfig config,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return topologyConfigService.modify(config, context.getTarget());
    }

    /**
     * 删除拓扑实例
     *
     * @param id
     * @param context
     * @return
     */

    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary ="拓扑实例删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return topologyConfigService.remove(id, context.getTarget());
    }

    /**
     * 查询实例拓扑详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(summary = "实例拓扑详情")
    public GeneralResult<TopologyConfig> detail(@PathVariable(value = "id") Long id) {
        return topologyConfigService.detail(id);
    }

    /**
     * 根据起始模型查询最大拓扑层级
     *
     * @param categoryId
     * @param startModuleId
     * @return
     */
    @GetMapping(value = "/max/level")
    @Operation(summary = "根据起始模型查询最大拓扑层级")
    public GeneralResult<Integer> maxLevel(@RequestParam("categoryId") Long categoryId,
                                           @RequestParam("startModuleId") Long startModuleId) {
        return topologyConfigService.maxLevel(categoryId, startModuleId);
    }

    /**
     * 获取拓扑路径
     *
     * @param categoryId
     * @param startModuleId
     * @param topologyLevel
     * @param ignoreModuleId
     * @param wayModuleId
     * @return
     */
    @GetMapping(value = "/path")
    @Operation(summary = "获取拓扑路径")
    public GeneralResult<List<PathModel>> topologyPath(@RequestParam("categoryId") Long categoryId,
                                                       @RequestParam("startModuleId") Long startModuleId, @RequestParam("topologyLevel") Integer topologyLevel,
                                                       @RequestParam(value = "ignoreModuleId", required = false) String ignoreModuleId,
                                                       @RequestParam(value = "wayModuleId", required = false) String wayModuleId) {
        return topologyConfigService.topologyPath(categoryId, startModuleId, topologyLevel, ignoreModuleId,
                wayModuleId);
    }

    /**
     * 获取查询项以及列表项
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}/dynamic/item")
    @Operation(summary = "获取查询项以及列表项")
    public GeneralResult<ColumnAndSearchItemModel> searchItemList(@PathVariable(value = "id") Long id) {
        return topologyConfigService.searchItemList(id);
    }

    /**
     * 查询资源列表
     *
     * @param pager
     * @return
     */
    @GetMapping(value = "/resource")
    @Operation(summary = "查询资源列表")
    public GeneralResult<GridBean<Map<String, Object>>> resourcelist(Pager pager) {
        return topologyConfigService.resourcelist(pager);
    }

    /**
     * 根据实例拓扑id查询对应起始模型的实例集合
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}/startmodule/instance")
    @Operation(summary = "根据实例拓扑id查询对应起始模型的实例集合")
    public GeneralResult<List<ResponseInstanceTopologyModel>>
    startModuleInstanceList(@PathVariable(value = "id") Long id) {
        return topologyConfigService.startModuleInstanceList(id);
    }

    /**
     * 实例拓扑-资源列表导出
     *
     * @param id
     * @param response
     * @return
     */
    @GetMapping("/{id}/export")
    @Operation(summary = "实例拓扑资源列表导出")
    public GeneralResult<Void> download(@PathVariable Long id, HttpServletResponse response) {
        return topologyConfigService.download(id, response);
    }

    /**
     * 实例拓扑-全局拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    @GetMapping(value = "/topo")
    @Operation(summary = "全局拓扑")
    public GeneralResult<InstanceTopo> topoList(@RequestParam(value = "id") Long id,
                                                @RequestParam(value = "instanceId") Long instanceId) {
        return topologyConfigService.topoList(id, instanceId);
    }

    /**
     * 实例资源拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    @GetMapping(value = "/topo/instances/{instanceId}")
    @Operation(summary = "实例资源拓扑")
    public GeneralResult<InstanceTopo> topology(@RequestParam(value = "id") Long id,
                                                @PathVariable Long instanceId) {
        return topologyConfigService.topology(id, instanceId);
    }

    /**
     * 实例拓扑-根据实例id查找下一层级实例拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    @GetMapping(value = "/next/topo")
    @Operation(summary = "根据实例id查找下一层级实例拓扑")
    public GeneralResult<InstanceTopo> nextTopoList(@RequestParam(value = "id") Long id,
                                                    @RequestParam(value = "moduleId") Long moduleId, @RequestParam(value = "instanceId") Long instanceId) {
        return topologyConfigService.nextTopoList(id, moduleId, instanceId);
    }
}
