package com.bocloud.ams.booter.scheduler;

import java.util.concurrent.TimeUnit;

import com.megatron.framework.core.CurrentService;
import com.megatron.framework.lock.AutoCloseLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Description: 模型实例关系定时任务
 * @Title: ModuleInstanceRelationScheduler
 * @Author: majiali
 * @CreateTime: 2022/6/9 15:53
 */
@Slf4j
@Component
public class ModuleInstanceRelationScheduler {

    @Autowired
    private InstanceAutoRelationService instanceAutoRelationService;
    @Autowired
    private CurrentService service;
    @Autowired
    private LockFactory lockFactory;

    @Scheduled(cron = "${module.instance.relation.schedule:0 0 2 * * ?}") // 每天凌晨两点执行
    private void autoRelation() {
        if (!service.isLeader()) {
            log.info("当前服务不是主服务，不执行定时任务");
            return;
        }
        log.info("开始定时处理模型实例关系");
        String path = ModuleInstanceRelationScheduler.class.getSimpleName() + "_autoRelation";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            instanceAutoRelationService.autoRelation();
        } catch (Exception e) {
            log.error("Auto relation scheduler fail:", e);
        }
        log.info("结束定时处理模型实例关系");
    }
}
