package com.bocloud.ams.booter.syslog;

import ch.qos.logback.core.net.SyslogOutputStream;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

public class SyslogOutputStreamTcpAndUdp extends SyslogOutputStream {

    private static final int MAX_LEN_TCP = 1024;

    private InetAddress addressTcp;
    private Socket socketTcp;
    private ByteArrayOutputStream baosTcp = new ByteArrayOutputStream();
    final private int portTcp;

    public SyslogOutputStreamTcpAndUdp(String syslogHost, int port) throws SocketException, UnknownHostException {
        super(syslogHost, port);
        this.addressTcp = InetAddress.getByName(syslogHost);
        this.portTcp = port;
        try {
            this.socketTcp = new Socket(addressTcp, portTcp);
        } catch (IOException e) {
            throw new SocketException(e);
        }
    }

    @Override
    public void write(byte[] byteArray, int offset, int len) throws IOException {
        super.write(byteArray, offset, len);
        baosTcp.write(byteArray, offset, len);
    }

    @Override
    public void flush() throws IOException {
        super.flush();

        byte[] bytes = baosTcp.toByteArray();

        // clean up for next round
        if (baosTcp.size() > MAX_LEN_TCP) {
            baosTcp = new ByteArrayOutputStream();
        } else {
            baosTcp.reset();
        }

        // after a failure, it can happen that bytes.length is zero
        // in that case, there is no point in sending out an empty message/
        if (bytes.length == 0) {
            return;
        }
        if (this.socketTcp != null) {
            socketTcp.getOutputStream().write(bytes);
        }
    }

    @Override
    public void close() {
        super.close();
        addressTcp = null;
        try {
            socketTcp.close();
        } catch (IOException ignored) {
        }
    }
}
