package com.bocloud.ams.entity.collection;


import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/10 0010 16:11
 * @Description:
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Table("ci_collection_task_instance")
@EqualsAndHashCode(callSuper = true)
public class CollectionTaskInstance extends GenericEntity {
    @PK(value = PKStrategy.AUTO)
    private Long id;
    @Column("uuid")
    private String uuid;
    /**
     * 任务类型
     */
    @Column("type")
    private String type;
    @Column("code")
    private String code;
    @Column("graph")
    private Object graph;
    @Column("cron")
    private String cron;
    /**
     * 开始时间
     */
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss:SSS")
    private Date gmtStart;
    /**
     * 结束时间
     */
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss:SSS")
    private Date gmtFinish;

    /**
     * 总耗时
     */
    @Column("finish_time")
    private Long finishTime;

    /**
     * 任务ID
     */
    @Column("task_id")
    private Long taskId;
    /**
     * 超时时间
     */
    @Column("timeout")
    private Long timeout;

    /**
     * 耗时
     */
    @Column("cost")
    private Double cost;

    /**
     * 输入参数
     */
    @Column("input")
    private String input;

    /**
     * 输出参数
     */
    @Column("output")
    private String output;

    /**
     * 采集结束时间
     */
    @Column("collection_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date collectionFinish;

    /**
     * 采集总耗时
     */
    @Column("collection_time")
    private Double collectionTime;

    /**
     * 采集状态
     */
    @Column("collection_status")
    private String collectionStatus;

    @IgnoreAll
    private List<Object> successes;
    @IgnoreAll
    private List<Object> failures;
    @IgnoreAll
    private Long moduleId;
    @IgnoreAll
    private String moduleCode;
    @IgnoreAll
    private String creator;
    @IgnoreAll
    private String tableName;

    public CollectionTaskInstance(Long creatorId, Long ownerId, Long menderId, String name, String status, String remark,
                                  String type, String cron, String collectionStatus) {
        this.setName(name);
        this.setStatus(status);
        this.setRemark(remark);
        this.type = type;
        this.cron = cron;
        this.setCreatorId(creatorId);
        this.setOwnerId(ownerId);
        this.setMenderId(menderId);
        this.collectionStatus = collectionStatus;
    }
}
