package com.bocloud.ams.entity.module;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

@Table("ci_property")
@Data
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class Property extends GenericEntity {
    @PK(value = PKStrategy.AUTO)
    private Long id;

    @IgnoreAll
    private Long propertyId;

    /**
     * 所属模型id
     */
    @Column("module_id")
    private Long moduleId;
    /**
     * 属性池ID
     */
    @Column("property_pool_id")
    private Long propertyPoolId;
    /**
     * 唯一编码
     */
    @Column("code")
    private String code;
    /**
     * 类型
     */
    @Column("type")
    private String type;
    /**
     * 所属分组
     */
    @Column("group_name")
    private String groupName;
    /**
     * 最小值
     */
    @Column("min_value")
    private Integer minValue;
    /**
     * 最大值
     */
    @Column("max_value")
    private Integer maxValue;
    /**
     * 最大长度
     */
    @Column("length")
    private Integer length;
    /**
     * 是否必填
     */
    @Column("is_required")
    private Boolean isRequired=false;

    /**
     * 默认值类型
     */
    @Column("default_value_type")
    private String defaultValueType;
    /**
     * 默认值
     */
    @Column("default_value")
    private String defaultValue;
    /**
     * 是否唯一
     */
    @Column("is_uniqued")
    private Boolean isUniqued=false;
    /**
     * 排序数
     */
    @Column("order_num")
    private Integer orderNum;
    /**
     * 小数位数
     */
    @Column("decimal_num")
    private Integer decimalNum;
    /**
     * 单位
     */
    @Column("unit")
    private String unit;
    /**
     * 校验id
     */
    @Column("regexp_id")
    private Long regexpId;
    /**
     * 0:用户定义 1:系统内置
     */
    @Column("is_reserved")
    private Boolean isReserved=false;
    /**
     * 是否自动采集
     */
    @Column("is_auto_discovery")
    private Boolean isAutoDiscovery=false;
    /**
     * 列表项
     */
    @Column("is_title")
    private Boolean isTitle = false;
    /**
     * 只读
     */
    @Column("is_only_reader")
    private Boolean isOnlyReader;
    /**
     * 是否是搜索项
     */
    @Column("is_search")
    private Boolean isSearch=false;
    /**
     * 规定json格式
     */
    @Column("json_schema")
    private String jsonSchema;
    /**
     * 前缀标识
     */
    @Column("prefix_identifier")
    private String prefixIdentifier;
    /**
     * 起始值
     */
    @Column("start_num")
    private Integer startNum;

    /**
     * 是否是主键
     */
    @Column("is_key_property")
    private Boolean isKeyProperty;

    /**
     * 表格类型属性结构体定义类型 0：新建定义  1：从已有模型中引入
     */
    @Column("table_type")
    private String tableType;

    /**
     * 表格类型属性引入已有模型的模型ID
     */
    @Column("link_module_id")
    private Long linkModuleId;

    /**
     * 表格类型属性引入已有模型的属性id，以逗号隔开
     */
    @Column("property_ids")
    private String propertyIds;

    @IgnoreAll
    private List<PropertyItem> itemList;

    @IgnoreAll
    private RegularExpression regularExpression;

    public Integer getTableLength(){
        return  this.length==null?255:this.length;
    }

    public boolean equalTable(Property cmdbProperty) {
        if (!this.getCode().equals(cmdbProperty.getCode())) {
            return false;
        }
        if(!this.getTableLength().equals(cmdbProperty.getTableLength())){
            return  false;
        }
        return true;
    }

    public Property(String code, String name, String type, String groupName, int length, Boolean required,
                        Boolean uniqued, int orderNum, Long moduleId, Boolean reserved, Boolean autoDiscovery, Boolean title,
                        Long creatorId, String defaultValueType, String defaultValue, Boolean isOnlyReader, Boolean isSearch) {
        super();
        this.code = code;
        super.setName(name);
        this.type = type;
        this.groupName = groupName;
        this.length = length;
        //是否必填
        this.isRequired = required;
        //是否唯一
        this.isUniqued = uniqued;
        this.orderNum = orderNum;
        this.moduleId = moduleId;
        //0:用户定义 1:系统内置
        this.isReserved = reserved;
        this.isAutoDiscovery = autoDiscovery;
        //列表项
        this.isTitle = title;
        this.defaultValueType = defaultValueType;
        this.defaultValue = defaultValue;
        //只读
        this.isOnlyReader = isOnlyReader;
        //是否是搜索项
        this.isSearch = isSearch;
        super.setCreatorId(creatorId);
        super.setMenderId(creatorId);
        super.setGmtCreate(new Date());
        super.setGmtModify(new Date());
    }
}
