package com.bocloud.ams.repository.association;

import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/9 0009 17:18
 * @Description:
 */
@Repository
public class AutoRelationRuleItemRepository extends BasicGenericDao<AutoRelationRuleItem, Long> {

    public AutoRelationRuleItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean deleteByRuleId(Long ruleId, Long userId) throws Exception {
        String sql =
            "update ci_auto_relation_rule_item set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where relation_rule_id = :ruleId ";
        Map<String, Object> params = MapTools.simpleMap("ruleId", ruleId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<AutoRelationRuleItem> queryByRuleId(Long ruleId) throws Exception {
        String sql =
            "select b.name source_field_name,b.code source_field_code,b.type source_field_type, c.name target_field_name, c.code target_field_code,c.type target_field_type, a.* "
                + " from ci_auto_relation_rule_item a LEFT JOIN ci_property b ON a.source_field = b.id LEFT JOIN ci_property c ON a.target_field = c.id where a.is_deleted=0 and a.relation_rule_id = :ruleId ";
        return this.list(AutoRelationRuleItem.class, sql, MapTools.simpleMap("ruleId", ruleId));
    }

    public AutoRelationRuleItem query(Long id) throws Exception {
        String sql = "select * from ci_auto_relation_rule_item where id = :id";
        List<AutoRelationRuleItem> list = this.list(AutoRelationRuleItem.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

}
