package com.bocloud.ams.repository.audit;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.audit.OperationalAuditBak;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/23 0023 17:47
 * @Description:
 */
@Repository
public class OperationalAuditBakRepository extends BasicGenericDao<OperationalAuditBak, Long> {

    public OperationalAuditBakRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public void batchSaveOperationalAuditBak(List<OperationalAuditBak> audits, String tableName, Integer batchSize) {
        synchronized (OperationalAuditBak.class) {
            CmdbUtils.changeTableAnnotationValue(OperationalAuditBak.class, tableName);
            this.batchSave(audits, batchSize, false);
        }
    }

    public int countTable(String tableName) throws Exception {
        String sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :tableName;";
        Map<String, Object> params = MapTools.simpleMap("tableName", tableName);
        return this.countQuery(sql, params).intValue();
    }

    public void createTable(String operationalAuditTable, OperationalAuditBak operationalAudit) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + operationalAuditTable + " (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(operationalAudit);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL AUTO_INCREMENT,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        sqlBuilder.append(" " + fieldDBType + " NOT NULL,\n");
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                    }

                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        this.execute(sqlBuilder.toString(), null);
    }


    public int queryCount(String tableName, String leftTime, String rightTime) throws Exception {
        String sql = "select count(1) from " + tableName + " a  where a.is_deleted = 0 and a.gmt_create >= :leftTime and a.gmt_create <= :rightTime ";
        Map<String, Object> params = new HashMap<>();
        params.put("leftTime", leftTime);
        params.put("rightTime", rightTime);
        return this.countQuery(sql, params).intValue();
    }

    public int count(List<Param> params, List<String> finalInstanceTableNames) throws Exception {
        String sql = "";
        if (finalInstanceTableNames.size() == 1) {
            sql = "select count(1) from " + finalInstanceTableNames.get(0) + " b where b.is_deleted = 0";
        } else {
            sql = "select count(1) from (";
            for (int i = 0; i < finalInstanceTableNames.size(); i++) {
                if (i > 0) {
                    sql += " union all ";
                }
                sql += "(select * from " + finalInstanceTableNames.get(i) + " a where a.is_deleted = 0 )";

            }
            sql += ") b where 1=1 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "b");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<OperationalAuditBak> list(Pager pager, List<String> finalInstanceTableNames) throws Exception {

        String sql = "";
        if (finalInstanceTableNames.size() == 1) {
            sql = "select * from " + finalInstanceTableNames.get(0) + " b where b.is_deleted = 0";
        } else {
            sql = "select * from (";
            for (int i = 0; i < finalInstanceTableNames.size(); i++) {
                if (i > 0) {
                    sql += " union all ";
                }
                sql += "(select * from " + finalInstanceTableNames.get(i) + " a where a.is_deleted = 0 )";

            }
            sql += ") b where 1=1 ";
        }

        sql = getQueryBuilder().buildRaw(sql, pager, "b");

        return this.list(OperationalAuditBak.class, sql);
    }

}
