package com.bocloud.ams.repository.collection;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionHistoryBak;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Map;

/**
 * @author huliwei
 * @date 2023/4/25
 */
@Repository("collectionHistoryBakRepository")
public class CollectionHistoryBakRepository extends BasicGenericDao<CollectionHistoryBak, Long> {
    public CollectionHistoryBakRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public int countTable(String tableName) throws Exception {
        String sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :tableName;";
        Map<String, Object> params = MapTools.simpleMap("tableName", tableName);
        return this.countQuery(sql, params).intValue();
    }

    public void createTable(String tableName, CollectionHistoryBak collectionHistoryBak) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + tableName + " (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(collectionHistoryBak);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL ,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        sqlBuilder.append(" " + fieldDBType + " NOT NULL,\n");
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        this.execute(sqlBuilder.toString(), null);
    }
}
