package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/13 0013 16:06
 * @Description:
 */
@Repository
public class CollectionTaskInstanceRepository extends BasicGenericDao<CollectionTaskInstance, Long> {

    public CollectionTaskInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public boolean updateStatus(Long id, String status) throws Exception {
        String sql = "update ci_collection_task_instance set status = :status, gmt_modify = :gmtModify , gmt_finish = :gmtFinish where id = :id and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("gmtFinish", new Date());
        return this.execute(sql, params) > 0;
    }

    public List<CollectionTaskInstance> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        // String sql = "select cti.*, cct.module_id as module_id, cm.code as module_code from
        // ci_collection_task_instance cti left join ci_co        // cm on cct.module_id = cm.id where cti.is_deleted = 0 ";llection_task cct on cti.task_id = cct.id left join ci_module
        String sql = "select cti.id,cti.name,cti.type,cti.status,cti.is_deleted,cti.gmt_create,cti.gmt_modify,cti.creator_id,cti.mender_id,cti.uuid,cti.code,cti.timeout,cti.graph,cti.cron,cti.task_id,cti.gmt_finish,cti.finish_time,cti.input,cti.collection_finish,cti.collection_time,cti.collection_status, cct.module_id as module_id, cm.code as module_code from ci_collection_task_instance cti left join ci_collection_task cct on cti.task_id = cct.id left join ci_module cm on cct.module_id = cm.id  where cti.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "cti");
        return this.list(CollectionTaskInstance.class, sql, getQueryBuilder().getParam(params));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from ci_collection_task_instance a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionTaskInstance> list = this.list(CollectionTaskInstance.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionTaskInstance instance : list) {
            beans.add(new SimpleBean(instance.getId(), instance.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_collection_task_instance a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<CollectionTaskInstance> list(List<Param> params) throws Exception {
        String sql = "select * from ci_collection_task_instance a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.list(CollectionTaskInstance.class, sql, param);
    }

    public CollectionTaskInstance query(Long id) throws Exception {
        String sql = "select * from ci_collection_task_instance where id = :id and is_deleted = 0";
        List<CollectionTaskInstance> list = this.list(CollectionTaskInstance.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public CollectionTaskInstance queryByTaskId(String taskId) throws Exception {
        String sql = "select * from ci_collection_task_instance where uuid = :uuid and is_deleted = 0";
        List<CollectionTaskInstance> list = this.list(CollectionTaskInstance.class, sql, MapTools.simpleMap(Common.UUID, taskId));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(List<Long> ids) {
        String sql = "delete from ci_collection_task_instance where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.execute(sql, params) > 0;
    }

    public List<CollectionTaskInstance> query(String taskInstanceBakName, String leftTime, String rightTime) throws Exception {
        String sql = "select id from " + taskInstanceBakName + " a  where a.is_deleted = 0 and a.gmt_create >= :leftTime and a.gmt_create <= :rightTime ";
        Map<String, Object> params = new HashMap<>();
        params.put("leftTime", leftTime);
        params.put("rightTime", rightTime);
        List<CollectionTaskInstance> list = this.list(CollectionTaskInstance.class, sql, params);
        return list;
    }

    public List<CollectionTaskInstance> list(Pager pager, String taskInstanceBakName) throws Exception {
        String sql = "select cti.id,cti.name,cti.type,cti.status,cti.is_deleted,cti.gmt_create,cti.gmt_modify,cti.creator_id,cti.mender_id,cti.uuid,cti.code,cti.timeout,cti.graph,cti.cron,cti.task_id,cti.gmt_finish,cti.finish_time,cti.input,cti.collection_finish,cti.collection_time,cti.collection_status, cct.module_id as module_id, cm.code as module_code , '" + taskInstanceBakName + "' AS table_name  from " + taskInstanceBakName + " cti left join ci_collection_task cct on cti.task_id = cct.id left join ci_module cm on cct.module_id = cm.id  where cti.is_deleted = 0  ";
        sql = getQueryBuilder().buildRaw(sql, pager, "cti");
        return this.list(CollectionTaskInstance.class, sql);
    }

    public List<CollectionTaskInstance> listIdBak(List<Param> params, Map<String, String> sorter, String taskInstanceBakName) throws Exception {
        String sql = "select cti.id , '" + taskInstanceBakName + "' AS table_name  from " + taskInstanceBakName + " cti  where cti.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, params, sorter, "cti");
        return this.list(CollectionTaskInstance.class, sql);
    }

    public int count(List<Param> params, String taskInstanceBakName) throws Exception {
        String sql = "select count(*) from " + taskInstanceBakName + " a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public CollectionTaskInstance query(Long id, String tableName) throws Exception {
        String sql = "select * , '" + tableName + "' as table_name from " + tableName + " where id = :id and is_deleted = 0";
        List<CollectionTaskInstance> list = this.list(CollectionTaskInstance.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

}
