package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionTask;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author liuyuanyuan
 * @Date 2022/6/8 0008 17:06
 * @Description:
 */
@Repository
public class CollectionTaskRepository extends BasicGenericDao<CollectionTask, Long> {
    public CollectionTaskRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<CollectionTask> list(int page, int rows, List<Param> params, Map<String, String> sorter)
        throws Exception {
        String sql =
            "select cct.*,c.name as template_name, cm.name as module_name from ci_collection_task cct left join ci_collection_template c on cct.template_id = c.id left join ci_module cm on cct.module_id = cm.id where cct.is_deleted = 0 and cm.status = 'ENABLE' ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "cct");
        return this.list(CollectionTask.class, sql, getQueryBuilder().getParam(params));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql =
            "select a.id,a.name from ci_collection_task a left join ci_module b on a.module_id = b.id  where b.status ='ENABLE' and a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionTask> list = this.list(CollectionTask.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionTask task : list) {
            beans.add(new SimpleBean(task.getId(), task.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql =
            "select count(1) from ci_collection_task a left join ci_module b on a.module_id = b.id  where b.status ='ENABLE' and a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public CollectionTask query(Long id) throws Exception {
        String sql = "select * from ci_collection_task where id = :id and is_deleted = 0";
        List<CollectionTask> list = this.list(CollectionTask.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<CollectionTask> queryByTemplateId(Long templateId) throws Exception {
        String sql = "select * from ci_collection_task where template_id = :templateId and is_deleted = 0";
        return this.list(CollectionTask.class, sql, MapTools.simpleMap("templateId", templateId));
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql =
            "update ci_collection_task set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name, String type) throws Exception {
        String sql = "select * from ci_collection_task a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        if (!StringUtils.isEmpty(type)) {
            sql += " and a.type = :type";
            params.put("type", type);
        }
        List<CollectionTask> list = this.list(CollectionTask.class, sql, params);
        return !list.isEmpty();
    }

    public List<CollectionTask> findAllByRunning(String status) {
        String sql = "select a.* from ci_collection_task a where a.status =:status  and a.is_deleted = 0";
        return this.list(CollectionTask.class, sql, MapTools.simpleMap("status", status));
    }

    public Integer changeStatus(Long id, String status, Long menderId) {
        String sql =
            "update ci_collection_task set status =:status , mender_id = :menderId ,gmt_modify = now()  where id = :id  and is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap("id", id);
        param.put("status", status);
        param.put("menderId", menderId);
        return this.execute(sql, param);
    }

    public List<CollectionTask> findScriptTaskByModuleId(Long moduleId) {
        String sql =
            "select c.* from ci_collection_task c,ci_collection_template t where c.template_id =t.id and t.is_deleted =0 and c.is_deleted =0 "
                + " and t.collection_type ='script' and t.module_id = :moduleId ";
        return this.list(CollectionTask.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

}
