package com.bocloud.ams.repository.config;

import com.bocloud.ams.entity.config.CmdbConfig;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * @author majiali
 * @date 2023/4/20
 */
@Repository("cmdbConfigRepository")
public class CmdbConfigRepository extends BasicGenericDao<CmdbConfig, Long> {

    public CmdbConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean modify(CmdbConfig config) throws DataBaseException {
        return this.baseUpdate(config);
    }

    public CmdbConfig query(Long id) throws DataBaseException {
        String sql = "select * from ci_cmdb_config where id = :id";
        List<CmdbConfig> list = this.list(CmdbConfig.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<CmdbConfig> listAll() throws DataBaseException {
        String sql = "select a.* from ci_cmdb_config a";
        return this.list(CmdbConfig.class, sql);
    }

    public CmdbConfig queryByCode(String code) throws DataBaseException {
        String sql = "select * from ci_cmdb_config where code = :code";
        List<CmdbConfig> list = this.list(CmdbConfig.class, sql, MapTools.simpleMap(Common.CODE, code));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean batchUpdate(List<CmdbConfig> configs, String[] fields) throws DataBaseException {
        return this.batchUpdate(configs, fields, configs.size());
    }

    public List<CmdbConfig> listByCodes(Set<String> codes) throws DataBaseException {
        String sql = "select * from ci_cmdb_config where code in (:code)";
        return this.list(CmdbConfig.class, sql, MapTools.simpleMap(Common.CODE, codes));
    }

    public List<CmdbConfig> listByCategory(String category) throws DataBaseException {
        String sql = "select * from ci_cmdb_config where category = (:category)";
        return this.list(CmdbConfig.class, sql, MapTools.simpleMap("category", category));
    }

}
