package com.bocloud.ams.repository.instance;

import com.bocloud.ams.entity.instance.InstanceHistory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * @Author yangzhenrong
 * @Date 2022/5/16 16:48
 * @Version 1.0
 **/
@Repository
public class InstanceHistoryRepository extends BasicGenericDao<InstanceHistory, Long> {

    public InstanceHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InstanceHistory> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_instance_history a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InstanceHistory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_instance_history a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<InstanceHistory> listByInstanceId(Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.* from ci_instance_history a where module_id = :moduleId and instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceHistory.class, sql, params);
    }

    public boolean remove(Long moduleId, Long instanceId) throws Exception {
        String sql = "delete from ci_instance_history where module_id = :moduleId and instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemove(Long moduleId, List<Long> instanceIds) throws Exception {
        String sql = "delete from ci_instance_history where module_id = :moduleId and instance_id in (:instanceIds) ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

}
