package com.bocloud.ams.repository.instance;

import com.bocloud.ams.entity.instance.InstanceRelationHistoryRecord;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * @Author majiali
 * @Date 2022/6/8 15:49
 * @Version 1.0
 **/
@Repository
public class InstanceRelationHistoryRecordRepository extends BasicGenericDao<InstanceRelationHistoryRecord, Long> {

    public InstanceRelationHistoryRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InstanceRelationHistoryRecord> list(Pager pager) throws Exception {
        String sql = "select a.*,b.name module_relation_name,c.name relation_category_name from ci_instance_relation_history_record a " + " left join ci_module_relation b on a.module_relation_id =b.id " + " left join ci_relation_category c on a.relation_category_id =c.id " + " where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InstanceRelationHistoryRecord.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_instance_relation_history_record a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public InstanceRelationHistoryRecord query(Long id) throws Exception {
        String sql = "select a.*,b.name module_name from ci_instance_relation_history_record a left join ci_module b on a.module_id  =b.id where a.id = :id and a.is_deleted=0 ";
        List<InstanceRelationHistoryRecord> list = this.list(InstanceRelationHistoryRecord.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InstanceRelationHistoryRecord> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from ci_instance_relation_history_record where id in ( :ids ) and is_deleted = 0";
        return this.list(InstanceRelationHistoryRecord.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<InstanceRelationHistoryRecord> queryByCondition(Long instanceId, Long moduleId, String status, Long moduleRelationId) throws Exception {
        String sql = "select a.*,b.name module_name from ci_instance_relation_history_record a left join ci_module b on a.module_id  =b.id where a.instance_id = :instanceId and a.module_id = :moduleId and a.status=:status and a.module_relation_id= :moduleRelationId and a.is_deleted = 0 order by a.gmt_create desc";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("moduleId", moduleId);
        params.put("status", status);
        params.put("moduleRelationId", moduleRelationId);
        return this.list(InstanceRelationHistoryRecord.class, sql, params);
    }

}
