package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.ModuleGroup;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/4/29 0029 16:14
 * @Description:
 */
@Repository
public class ModuleGroupRepository extends BasicGenericDao<ModuleGroup, Long> {
    public ModuleGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_module_group set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<ModuleGroup> queryRoot() throws Exception {
        String sql = "select * from ci_module_group where is_deleted =0 and parent_id =0 ORDER BY gmt_create DESC ";
        return this.list(ModuleGroup.class, sql, null);
    }
    public List<ModuleGroup> queryRootInName(List<String> nameList) throws Exception {
        String sql = "select * from ci_module_group where is_deleted =0 and parent_id =0 and name in (:name) ORDER BY gmt_create DESC ";
        Map<String, Object> params = MapTools.simpleMap("name", nameList);
        return this.list(ModuleGroup.class, sql, params);
    }

    public List<ModuleGroup> queryChildren(Long id) throws Exception {
        String sql = "select * from ci_module_group where is_deleted =0 and parent_id = :id ORDER BY gmt_create DESC ";
        return this.list(ModuleGroup.class, sql, MapTools.simpleMap("id", id));
    }

    public List<ModuleGroup> all() throws Exception {
        String sql = "select * from ci_module_group where is_deleted=0 ";
        return this.list(ModuleGroup.class, sql, null);
    }

    public int countSameNameOrCode(String name, String code) throws Exception {
        String sql = "select count(1) from ci_module_group where is_deleted=0 ";
        if (null != code) {
            sql += " and code = :code ";
        } else if (null != name) {
            sql += " and name = :name ";
        }
        Map<String, Object> params = MapTools.simpleMap("code", code);
        params.put("name", name);
        return this.countQuery(sql, params).intValue();
    }

    public ModuleGroup query(Long id) throws Exception {
        String sql = "select * from ci_module_group a where a.id = :id and is_deleted=0 ";
        List<ModuleGroup> list = this.list(ModuleGroup.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ModuleGroup> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from ci_module_group a where a.id in (:ids) and is_deleted=0 ";
        return this.list(ModuleGroup.class, sql, MapTools.simpleMap("ids", ids));
    }

}
