package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.PropertyItem;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/5 12:01
 */
@Repository
public class PropertyItemRepository extends BasicGenericDao<PropertyItem, Long> {

    public PropertyItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean deleteByModuleId(Long moduleId, Long userId) throws Exception {
        String sql = "update ci_property_item set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where module_id = :moduleId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<PropertyItem> queryByModuleId(Long moduleId) {
        String sql = "select a.*  from ci_property_item a  where a.module_id = :moduleId and a.is_deleted = 0";
        return this.list(PropertyItem.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<PropertyItem> queryByPropertyId(Long propertyId) {
        String sql = "select a.*  from ci_property_item a  where a.property_id = :propertyId and a.is_deleted = 0";
        return this.list(PropertyItem.class, sql, MapTools.simpleMap("propertyId", propertyId));
    }

    public List<PropertyItem> queryByModuleIdAndPropertyIds(Long moduleId, List<Long> propertyIds) {
        String sql = "select a.*  from ci_property_item a  where a.module_id = :moduleId and a.property_id in(:propertyIds) and a.is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("propertyIds", propertyIds);
        paramMap.put("moduleId", moduleId);
        return this.list(PropertyItem.class, sql, paramMap);
    }

}
