package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.PropertyPool;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: 属性池Dao
 * @Author: majiali
 * @CreateTime: 2022/5/5 14:03
 */
@Repository
public class PropertyPoolRepository extends BasicGenericDao<PropertyPool, Long> {

    public PropertyPoolRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_property_pool set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<PropertyPool> list(Pager pager) throws Exception {
        String sql = "select a.*,c.expression from ci_property_pool a left join ci_regular_expression c on a.regexp_id=c.id  where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(PropertyPool.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_property_pool a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public PropertyPool query(Long id) throws Exception {
        String sql = "select a.*,c.expression from ci_property_pool a left join ci_regular_expression c on a.regexp_id=c.id where a.id = :id and a.is_deleted=0 ";
        List<PropertyPool> list = this.list(PropertyPool.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public PropertyPool queryByCode(String code) throws Exception {
        String sql = "select a.* from ci_property_pool a where a.code = :code and a.is_deleted=0 ";
        List<PropertyPool> list = this.list(PropertyPool.class, sql, MapTools.simpleMap("code", code));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<PropertyPool> listByPropertyIds(List<Long> propertyIds) throws Exception {
        String sql = "select a.* from ci_property_pool a,ci_property b where a.is_deleted=0 and a.id=b.property_pool_id" + " and b.is_deleted=0 and b.property_pool_id is not null  and b.id in (:propertyIds)";
        return this.list(PropertyPool.class, sql, MapTools.simpleMap("propertyIds", propertyIds));
    }

    public List<PropertyPool> listByPoolIds(List<Long> poolIds) throws Exception {
        String sql = " select a.* from ci_property_pool a , ci_property b where a.is_deleted=0 and a.id=b.property_pool_id " + " and b.is_deleted=0 and b.property_pool_id is not null and a.id in (:poolIds) ";
        return this.list(PropertyPool.class, sql, MapTools.simpleMap("poolIds", poolIds));
    }

}
