package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.Relation;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: 关联关系Dao
 * @Author: majiali
 * @CreateTime: 2022/5/7 14:03
 */
@Repository
public class RelationRepository extends BasicGenericDao<Relation, Long> {

    public RelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_relation set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<Relation> list(Pager pager) throws Exception {
        String sql = "select a.*,b.name category_name from ci_relation a,ci_relation_category b where a.category_id =b.id and a.is_deleted=0 and b.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(Relation.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_relation a,ci_relation_category b where a.category_id =b.id and a.is_deleted=0 and b.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from ci_relation a where is_deleted=0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<Relation> list = this.list(Relation.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (Relation relation : list) {
            beans.add(new SimpleBean(relation.getId(), relation.getName() + "(反向关系名称：" + relation.getReverseName() + ")"));
        }
        return beans;
    }

    public Relation queryByCode(String code, Long categoryId) throws Exception {
        String sql = "select a.* from ci_relation a where a.code = :code and is_deleted=0 and category_id = :categoryId";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        params.put("categoryId", categoryId);
        List<Relation> list = this.list(Relation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public Relation query(Long id) throws Exception {
        String sql = "select a.* from ci_relation a where a.id = :id and is_deleted=0 ";
        List<Relation> list = this.list(Relation.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<Relation> queryByCategoryId(Long categoryId) throws Exception {
        String sql = "select a.* from ci_relation a where a.is_deleted=0 and a.category_id =:categoryId";
        return this.list(Relation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

}
