package com.bocloud.ams.repository.utils;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.bocloud.ams.annotation.ColumnType;
import org.apache.commons.lang3.EnumUtils;
import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionHistoryItem;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.Table;

import lombok.extern.slf4j.Slf4j;

/**
 * @ClassName: CmdbUtils
 * @Description
 * @Author yangzhenrong
 * @Date 2022/5/11 14:56
 * @Version 1.0
 **/
@Slf4j
public class CmdbUtils {

    public static final String CMDB_INSTANCE = "ci_instance_";
    public static final String CMDB_INSTANCE_VALUE = "ci_instance_value_";
    public static final String CMDB_INSTANCE_RELATION = "ci_instance_relation_";

    public static final String TYPE_STRING = "java.lang.String";
    public static final String TYPE_LONG = "java.lang.Long";
    public static final String TYPE_BOOLEAN = "java.lang.Boolean";
    public static final String TYPE_DATE = "java.util.Date";
    public static final String TYPE_FLOAT = "java.lang.Float";
    public static final String TYPE_DOUBLE = "java.lang.Double";
    public static final String TYPE_BIGDECIMAL = "java.math.BigDecimal";
    public static final String TYPE_INTEGER = "java.lang.Integer";
    public static final String INSTANCE_RELATION_TYPE_SOURCE = "SOURCE";
    public static final String INSTANCE_RELATION_TYPE_TARGET = "TARGET";
    public static final String CMDB_OPERATIONAL_AUDIT_BAK = "ci_operational_audit_bak_";

    // 转换数据类型
    public static String judgeDataType(String type) {
        switch (type) {
            case TYPE_STRING:
                return "varchar";
            case TYPE_LONG:
                return "bigint";
            case TYPE_BOOLEAN:
                return "tinyint";
            case TYPE_DATE:
                return "datetime";
            case TYPE_FLOAT:
                return "float";
            case TYPE_DOUBLE:
                return "double";
            case TYPE_BIGDECIMAL:
                return "decimal";
            case TYPE_INTEGER:
                return "int";
            default:
                return "varchar";
        }
    }

    /***
     * 获取对象指定注解Column的相关字段及类型
     * 
     * @param object
     * @return
     */
    public static Map<String, JSONObject> getDeclaredFieldsInfo(Object object) {
        Map<String, JSONObject> map = new LinkedHashMap<>();
        // 父类属性
        Field[] genericFields = object.getClass().getSuperclass().getDeclaredFields();
        // 属性
        Field[] fields = object.getClass().getDeclaredFields();
        for (Field field : fields) {
            boolean annotationColumn = field.isAnnotationPresent(Column.class);
            if (annotationColumn) {
                String annotationVal = field.getAnnotation(Column.class).value();
                String fieldType = field.getType().getName();
                boolean nullable = field.getAnnotation(Column.class).nullable();
                long length = field.getAnnotation(Column.class).length();
                JSONObject fieldJSON = new JSONObject();
                fieldJSON.put("fieldType", fieldType);
                fieldJSON.put("length", length);
                fieldJSON.put("nullable", nullable);
                boolean columnType = field.isAnnotationPresent(ColumnType.class);
                if(columnType){
                    String columnTypeValue = field.getAnnotation(ColumnType.class).value();
                    fieldJSON.put("fieldDBType", columnTypeValue);
                }
                map.put(annotationVal, fieldJSON);
            }
        }
        for (Field field : genericFields) {
            boolean annotationColumn = field.isAnnotationPresent(Column.class);
            if (annotationColumn) {
                String annotationVal = field.getAnnotation(Column.class).value();
                String fieldType = field.getType().getName();
                boolean nullable = field.getAnnotation(Column.class).nullable();
                long length = field.getAnnotation(Column.class).length();
                JSONObject fieldJSON = new JSONObject();
                fieldJSON.put("fieldType", fieldType);
                fieldJSON.put("length", length);
                fieldJSON.put("nullable", nullable);
                boolean columnType = field.isAnnotationPresent(ColumnType.class);
                if(columnType){
                    String columnTypeValue = field.getAnnotation(ColumnType.class).value();
                    fieldJSON.put("fieldDBType", columnTypeValue);
                }
                map.put(annotationVal, fieldJSON);
            }
        }
        return map;
    }
    // public static Map<String, String> getDeclaredFieldsInfo(Object object) {
    // Map<String, String> map = new LinkedHashMap<>();
    // //父类属性
    // Field[] genericFields = object.getClass().getSuperclass().getDeclaredFields();
    // //属性
    // Field[] fields = object.getClass().getDeclaredFields();
    // for (Field field : fields) {
    // boolean annotationColumn = field.isAnnotationPresent(Column.class);
    // if (annotationColumn) {
    // String annotationVal = field.getAnnotation(Column.class).value();
    // String fieldType = field.getType().getName();
    // map.put(annotationVal, fieldType);
    // }
    // }
    // for (Field field : genericFields) {
    // boolean annotationColumn = field.isAnnotationPresent(Column.class);
    // if (annotationColumn) {
    // String annotationVal = field.getAnnotation(Column.class).value();
    // String fieldType = field.getType().getName();
    // map.put(annotationVal, fieldType);
    // }
    // }
    // return map;
    // }

    /***
     * CmdbInstanceValue数据行转列
     * 
     * @param list
     * @return
     */
    public static JSONObject instanceValueRowToCol(List<InstanceValue> list) {
        JSONObject object = new JSONObject();
        if (ListTool.isEmpty(list)) {
            return object;
        }
        Map<String, Object> tempMap = new HashMap<>();
        for (InstanceValue instanceValue : list) {
            if (!StringUtils.isEmpty(instanceValue.getPropertyCode())) {
                tempMap.put(instanceValue.getPropertyCode(), instanceValue.getPropertyValue());
            }
        }
        for (Map.Entry<String, Object> entry : tempMap.entrySet()) {
            object.put(entry.getKey(), entry.getValue());
        }
        return object;
    }

    /***
     * 更改Table注解值
     * 
     * @param clazz
     * @param newTableValue
     */
    public static void changeTableAnnotationValue(Class clazz, String newTableValue) {
        try {
            Table tableAnnotation = (Table)clazz.getAnnotation(Table.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(tableAnnotation);
            // 获取私有 memberValues 属性
            Field memberValuesField = invocationHandler.getClass().getDeclaredField("memberValues");
            memberValuesField.setAccessible(true);
            Map<String, Object> memberValuesValue = (Map<String, Object>)memberValuesField.get(invocationHandler);
            // 修改属性值
            memberValuesValue.put("value", newTableValue);
        } catch (Exception e) {
            log.error("changeTableAnnotationValue fail", e);
        }
    }

    public static Class changeTableAnnotation(Class clazz, String newTableValue) {
        try {
            Table tableAnnotation = (Table)clazz.getAnnotation(Table.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(tableAnnotation);
            // 获取私有 memberValues 属性
            Field memberValuesField = invocationHandler.getClass().getDeclaredField("memberValues");
            memberValuesField.setAccessible(true);
            Map<String, Object> memberValuesValue = (Map<String, Object>)memberValuesField.get(invocationHandler);
            // 修改属性值
            memberValuesValue.put("value", newTableValue);
        } catch (Exception e) {
            log.error("changeTableAnnotation fail", e);
        }
        return clazz;
    }

    /***
     * 判断字符串是否满足正则表达式
     * 
     * @param regex
     * @param source
     * @return
     */
    public static boolean contains(String regex, String source) {
        Pattern pattern = Pattern.compile(regex);
        return pattern.matcher(source).find();
    }

    public static boolean isJSON(String str) {
        boolean result = false;
        try {
            Object obj = JSON.parse(str);
            result = true;
        } catch (Exception e) {
            result = false;
        }
        return result;
    }

    public static boolean isDate(String str) {
        // 日期格式校验
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            sdf.parse(str);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    public static boolean isTime(String str) {
        // 时间格式校验
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        try {
            sdf.parse(str);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    /***
     * 判断是否为布尔型
     * 
     * @param str
     * @return
     */
    public static boolean isBool(String str) {
        try {
            if ("true".equalsIgnoreCase(str) || "0".equals(str) || "false".equalsIgnoreCase(str) || "1".equals(str)) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
    }

    /***
     * 判断是否为数值
     * 
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        try {
            Pattern pattern = Pattern.compile("^(\\-|\\+)?\\d+(\\.\\d+)?$");
            Matcher isNum = pattern.matcher(str);
            if (!isNum.matches()) {
                return false;
            }
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /***
     * 从查询参数内获取指定字段的值
     * 
     * @param params
     * @param field
     * @return
     */
    public static String getField(List<Param> params, String field) {
        if ((null != params) && (!params.isEmpty())) {
            for (Param param : params) {
                for (String key : param.getParam().keySet()) {
                    if (field.equals(key)) {
                        return param.getParam().get(key).toString();
                    }
                }
            }
        }
        return null;
    }

    /***
     * 带下划线字段转驼峰
     * 
     * @param name
     * @return
     */
    public static String toCamelCase(String name) {
        if (null == name) {
            return null;
        }
        if (name.contains("_")) {
            name = name.toLowerCase();
            StringBuilder sb = new StringBuilder(name.length());
            boolean upperCase = false;
            for (int i = 0; i < name.length(); i++) {
                char c = name.charAt(i);
                if (c == '_') {
                    upperCase = true;
                } else if (upperCase) {
                    sb.append(Character.toUpperCase(c));
                    upperCase = false;
                } else {
                    sb.append(c);
                }
            }
            return sb.toString();
        } else {
            return name;
        }
    }

    // 判断某个值是否在枚举类中
    public static boolean isInclude(Class enumClass, String value) {
        List enumList = EnumUtils.getEnumList(enumClass);
        for (int i = 0; i < enumList.size(); i++) {
            Object en = enumList.get(i);
            Class<?> enClass = en.getClass();
            try {
                Method method = enClass.getMethod("getValue"); // 需要与枚举类方法对应
                Object invoke = method.invoke(en, null);
                if (invoke.toString().equalsIgnoreCase(value)) {
                    return true;
                }
            } catch (Exception e) {
                return false;
            }
        }
        return false;
    }

    public static void main(String[] args) throws Exception {
        // System.out.println(CmdbUtils.isNumeric("22.233333333333333333"));
        // System.out.println(CmdbUtils.changeTableAnnotation(CmdbInstanceValue.class,
        // "ci_instance_value_alll_in_one"));

        // Object propertyValueJSON = JSON.parse(String.valueOf("[{2: \"2\"}]"));
        // System.out.println(propertyValueJSON.toString());
        // System.out.println(CmdbUtils.isInclude(CmdbInstanceEnum.authority.class, "ACCESS"));
        //
        // System.out.println(CmdbInstanceEnum.authority.valueOf("ACCESS").getName());
        // List<Map<String, Object>> list = new ArrayList<>();
        // Map<String, Object> map = new HashMap<>();
        // map.put("name", "test");
        // map.put("module_id", 5L);
        // list.add(map);
        // Map<String, Object> map11 = new HashMap<>();
        // map11.put("name", "test6");
        // map11.put("module_id", 6L);
        // list.add(map11);
        //
        // Iterator<Map<String, Object>> iterator = list.iterator();
        // while (iterator.hasNext()) {
        // Map<String, Object> map2 = iterator.next();
        // map2.put("eee","eeee");
        //// Iterator<Map.Entry<String, Object>> it = map2.entrySet().iterator();
        //// Map<String, Object> newMap = new HashMap<>();
        //// while (it.hasNext()) {
        //// Map.Entry<String, Object> entry = it.next();
        //// if ("module_id".equals(entry.getKey())) {
        //// newMap.put(entry.getKey(), map2.get(entry.getKey()));
        //// it.remove();
        //// }
        //// }
        //// for (String key : newMap.keySet()) {
        //// map2.put(CmdbUtils.toCamelCase(key), newMap.get(key));
        ////
        //// }
        // }
        // System.out.println(JSON.toJSONString(list));
        // System.out.println(JSON.toJSONString(list));
        // System.out.println(InstanceHistory.catalog.INSTANCE);

        // String str = "2022-03-03 12:22";
        // System.out.println(Integer.valueOf(null));
        // System.out.println(Integer.valueOf(0));
        // System.out.println(Integer.valueOf("ddddd"));

        // System.out.println(CmdbUtils.isBool("0"));

        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE ci_collection_history_item (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(new CollectionHistoryItem());
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL AUTO_INCREMENT,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        System.out.println(sqlBuilder.toString());

        // Map<String,Object> lmap = new HashMap<String,Object>();
        //
        // List<CmdbModuleInstanceValue> instanceValues=new ArrayList<>();
        //
        // for (int i=0;i<10;i++) {
        // CmdbModuleInstanceValue instanceValue = new CmdbModuleInstanceValue();
        // instanceValue.setPropertyCode("code" + i);
        // instanceValue.setPropertyValue("value" + i);
        // instanceValues.add(instanceValue);
        // }
        // for (CmdbModuleInstanceValue instanceValue:instanceValues){
        // String code=instanceValue.getPropertyCode();
        // String value=instanceValue.getPropertyValue();
        // lmap.put(code,value);
        // }
        // JSONObject object = new JSONObject();
        // object.put("uuid", "uuid");
        // object.put("instanceId", "instanceId");
        // for (Map.Entry<String, Object> entry : lmap.entrySet()) {
        // object.put(entry.getKey(),entry.getValue());
        // }
        // System.out.println(object.toJSONString());

        // try {
        // changeTableAnnotationValue(CmdbInstance.class, "essssss");
        // Class<CmdbInstance> cmdbModuleInstanceClass = CmdbInstance.class;
        // Table tableAnnotation = cmdbModuleInstanceClass.getAnnotation(Table.class);
        //// InvocationHandler invocationHandler = Proxy.getInvocationHandler(tableAnnotation);
        //// // 获取私有 memberValues 属性
        //// Field memberValuesField = invocationHandler.getClass().getDeclaredField("memberValues");
        //// memberValuesField.setAccessible(true);
        //// Map<String, Object> memberValuesValue = (Map<String, Object>) memberValuesField.get(invocationHandler);
        //// // 修改属性值
        //// memberValuesValue.put("value", "123");
        //
        // System.out.println(tableAnnotation.value());
        //
        //
        // } catch (Exception e) {
        // }

    }
}
