package com.bocloud.ams.service.audit.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.TableLog;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.audit.OperationalAuditBak;
import com.bocloud.ams.entity.enums.OperationalAuditType;
import com.bocloud.ams.repository.TableLogRepository;
import com.bocloud.ams.repository.audit.OperationalAuditBakRepository;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.DateTools;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/23 0023 17:46
 * @Description:
 */
@Slf4j
@Service("operationalAuditService")
public class OperationalAuditServiceImpl implements OperationalAuditService {

    @Autowired
    private OperationalAuditRepository operationalAuditRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private OperationalAuditBakRepository operationalAuditBakRepository;
    @Autowired
    private TableLogRepository tableLogRepository;

    @Override
    public GeneralResult<GridBean<OperationalAudit>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            int total = operationalAuditRepository.count(params);
            List<OperationalAudit> audits = operationalAuditRepository.list(pager);
            for (OperationalAudit audit : audits) {
                audit.setOperator(redisTemplate.opsForValue().get("User_Name_" + audit.getCreatorId()));
            }
            GridBean<OperationalAudit> bean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, audits);
            return new GeneralResult<GridBean<OperationalAudit>>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("query cmdb operational audit list fail", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<OperationalAudit> deatil(Long id) {
        try {
            OperationalAudit audit = operationalAuditRepository.query(id);
            if (null == audit) {
                return new GeneralResult<>(false, "记录不存在");
            }
            audit.setOperator(redisTemplate.opsForValue().get("User_Name_" + audit.getCreatorId()));
            return new GeneralResult<>(true, audit, "查询成功");
        } catch (Exception e) {
            log.error("query cmdb operational audit deatil fail", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 格式化实例中的日期和时间
     *
     * @param instanceMap
     */
    private void formatDate(Map<String, Object> instanceMap) {
        instanceMap.forEach((key, value) -> {
            if (value instanceof Date) {
                instanceMap.put(key, DateTools.formatTime2String((Date) value, "yyyy-MM-dd"));
            }
            if (value instanceof Timestamp) {
                instanceMap.put(key, DateTools.timestamp2DateTime(((Timestamp) value).getTime()));
            }
            if ((key.equals("gmtCreate") || key.equals("gmtModify")) && (value instanceof Long)) {
                Instant instant = Instant.ofEpochMilli(Long.valueOf(value.toString()));
                LocalDateTime time = LocalDateTime.ofInstant(instant, ZoneId.systemDefault());
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                formatter.format(time);
                instanceMap.put(key, time);
            }
        });
        if (null != instanceMap.get("creatorId")) {
            Long creatorId = Long.valueOf(instanceMap.get("creatorId").toString());
            instanceMap.put("creatorId", redisTemplate.opsForValue().get("User_Name_" + creatorId));
        }
        if (null != instanceMap.get("menderId")) {
            Long menderId = Long.valueOf(instanceMap.get("menderId").toString());
            instanceMap.put("menderId", redisTemplate.opsForValue().get("User_Name_" + menderId));
        }
    }

    @Override
    public OperationalAudit buildLog(OperationalAuditType type, String moduleName, String instanceName,
                                     String moduleCode, Long id, Long userId, Map<String, Object> beforeMap, Map<String, Object> afterMap) {
        try {
            OperationalAudit audit = new OperationalAudit();
            audit.setTypeCode(type.getType());
            audit.setType(type.getTypeDesc());
            audit.setName(String.format("%s-%s", moduleName, instanceName));
            audit.setModuleCode(moduleCode);
            audit.setInstanceId(id);
            if (beforeMap != null) {
                formatDate(beforeMap);
                audit.setOaBefore(JSONObject.toJSONString(beforeMap));
            }
            if (afterMap != null) {
                formatDate(afterMap);
                audit.setOaAfter(JSONObject.toJSONString(afterMap));
            }
            audit.setCreatorId(userId);
            audit.setOperator(redisTemplate.opsForValue().get("User_" + userId));
            return audit;
        } catch (Exception e) {
            log.error("审计操作记录保存失败：", e);
            return null;
        }
    }

    @Override
    public GeneralResult<GridBean<OperationalAuditBak>> bakList(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            String startTime = null;
            String endTime = null;
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("gmtCreate")) {
                    String time = paramMap.get("gmtCreate").toString();
                    if (Sign.GET.name().equals(param.getSign().name())) {
                        startTime = time;
                    } else if (Sign.LET.name().equals(param.getSign().name())) {
                        endTime = time;
                    }
                }
            }
            if (StringUtils.isEmpty(startTime) || StringUtils.isEmpty(endTime)) {
                return new GeneralResult<>(false, "日期必传！");
            }
            List<OperationalAuditBak> list = new ArrayList<>();
            //根据模板表查询出所有日志表 然后 使用并行流查询包含参数日期的库表
            List<TableLog> tableLogs = tableLogRepository.query(CmdbUtils.CMDB_OPERATIONAL_AUDIT_BAK);
            String finalTimeGet = startTime;
            String finalTimeLet = endTime;
            //获取到数据中包含所查询的日期的历史备份表
            List<String> tableList = tableLogs.parallelStream().map(item -> item.getName()).filter(item -> {
                int count = 0;
                try {
                    count = operationalAuditBakRepository.queryCount(item, finalTimeGet, finalTimeLet);
                } catch (Exception e) {
                    log.error("查询操作审计备份表数量异常",e);
                }
                return count > 0;
            }).collect(Collectors.toList());
            int total = 0;
            if(CollectionUtils.isNotEmpty(tableList)){
                total = operationalAuditBakRepository.count(pager.getParams(), tableList);
                list = operationalAuditBakRepository.list(pager, tableList);
                for (OperationalAuditBak audit : list) {
                    audit.setOperator(redisTemplate.opsForValue().get("User_Name_" + audit.getCreatorId()));
                }
            }
            GridBean<OperationalAuditBak> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询备份数据记录成功");
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询备份数据记录失败");
        }
    }

}
