package com.bocloud.ams.service.collection.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionHistory;
import com.bocloud.ams.entity.collection.CollectionHistoryItem;
import com.bocloud.ams.entity.collection.CollectionTask;
import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.entity.enums.CollectionEnum;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.CollectionHistoryModel;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.repository.collection.CollectionHistoryItemRepository;
import com.bocloud.ams.repository.collection.CollectionHistoryRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceRepository;
import com.bocloud.ams.repository.collection.CollectionTaskRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.collection.CollectionHistoryService;
import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.bocloud.ams.service.instance.InstanceService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.JSONTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("collectionHistoryService")
public class CollectionHistoryServiceImpl implements CollectionHistoryService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CollectionHistoryRepository historyRepository;
    @Autowired
    private CollectionHistoryItemRepository historyItemRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private InstanceService instanceService;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private CollectionTaskInstanceRepository taskInstanceRepository;
    @Autowired
    private InstanceAutoRelationService instanceAutoRelationService;
    @Autowired
    private CollectionTaskRepository collectionTaskRepository;
    @Autowired
    private PropertyRepository propertyRepository;

    @Override
    public GeneralResult<GridBean<Map<String, Object>>> instances(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Map<String, Object>> list = new ArrayList<>();
            int total = 0;
            // 模型
            Long moduleId = null;
            String moduleCode = "";
            String historyType = null;
            Long taskInstanceId = null;
            // 实例属性条件查询
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("moduleCode")) {
                    if (!StringUtils.isEmpty(paramMap.get("moduleCode").toString())) {
                        moduleCode = paramMap.get("moduleCode").toString();
                    }
                    paramMap.remove("moduleCode");
                }
                if (paramMap.containsKey("historyType")) {
                    if (!StringUtils.isEmpty(paramMap.get("historyType").toString())) {
                        historyType = paramMap.get("historyType").toString();
                    }
                    paramMap.remove("historyType");
                }
                if (paramMap.containsKey("taskInstanceId")) {
                    if (!StringUtils.isEmpty(paramMap.get("taskInstanceId").toString())) {
                        taskInstanceId = Long.valueOf(paramMap.get("taskInstanceId").toString());
                    }
                    paramMap.remove("taskInstanceId");
                }
                if (paramMap.containsKey("moduleId")) {
                    if (!StringUtils.isEmpty(paramMap.get("moduleId").toString())) {
                        moduleId = Long.valueOf(paramMap.get("moduleId").toString());
                    }
                }
            }
            // 拼接查询条件
            if (null != historyType) {
                List<CollectionHistory> historyList = historyRepository.queryByIdAndType(taskInstanceId, historyType);
                List<Long> historyIds =
                    historyList.stream().map(history -> history.getInstanceId()).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(historyIds)) {
                    HashMap<String, Object> inMap = Maps.newHashMap();
                    inMap.put("id", generate(historyIds));
                    Param inParam = new Param();
                    inParam.setParam(inMap);
                    inParam.setSign(Sign.IN);
                    pager.getParams().add(inParam);
                    String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
                    String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
                    total = instanceRepository.count(pager.getParams(), table, valueTable, userId, true);
                    list = this.instanceRepository.list(pager, table, valueTable, userId, true);
                    instanceService.convertInstanceValue(list, moduleId, userId, true);
                }
            }
            GridBean<Map<String, Object>> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("instance and value list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private Object generate(List<Long> hostIds) {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < hostIds.size(); i++) {
            builder.append(hostIds.get(i)).append(",");
        }
        builder.append(" -1 ");
        return builder.toString();
    }

    @Override
    public GeneralResult<GridBean<CollectionHistory>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.historyRepository.count(pager.getParams());
            List<CollectionHistory> list = this.historyRepository.list(pager);
            GridBean<CollectionHistory> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Collection history list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> create(CollectionHistory history) {
        try {
            this.historyRepository.save(history);
            List<CollectionHistoryItem> historyItems = history.getHistoryItems();
            if (CollectionUtils.isNotEmpty(historyItems)) {
                for (CollectionHistoryItem item : historyItems) {
                    item.setHistoryId(history.getId());
                    historyItemRepository.save(item);
                }
            }
            return new GeneralResult<>(true, "新增成功");
        } catch (Exception e) {
            log.error("Collection history create error:", e);
            return new GeneralResult<>(false, "新增失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(CollectionHistory history) {
        final String path = CollectionHistory.class.getSimpleName() + "_" + history.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionHistory temp = historyRepository.query(history.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "记录不存在");
            }
            LocaleBeanUtils.copyProperties(temp, history);
            historyRepository.update(temp);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Collection history modify error:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<List<CollectionHistory>> listByTaskInstanceId(Long taskInstanceId) {
        try {
            List<CollectionHistory> histories = historyRepository.listByTaskInstanceId(taskInstanceId);
            for (CollectionHistory history : histories) {
                List<CollectionHistoryItem> items = historyItemRepository.listByHistoryId(history.getId());
                history.setHistoryItems(items);
            }
            return new GeneralResult<>(true, histories, "查询成功");
        } catch (Exception e) {
            log.error("Collection history list error:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<CollectionHistory> detail(Long id) {
        try {
            CollectionHistory history = historyRepository.query(id);
            List<CollectionHistoryItem> items = historyItemRepository.listByHistoryId(id);
            history.setHistoryItems(items);
            return new GeneralResult<>(true, history, "查询成功");
        } catch (Exception e) {
            log.error("Collection history detail error:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<String> batchModify(CollectionHistoryModel historyModel, Long userId) {
        if (null == historyModel.getIds() || historyModel.getIds().isEmpty()) {
            return new GeneralResult<>(true, "批量操作失败");
        }
        Long taskInstanceId = null;
        try {
            for (Long id : historyModel.getIds()) {
                String path = CollectionHistory.class.getSimpleName() + "_" + id;
                try (AutoCloseLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                    CollectionHistory bean = historyRepository.query(id);
                    if (null == bean) {
                        return new GeneralResult<>(false, "数据不存在");
                    }
                    taskInstanceId = bean.getTaskInstanceId();
                    List<CollectionHistoryItem> items = historyItemRepository.listByHistoryId(id);
                    // 取消则标记成功
                    if (CollectionEnum.historyOperate.CANCEL.name().equalsIgnoreCase(historyModel.getAction())) {
                        bean.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                    } else if (CollectionEnum.historyOperate.UPDATE.name().equalsIgnoreCase(historyModel.getAction())
                        && CollectionEnum.historyStatus.RECONCILED.name().equalsIgnoreCase(bean.getStatus())) {
                        // 手工调和
                        JSONObject instanceObject = new JSONObject();
                        instanceObject.put("moduleId", bean.getModuleId());
                        CiModule module = moduleRepository.query(bean.getModuleId());
                        if (null != module) {
                            instanceObject.put("moduleCode", module.getCode());
                        }
                        List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(bean.getModuleId());
                        Map<String, Property> propertyMap = new HashMap<>();
                        if (CollectionUtils.isNotEmpty(properties)) {
                            for (Property property : properties) {
                                propertyMap.put(property.getCode(), property);
                            }
                        }
                        Map<String, Object> dataMap = new HashMap<>();
                        for (CollectionHistoryItem item : items) {
                            if (propertyMap.containsKey(item.getPropertyCode())) {
                                Property property = propertyMap.get(item.getPropertyCode());
                                if ((PropertyConstant.PropertyType.CHECKBOX.equals(property.getType())
                                    || PropertyConstant.PropertyType.MULTISELECT.equals(property.getType())
                                    || PropertyConstant.PropertyType.JSON.equals(property.getType())
                                    || PropertyConstant.PropertyType.SINGLELINETABLE.equals(property.getType())
                                    || PropertyConstant.PropertyType.MULTILINETABLE.equals(property.getType()))
                                    && StringUtils.isNotEmpty(item.getAfterValue())) {
                                    GeneralResult result = JSONTools.isJSON(item.getAfterValue());
                                    if (result.isSuccess()) {
                                        instanceObject.put(item.getPropertyCode(),
                                            JSONObject.parseObject(item.getAfterValue()));
                                    } else {
                                        instanceObject.put(item.getPropertyCode(),
                                            JSONArray.parseArray(item.getAfterValue()));
                                    }
                                } else {
                                    instanceObject.put(item.getPropertyCode(), item.getAfterValue());
                                }
                                dataMap.put(item.getPropertyCode(), item.getAfterValue());
                            }
                        }
                        GeneralResult result = new GeneralResult<>(false, "");
                        if (CollectionEnum.historyType.ADD.name().equalsIgnoreCase(bean.getType())) {
                            // 如果名称为空则自动生产
                            if (null == instanceObject.getString("name")) {
                                instanceObject.put("name", "AutomaticGeneration" + System.currentTimeMillis());
                            }
                            result = instanceService.create(instanceObject, userId, false);
                            if (result.isSuccess()) {
                                Instance instance = (Instance)result.getData();
                                bean.setInstanceId(instance.getId());
                                bean.setInstanceUuid(instance.getUuid());
                                bean.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                                // 自动关联
                                CollectionTaskInstance taskInstance =
                                    taskInstanceRepository.query(bean.getTaskInstanceId());
                                CollectionTask task = collectionTaskRepository.query(taskInstance.getTaskId());
                                if (task.getAutoRelation()) {
                                    try {
                                        instanceAutoRelationService.instanceAutoRelation(instance, userId);
                                    } catch (Exception e) {
                                        log.error("采集任务中存在实例关系自动关联失败情况", e);
                                    }
                                }
                            } else {
                                bean.setStatus(CollectionEnum.historyStatus.FAILED.name());
                                bean.setReason(result.getMessage());
                            }
                        } else if (CollectionEnum.historyType.EDIT.name().equalsIgnoreCase(bean.getType())) {
                            String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
                            List<InstanceValue> instanceValues =
                                instanceValueRepository.listByInstanceId(bean.getInstanceId(), instanceValueTable);
                            for (InstanceValue instanceValue : instanceValues) {
                                if (!dataMap.containsKey(instanceValue.getPropertyCode())) {
                                    instanceObject.put(instanceValue.getPropertyCode(),
                                        instanceValue.getPropertyValue());
                                }
                            }
                            instanceObject.put(Common.ID, bean.getInstanceId());
                            instanceObject.put(Common.UUID, bean.getInstanceUuid());
                            result = instanceService.modify(instanceObject, userId);
                            if (result.isSuccess()) {
                                bean.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                                // 自动关联
                                CollectionTaskInstance taskInstance =
                                    taskInstanceRepository.query(bean.getTaskInstanceId());
                                CollectionTask task = collectionTaskRepository.query(taskInstance.getTaskId());
                                if (task.getAutoRelation()) {
                                    try {
                                        Instance instance = (Instance)result.getData();
                                        instanceAutoRelationService.instanceAutoRelation(instance, userId);
                                    } catch (Exception e) {
                                        log.error("采集任务中存在实例关系自动关联失败情况", e);
                                    }
                                }
                            } else {
                                bean.setStatus(CollectionEnum.historyStatus.FAILED.name());
                                bean.setReason(result.getMessage());
                            }
                        }
                    }
                    historyRepository.update(bean);
                    if (checkInstanceHistory(bean.getTaskInstanceId(), false)) {
                        if (CollectionEnum.historyStatus.SUCCESS.name().equals(bean.getStatus())
                            && checkInstanceHistory(bean.getTaskInstanceId(), true)) {
                            taskInstanceRepository.updateStatus(bean.getTaskInstanceId(),
                                CollectionEnum.Status.SUCCESS.name());
                        } else if (CollectionEnum.historyStatus.FAILED.name().equals(bean.getStatus())) {
                            taskInstanceRepository.updateStatus(bean.getTaskInstanceId(),
                                CollectionEnum.Status.ERROR.name());
                        }
                    }
                } catch (Exception e) {
                    log.error("Collection history batch modify error:", e);
                    // 复查任务状态
                    CollectionTaskInstance taskInstance = taskInstanceRepository.query(taskInstanceId);
                    return new GeneralResult<>(false, taskInstance.getStatus(), "批量修改失败");
                }
            }
            // 复查任务状态
            CollectionTaskInstance taskInstance = taskInstanceRepository.query(taskInstanceId);
            return new GeneralResult<>(true, taskInstance.getStatus(), "修改成功");
        } catch (Exception e) {
            log.error("Collection history batch modify error:", e);
            try {
                CollectionTaskInstance taskInstance = taskInstanceRepository.query(taskInstanceId);
                return new GeneralResult<>(false, taskInstance.getStatus(), "批量修改失败");
            } catch (Exception ex) {
                log.error("query task inatance is error:", e);
            }
            return new GeneralResult<>(false, "批量修改失败");
        }
    }

    @Override
    public GeneralResult<GridBean<CollectionHistory>> backupHistory(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            String tableName = null;
            for (Param param : params) {
                Map<String, Object> map = param.getParam();
                if(null != map && map.containsKey("tableName")){
                    tableName = map.get("tableName").toString();
                    map.remove("tableName");
                }
            }
            if(StringUtils.isEmpty(tableName)){
                return new GeneralResult<>(false, "备份表参数必传");
            }
            int total = this.historyRepository.count(tableName,pager.getParams());
            List<CollectionHistory> list = this.historyRepository.list(tableName,pager);
            int i = tableName.lastIndexOf("_");
            String substring = tableName.substring(i+1);
            for (CollectionHistory collectionHistory : list) {
                collectionHistory.setTableName("ci_collection_history_item_bak_" + substring);
            }
            GridBean<CollectionHistory> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Collection history list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<Map<String, Object>>> backupList(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            String tableName = null;
            for (Param param : params) {
                Map<String, Object> map = param.getParam();
                if(null != map && map.containsKey("tableName")){
                    tableName = map.get("tableName").toString();
                    map.remove("tableName");
                }
            }
            if(StringUtils.isEmpty(tableName)){
                return new GeneralResult<>(false, "备份表参数必传");
            }
            List<Map<String, Object>> list = new ArrayList<>();
            int total = 0;
            // 模型
            Long moduleId = null;
            String moduleCode = "";
            String historyType = null;
            Long taskInstanceId = null;
            // 实例属性条件查询
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("moduleCode")) {
                    if (!StringUtils.isEmpty(paramMap.get("moduleCode").toString())) {
                        moduleCode = paramMap.get("moduleCode").toString();
                    }
                    paramMap.remove("moduleCode");
                }
                if (paramMap.containsKey("historyType")) {
                    if (!StringUtils.isEmpty(paramMap.get("historyType").toString())) {
                        historyType = paramMap.get("historyType").toString();
                    }
                    paramMap.remove("historyType");
                }
                if (paramMap.containsKey("taskInstanceId")) {
                    if (!StringUtils.isEmpty(paramMap.get("taskInstanceId").toString())) {
                        taskInstanceId = Long.valueOf(paramMap.get("taskInstanceId").toString());
                    }
                    paramMap.remove("taskInstanceId");
                }
                if (paramMap.containsKey("moduleId")) {
                    if (!StringUtils.isEmpty(paramMap.get("moduleId").toString())) {
                        moduleId = Long.valueOf(paramMap.get("moduleId").toString());
                    }
                }
            }
            // 拼接查询条件
            if (null != historyType) {
                List<CollectionHistory> historyList = historyRepository.queryByIdAndType(tableName,taskInstanceId, historyType);
                List<Long> historyIds = historyList.stream().map(history -> history.getInstanceId()).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(historyIds)) {
                    HashMap<String, Object> inMap = Maps.newHashMap();
                    inMap.put("id", generate(historyIds));
                    Param inParam = new Param();
                    inParam.setParam(inMap);
                    inParam.setSign(Sign.IN);
                    pager.getParams().add(inParam);
                    String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
                    String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
                    total = instanceRepository.count(pager.getParams(), table, valueTable, userId, true);
                    list = this.instanceRepository.list(pager, table, valueTable, userId, true);
                    int i = tableName.lastIndexOf("_");
                    String substring = tableName.substring(i+1);
                    for (Map<String, Object> map : list) {
                        map.put("tableName","ci_collection_history_item_bak_"+substring);
                    }
                    instanceService.convertInstanceValue(list, moduleId, userId, true);
                }
            }
            GridBean<Map<String, Object>> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("instance and value list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private boolean checkInstanceHistory(Long taskInstanceId, boolean needAllSuccess) {
        try {
            List<CollectionHistory> histories = historyRepository.listByTaskInstanceId(taskInstanceId);
            for (CollectionHistory history : histories) {
                if (needAllSuccess) {
                    if (!CollectionEnum.historyStatus.SUCCESS.name().equals(history.getStatus())) {
                        return false;
                    }
                } else {
                    if (CollectionEnum.historyStatus.RECONCILED.name().equals(history.getStatus())) {
                        return false;
                    }
                }
            }
            return true;
        } catch (Exception e) {
            return false;
        }
    }

}
