package com.bocloud.ams.service.collection.impl;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.recipes.leader.LeaderLatchListener;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronExpression;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionComponent;
import com.bocloud.ams.entity.collection.CollectionTask;
import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.entity.collection.CollectionTemplate;
import com.bocloud.ams.entity.enums.ApplyStatus;
import com.bocloud.ams.entity.enums.CollectionEnum;
import com.bocloud.ams.entity.enums.Status;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.OperateParam;
import com.bocloud.ams.entity.model.TaskGraph;
import com.bocloud.ams.entity.model.TaskInstanceBean;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.repository.collection.CollectionComponentRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceRepository;
import com.bocloud.ams.repository.collection.CollectionTaskRepository;
import com.bocloud.ams.repository.collection.CollectionTemplateRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.collection.CollectionTaskInstanceService;
import com.bocloud.ams.service.collection.CollectionTaskService;
import com.bocloud.ams.service.internal.AmsInternalService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.bocloud.ams.task.executor.TaskJobSubmit;
import com.bocloud.ams.task.model.Job;
import com.bocloud.ams.task.model.enums.TaskParam;
import com.google.common.collect.Lists;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.IDFactory;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/8 0008 17:12
 * @Description:
 */
@Slf4j
@Service("collectionTaskService")
public class CollectionTaskServiceImpl implements CollectionTaskService {

    private static final String FIXED = "fixed";
    private static final String SKIPED = "skiped";
    private static final Map<Long, ScheduledFuture<?>> SCHEDULED_FUTURES = new ConcurrentHashMap<>();
    private ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CollectionTaskRepository collectionTaskRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private CollectionTemplateRepository collectionTemplateRepository;
    @Autowired
    private CollectionTaskInstanceRepository collectionTaskInstanceRepository;
    @Autowired
    private TaskJobSubmit taskJobSubmit;
    @Autowired
    private CollectionTaskInstanceService collectionTaskInstanceService;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private AmsInternalService amsInternalService;
    @Autowired
    private CollectionComponentRepository collectionComponentRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private PropertyRepository propertyRepository;

    @Override
    public GeneralResult<GridBean<CollectionTask>> list(Pager pager) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.collectionTaskRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.collectionTaskRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<CollectionTask> list = this.collectionTaskRepository.list(pager.getPage(), pager.getRows(),
                    pager.getParams(), pager.getSorter());
                list.forEach(
                    task -> task.setCreator(redisTemplate.opsForValue().get("User_Name_" + task.getCreatorId())));
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<CollectionTask>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Collection Task list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private int[] getRange(String field, int min, int max) {
        int[] result = new int[2];
        if (!field.contains("-")) {
            result[0] = result[1] = Integer.valueOf(field);
        } else {
            String[] split = org.springframework.util.StringUtils.delimitedListToStringArray(field, "-");
            if (split.length > 2) {
                throw new IllegalArgumentException("Range has more than two fields: '" + field + "' in expression");
            }
            result[0] = Integer.valueOf(split[0]);
            result[1] = Integer.valueOf(split[1]);
        }
        if (result[0] >= max || result[1] >= max) {
            throw new IllegalArgumentException("Range exceeds maximum (" + max + "): '" + field + "' in expression");
        }
        if (result[0] < min || result[1] < min) {
            throw new IllegalArgumentException("Range less than minimum (" + min + "): '" + field + "' in expression");
        }
        if (result[0] > result[1]) {
            throw new IllegalArgumentException("Invalid inverted range: '" + field + "' in expression");
        }
        return result;
    }

    private void buildGraph(CollectionTask task, CollectionTaskInstance instance) throws Exception {
        JSONObject graph = new JSONObject();
        // 获取采集插件信息
        CollectionTemplate template = collectionTemplateRepository.query(task.getTemplateId());
        TaskGraph templateGraph = JSONObject.parseObject(String.valueOf(template.getGraph()), TaskGraph.class);
        TaskGraph taskGraph = JSONObject.parseObject(String.valueOf(task.getGraph()), TaskGraph.class);
        instance.setTimeout(templateGraph.getTimeout());
        JSONObject fronts = new JSONObject();
        if (templateGraph.getCollectionType().equalsIgnoreCase(CollectionEnum.collectionType.SCRIPT.getValue())) {
            instance.setCode(CollectionEnum.workerCode.SCRIPT.getValue());
            fronts.put("content", templateGraph.getContent());
            fronts.put("category", templateGraph.getCategory());
            if (null != taskGraph.getParams()) {
                fronts.put("params", JSONArray.parseArray(taskGraph.getParams()));
            }
            // 检查引用cmdb实例的目标参数
            JSONObject target = JSONObject.parseObject(taskGraph.getTarget());
            List<CollectionComponent> components = collectionComponentRepository.queryByTemplateId(template.getId(),
                CollectionEnum.parameter.INPUT_TARGET.getValue(),
                CollectionEnum.parameterType.CMDB_INSTANCE.getValue());
            Boolean systemIsConfigure = false;
            if (!components.isEmpty()) {
                Map<Long, CollectionComponent> componentMap = components.stream()
                    .collect(Collectors.toMap(CollectionComponent::getPropertyId, Function.identity()));
                // 查询实例数据
                String moduleCode = components.get(0).getModuleCode();
                String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
                List<JSONObject> data = target.getJSONArray("data").toJavaList(JSONObject.class);
                for (JSONObject targetData : data) {
                    List<InstanceValue> values =
                        instanceValueRepository.listByInstanceId(targetData.getLong("id"), valueTable);
                    Map<String, InstanceValue> valueMap =
                        values.stream().collect(Collectors.toMap(InstanceValue::getPropertyCode, Function.identity()));
                    for (Long propertyId : componentMap.keySet()) {
                        CollectionComponent component = componentMap.get(propertyId);
                        switch (component.getName()) {
                            case "AGENTS":
                                targetData.put("host", valueMap.get(component.getPropertyCode()).getPropertyValue());
                                break;
                            case "PORT":
                                targetData.put("port", valueMap.get(component.getPropertyCode()).getPropertyValue());
                                break;
                            case "USER":
                                targetData.put("user", valueMap.get(component.getPropertyCode()).getPropertyValue());
                                break;
                            case "PASSWORD":
                                String pasd = valueMap.get(component.getPropertyCode()).getPropertyValue();
                                // 处理未加密的密码
                                if (CollectionEnum.parameterType.CMDB_INSTANCE.getValue()
                                    .equalsIgnoreCase(component.getParameterType())) {
                                    Property property = propertyRepository.query(component.getPropertyId());
                                    if (!PropertyConstant.PropertyType.TEXTENCRYPT
                                        .equalsIgnoreCase(property.getType())) {
                                        AESEncryptor aesEncryptor = new AESEncryptor();
                                        pasd = aesEncryptor.encrypt(pasd, null);
                                    }
                                }

                                targetData.put("pasd", pasd);
                                break;
                            case "SYSTEM":
                                String system = valueMap.get(component.getPropertyCode()).getPropertyValue();
                                targetData.put("category",
                                    system.toLowerCase().contains("windows") ? "windows" : "linux");
                                systemIsConfigure = true;
                                break;
                            default:
                                break;
                        }
                    }
                }
                target.put("data", data);
            }
            if (!systemIsConfigure) {
                List<JSONObject> data = target.getJSONArray("data").toJavaList(JSONObject.class);
                for (JSONObject targetData : data) {
                    String system = targetData.getString("category");
                    targetData.put("category", system.toLowerCase().contains("windows") ? "windows" : "linux");
                }
                target.put("data", data);
            }
            graph.put("target", target);
            // 构建input
            templateGraph.setParams(taskGraph.getParams());
            JSONObject input = new JSONObject();
            input.put("fronts", templateGraph);
            input.put("target", target);
            instance.setInput(input.toJSONString());
        } else if (templateGraph.getCollectionType()
            .equalsIgnoreCase(CollectionEnum.collectionType.DATABASE.getValue())) {
            instance.setCode(CollectionEnum.workerCode.DATABASE.getValue());
            fronts.put("content", templateGraph.getContent());
            fronts.put("jdbcUrl", templateGraph.getJdbcUrl());
            fronts.put("username", templateGraph.getUsername());
            fronts.put("category", templateGraph.getCategory());
            fronts.put("database", templateGraph.getDatabase());
            fronts.put("password", templateGraph.getPassword());
            // 构建input
            JSONObject input = new JSONObject();
            input.put("fronts", JSONObject.parseObject(String.valueOf(template.getGraph())));
            instance.setInput(input.toJSONString());
        } else if (templateGraph.getCollectionType()
            .equalsIgnoreCase(CollectionEnum.collectionType.METHOD.getValue())) {
            instance.setCode(CollectionEnum.workerCode.METHOD.getValue());
            fronts.put("method", templateGraph.getMethod());
            fronts.put("format", templateGraph.getFormat());
            fronts.put("url", templateGraph.getUrl());
            fronts.put("params", JSONObject.parseObject(templateGraph.getParams()));
            fronts.put("headers", JSONObject.parseObject(templateGraph.getHeaders()));
            fronts.put("body", templateGraph.getBody());
            // 构建input
            JSONObject input = new JSONObject();
            input.put("fronts", JSONObject.parseObject(String.valueOf(template.getGraph())));
            instance.setInput(input.toJSONString());
        }
        graph.put("fronts", fronts);
        graph.put("timeout", templateGraph.getTimeout());
        instance.setGraph(JSONObject.toJSONString(graph));
    }

    @Override
    public GeneralResult<Long> start(Long id, RequestContext context) {
        try {
            CollectionTask task = this.basic(id);
            if (null == task) {
                return new GeneralResult<>(false, "任务不存在");
            }
            CollectionTaskInstance instance = this.initInstance(task, context);
            if (null == instance) {
                return new GeneralResult<>(false, "执行失败：原因可能是作业参数格式错误");
            }
            Job job = buildJob(instance);
            boolean result = taskJobSubmit.subJob(job);
            TaskInstanceBean instanceBean = new TaskInstanceBean();
            instanceBean.setId(instance.getId());
            if (result) {
                instanceBean.setStatus(Status.Inst.RUNNING.name());
                instanceBean.setCollectionStatus(Status.Inst.RUNNING.name());
                collectionTaskInstanceService.modify(instanceBean, context.getTarget());
                return new GeneralResult<>(true, instance.getId(), "采集任务[" + instance.getName() + "]启动成功");
            } else {
                instanceBean.setStatus(Status.Inst.FAILED.name());
                instanceBean.setCollectionStatus(Status.Inst.FAILED.name());
                collectionTaskInstanceService.modify(instanceBean, context.getTarget());
                return new GeneralResult<>(false, "启动失败");
            }
        } catch (Exception e) {
            log.info("启动失败", e);
            return new GeneralResult<>(false, "启动失败");
        }
    }

    private Job buildJob(CollectionTaskInstance task) {
        Job job = new Job();
        job.setPriority(1);
        job.setNeedFeedback(true);
        job.setReplaceOnExist(true);
        job.setParam(Common.NAME, task.getName());
        job.setParam(TaskParam.TASKTYPE.name(), "maintain");
        job.setParam(TaskParam.TASKDATA.name(), String.valueOf(task.getGraph()));
        job.setParam(TaskParam.TASKCODE.name(), task.getCode());
        job.setParam(TaskParam.TIMEOUT.name(), String.valueOf(task.getTimeout()));
        job.setMaxRetryTimes(0);
        job.setTaskId(task.getUuid());
        return job;
    }

    @Override
    public CollectionTaskInstance initInstance(CollectionTask task, RequestContext context) {
        try {
            // 创建任务实例
            Long userId = context.getTarget();
            CollectionTaskInstance instance = new CollectionTaskInstance(userId, userId, userId,
                task.getName() + System.currentTimeMillis(), Status.Inst.CREATED.name(), task.getRemark(),
                task.getType(), task.getCron(), Status.Inst.CREATED.name());
            // 解析graph
            buildGraph(task, instance);
            instance.setUuid(IDFactory.instance().uuid());
            instance.setTaskId(task.getId());
            instance.setProps(JSONObject.toJSONString(context));
            instance.setGmtStart(new Date());
            this.collectionTaskInstanceRepository.save(instance);
            return instance;
        } catch (Exception e) {
            log.error("initInstance [{}] exception !", task.getId(), e);
            return null;
        }
    }

    @Override
    public CollectionTask basic(Long id) {
        try {
            return this.collectionTaskRepository.query(id);
        } catch (Exception e) {
            log.error("Query task fail:", e);
            return null;
        }
    }

    /**
     * 校验cron表达式是否合法
     *
     * @param expression
     * @return
     */
    @Override
    public GeneralResult<CollectionTask> isValid(String expression) {
        String message = "保存任务失败：Cron表达式有误或者执行时间是过去的时间";
        try {
            String[] fields = org.springframework.util.StringUtils.tokenizeToStringArray(expression, " ");
            List<Integer> years = Lists.newArrayList();
            if (null == fields || fields.length < 6 || fields.length > 7) {
                return new GeneralResult<>(false, message);
            } else if (fields.length == 7) {
                if (fields[6].contains("/")) {
                    return new GeneralResult<>(false, message);
                }
                String[] values = org.springframework.util.StringUtils.delimitedListToStringArray(fields[6], ",");
                for (String value : values) {
                    int[] range = getRange(value, 0, 10000);
                    for (int i : range) {
                        years.add(i);
                    }
                }
                int currentYear = Calendar.getInstance().get(Calendar.YEAR);
                boolean flag = false;
                for (Integer year : years) {
                    if (year >= currentYear) {
                        flag = true;
                    }
                }
                if (!flag) {
                    return new GeneralResult<>(false, message);
                }
                expression =
                    fields[0] + " " + fields[1] + " " + fields[2] + " " + fields[3] + " " + fields[4] + " " + fields[5];
            }
            CronExpression generator = CronExpression.parse(expression);
            LocalDateTime time = generator.next(LocalDateTime.now());
            if (null == time) {
                return new GeneralResult<>(false, message);
            }
            if (fields.length == 7) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(Date.from(time.atZone(ZoneId.systemDefault()).toInstant()));
                int execYear = calendar.get(Calendar.YEAR);
                for (Integer year : years) {
                    if (execYear > year) {
                        return new GeneralResult<>(false, message);
                    }
                }
            }
            return new GeneralResult<>(true, "success");
        } catch (Exception e) {
            log.warn("Cron expression is invalid");
            return new GeneralResult<>(false, message);
        }
    }

    private GeneralResult<CollectionTask> saveTask(CollectionTask task, Long userId, boolean isAdd) {
        JSONObject graph = JSONObject.parseObject(JSONObject.toJSONString(task.getGraph()));
        try {
            if (StringUtils.isEmpty(task.getType())) {
                task.setType("standard");
            }
            // 校验cron表达式
            if (task.getType().equalsIgnoreCase(CollectionEnum.Task.SCHEDULED.getValue())) {
                GeneralResult<CollectionTask> result = isValid(task.getCron());
                if (result.isFailed()) {
                    return result;
                }
                task.setStatus(ApplyStatus.CREATED.name());
            }
            // 查询模板信息
            CollectionTemplate template = collectionTemplateRepository.query(task.getTemplateId());
            if (null == template) {
                return new GeneralResult<>(false, "采集插件不存在");
            }
            TaskGraph templateGraph = JSONObject.parseObject(String.valueOf(template.getGraph()), TaskGraph.class);
            if (CollectionEnum.collectionType.SCRIPT.getValue().equalsIgnoreCase(templateGraph.getCollectionType())) {
                JSONObject targets = graph.getJSONObject("target");
                List<JSONObject> target = targets.getJSONArray("data").toJavaList(JSONObject.class);
                boolean exists = false;
                if (null != target && target.size() > 0) {
                    exists = true;
                }
                if (!exists) {
                    return new GeneralResult<>(false, "采集作业【" + task.getName() + "】没有关联目标主机");
                }
                // // 密码加密
                // if (isAdd){
                // for (JSONObject object : target) {
                // AESEncryptor AESEncryptor = new AESEncryptor();
                // object.put("pasd", AESEncryptor.encrypt(object.getString("pasd"), null));
                // }
                // }
                graph.put("target", targets);
            }
            // isAdd为true新增任务
            task.setGraph(JSONObject.toJSONString(graph));
            if (isAdd) {
                task.setId(null);
                task.setCreatorId(userId);
                task.setUuid(IDFactory.instance().uuid());
                task.setGmtCreate(new Date());
                task.setStatus(ApplyStatus.DEVELOPING.name());
                this.collectionTaskRepository.save(task);
            } else {
                // 更新任务
                task.setGmtModify(new Date());
                task.setMenderId(userId);
                this.collectionTaskRepository.update(task);
            }
        } catch (Exception e) {
            log.error("Save task fail:", e);
            return new GeneralResult<>(false, "保存任务失败");
        }
        return new GeneralResult<>(true, task, "保存任务成功");
    }

    @Override
    public GeneralResult<CollectionTask> save(CollectionTask task, RequestContext context) {
        try {
            // 判断是定时任务还是常规任务
            if (task.getType().equals(CollectionEnum.Task.STANDARD.getValue())) {
                // 判断某种类的任务名称是否存在
                boolean flag = collectionTaskRepository.exists(task.getName(), task.getType());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
                // 检查任务中各节点是否正确 并入库
                GeneralResult<CollectionTask> result = saveTask(task, context.getTarget(), true);
                if (result.isSuccess()) {
                    return new GeneralResult<>(true, result.getData(), "添加任务成功");
                }
                return result;
            } else {
                CollectionTask queryEntity = collectionTaskRepository.query(task.getId());
                if (null == queryEntity) {
                    return GeneralResult.FAILED("数据不存在!");
                }
                if (CollectionEnum.Task.SCHEDULED.getValue().equals(queryEntity.getType())) {
                    return GeneralResult.FAILED("此定时任务已存在!");
                }
                if (StringUtils.isBlank(task.getCron())) {
                    return GeneralResult.FAILED("定时规则不能为空");
                }
                if (isValid(task.getCron()).isFailed()) {
                    return GeneralResult.FAILED("定时规则错误");
                }
                queryEntity.setType(CollectionEnum.Task.SCHEDULED.getValue());
                queryEntity.setStatus(CollectionEnum.Status.SUSPENDED.name());
                queryEntity.setCron(task.getCron());
                queryEntity.setMenderId(context.getTarget());
                collectionTaskRepository.update(queryEntity);
                // 是否启动自动采集
                return new GeneralResult<>(true, queryEntity, "新增成功");
            }
        } catch (Exception e) {
            log.error("query task error:", e);
            return new GeneralResult<>(false, "添加任务失败");
        }
    }

    @Override
    public GeneralResult<CollectionTask> modify(CollectionTask bean, RequestContext context) {
        String path = CollectionTask.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTask task = collectionTaskRepository.query(bean.getId());
            Assert.notNull(task, "数据不存在！");
            // 判断是定时任务还是常规任务
            if (bean.getType().equals(CollectionEnum.Task.STANDARD.getValue())) {
                if (!task.getName().equals(bean.getName())) {
                    // 判断某种类的任务名称是否存在
                    boolean flag = collectionTaskRepository.exists(bean.getName(), bean.getType());
                    if (flag) {
                        return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                    }
                }
                if (task.getStatus().equals(Status.Inst.WAITTING.name())) {
                    return new GeneralResult<>(false, "修改失败：任务正在执行");
                }
                BeanUtils.copyProperties(bean, task);
                GeneralResult<CollectionTask> result = saveTask(task, context.getTarget(), false);
                if (result.isSuccess()) {
                    return new GeneralResult<>(true, result.getData(), "修改任务成功");
                } else {
                    return result;
                }
            } else {
                if (CollectionEnum.Status.RUNNING.name().equals(task.getStatus())) {
                    return GeneralResult.FAILED("该任务正在运行，请先暂停该任务");
                }
                if (StringUtils.isBlank(bean.getCron())) {
                    return GeneralResult.FAILED("定时规则不能为空");
                }
                if (isValid(bean.getCron()).isFailed()) {
                    return GeneralResult.FAILED("定时规则错误");
                }
                task.setCron(bean.getCron());
                task.setMenderId(context.getTarget());
                collectionTaskRepository.update(task);
                // 是否启动自动采集
                return new GeneralResult<>(true, task, "修改成功");
            }
        } catch (Exception e) {
            log.error("Update task fail:", e);
            return new GeneralResult<>(false, "修改任务失败");
        }
    }

    @Override
    public GeneralResult<CollectionTask> modifyType(Long id, RequestContext context) {
        String path = CollectionTask.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTask task = collectionTaskRepository.query(id);
            if (null == task) {
                return GeneralResult.FAILED("数据不存在!");
            }
            // 判断是定时任务还是常规任务
            if (task.getType().equals(CollectionEnum.Task.STANDARD.getValue())) {
                return new GeneralResult<>(false, "当前任务不是定时任务,任务修改失败！");
            }
            if (!task.getStatus().equals(CollectionEnum.Status.SUSPENDED.name())) {
                return GeneralResult.FAILED("当前任务状态为非暂停状态，任务修改失败!");
            }
            task.setType(CollectionEnum.Task.STANDARD.getValue());
            task.setStatus(ApplyStatus.DEVELOPING.name());
            task.setCron(null);
            task.setMenderId(context.getTarget());
            collectionTaskRepository.update(task);
            // 是否启动自动采集
            return new GeneralResult<>(true, task, "任务修改成功！");
        } catch (Exception e) {
            log.error("modify task error:", e);
            return new GeneralResult<>(false, "任务修改失败！");
        }
    }

    @Override
    public GeneralResult<CollectionTask> detail(Long id) {
        try {
            CollectionTask task = this.collectionTaskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            task.setCreator(redisTemplate.opsForValue().get("User_Name_" + task.getCreatorId()));
            if (null != task.getMenderId()) {
                task.setMender(redisTemplate.opsForValue().get("User_Name_" + task.getMenderId()));
            }
            return new GeneralResult<>(true, task, "查询任务详细成功");
        } catch (Exception e) {
            log.error("Query task detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> checkCommand(CollectionTask task, Long userId) {
        try {
            // CollectionTemplate template = collectionTemplateRepository.query(task.getTemplateId());
            // if (template.getCollectionType().equalsIgnoreCase("script")) {
            // GeneralResult<Void> result = commandHandler(userId, template.getGraph());
            // if (result.isFailed()) {
            // return result;
            // }
            // }
            return new GeneralResult<>(true, "不存在高危命令");
        } catch (Exception e) {
            log.error("check Command is fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> operate(OperateParam operateParam, Long id, Long target) {
        String path = CollectionTask.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTask task = collectionTaskRepository.query(id);
            if (null == task) {
                return new GeneralResult<>(false, "任务不存在");
            }
            if ("run".equals(operateParam.getAction())) {
                return this.run(id, target);
            } else if ("stop".equals(operateParam.getAction())) {
                return this.stop(id, target);
            }
            return new GeneralResult<>(false, "不支持的操作类型");
        } catch (Exception e) {
            return GeneralResult.FAILED("修改失败");
        }
    }

    private GeneralResult<Void> run(Long id, Long userId) {
        boolean success = amsInternalService.scheduleInspectBroadcast(id, HttpMethod.GET);
        if (success) {
            collectionTaskRepository.changeStatus(id, CollectionEnum.Status.RUNNING.name(), userId);
        }
        return new GeneralResult<>(success, success ? "启动定时任务成功" : "启动定时任务失败");
    }

    private GeneralResult<Void> stop(Long id, Long userId) {
        boolean success = amsInternalService.scheduleInspectBroadcast(id, HttpMethod.DELETE);
        if (success) {
            collectionTaskRepository.changeStatus(id, CollectionEnum.Status.SUSPENDED.name(), userId);
        }
        return new GeneralResult<>(success, success ? "停止定时任务成功" : "停止定时任务失败");
    }

    @Override
    public GeneralResult<String> removeScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]删除定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            CollectionTask timing = collectionTaskRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.removeSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("RemoveScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public GeneralResult<String> addScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]添加定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            CollectionTask timing = collectionTaskRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.addSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("AddScheduleIfLeader fail:", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    private void addSchedule(CollectionTask task) {
        if (SCHEDULED_FUTURES.containsKey(task.getId())) {
            this.removeSchedule(task);
        }
        Trigger trigger = new CronTrigger(task.getCron());
        ScheduledFuture<?> future = taskScheduler.schedule(() -> {
            RequestContext context = new RequestContext();
            context.setTarget(task.getMenderId());
            this.start(task.getId(), context);
        }, trigger);
        SCHEDULED_FUTURES.put(task.getId(), future);
        log.info("定时任务[{}]已经添加到定时任务，调度策略：{}", task.getName(), task.getCron());
        log.info("定时任务：" + JSONObject.toJSONString(SCHEDULED_FUTURES.keySet()));
    }

    private void removeSchedule(CollectionTask collectionTask) {
        ScheduledFuture<?> future = SCHEDULED_FUTURES.remove(collectionTask.getId());
        if (future != null) {
            future.cancel(true);
        }
        log.info("定时任务[{}]已经从定时任务中移除", collectionTask.getName());
    }

    @Override
    public void onApplicationReadyEvent() {
        taskScheduler.setPoolSize(Runtime.getRuntime().availableProcessors());
        taskScheduler.setThreadNamePrefix("TimeInspect-");
        taskScheduler.initialize();
        CollectionTaskServiceImpl collectionTaskServiceImpl = this;
        LeaderLatchListener listener = new LeaderLatchListener() {
            @Override
            public void isLeader() {
                List<CollectionTask> tasks = collectionTaskRepository.findAllByRunning(Status.Inst.RUNNING.name());
                tasks.forEach(collectionTaskServiceImpl::addSchedule);
            }

            @Override
            public void notLeader() {
                List<CollectionTask> tasks = collectionTaskRepository.findAllByRunning(Status.Inst.RUNNING.name());
                tasks.forEach(collectionTaskServiceImpl::removeSchedule);
            }
        };
        try {
            currentService.addListener(listener);
        } catch (Exception e) {
            log.error("定时任务初始化失败：{}", e.getMessage());
        }
        if (currentService.isLeader()) {
            listener.isLeader();
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long target) {
        String path = CollectionTask.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTask task = this.collectionTaskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            Assert.isTrue(!task.getStatus().equals(Status.Inst.WAITTING.name()), "任务正在执行");
            Assert.isTrue(!task.getStatus().equals(Status.Inst.RUNNING.name()), "定时任务正在运行中,请先暂停");
            boolean result = this.collectionTaskRepository.delete(id, target);
            return result ? new GeneralResult<>(true, "删除任务成功") : new GeneralResult<>(false, "删除任务失败");
        } catch (Exception e) {
            log.error("Remove task fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

}
