package com.bocloud.ams.service.instance.impl;

import com.bocloud.ams.entity.enums.InstanceEnum;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceAuthority;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.repository.instance.InstanceAuthorityRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceAuthorityService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service("instanceAuthorityService")
public class InstanceAuthorityServiceImpl implements InstanceAuthorityService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceAuthorityRepository instanceAuthorityRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.instanceAuthorityRepository.count(pager.getParams());
            List<InstanceAuthority> list = this.instanceAuthorityRepository.list(pager);
            for (InstanceAuthority authority : list) {
                String roleName = redisTemplate.opsForValue().get("Role_Name_" + authority.getRoleId());
                if (StringUtils.isEmpty(authority.getAuths())) {
                    continue;
                }
                String[] auths = authority.getAuths().split(",");
                StringBuilder authNames = new StringBuilder();
                for (String auth : auths) {
                    authNames.append(InstanceEnum.authority.valueOf(auth).getName() + ",");
                }
                authority.setUserName(redisTemplate.opsForValue().get("User_Name_" + authority.getUserId()));
                authority.setRoleName(roleName);
                authority.setAuthNames(authNames.substring(0, authNames.length() - 1));
            }
            GridBean<InstanceAuthority> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority list fail:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> create(InstanceAuthority authority, Long userId) {
        try {
            if ( null == authority.getInstanceIds()
                    || authority.getInstanceIds().isEmpty()) {
                return new GeneralResult(false, "新增失败:关键属性[实例]为空");
            }

            if (StringUtils.isEmpty(authority.getAuths())) {
                return new GeneralResult(false, "新增失败:关键属性[权限]为空");
            }

            if (StringUtils.isEmpty(authority.getUserIds())
                    || null == authority.getRoleId()) {
                return new GeneralResult(false, "新增失败:关键属性[用户]为空");
            }
            List<String> authList = new ArrayList<>(Arrays.asList(authority.getAuths().split(",")));
            Long[] convert = (Long[]) ConvertUtils.convert(authority.getUserIds().split(","), Long.class);
            List<Long> userIdList = new ArrayList<>(Arrays.asList(convert));
            Long moduleId = authority.getModuleId();
            Long authRoleId = authority.getRoleId();
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return new GeneralResult(false, "新增失败:模型数据不存在");
            }
            String instanceTable = CmdbUtils.CMDB_INSTANCE + module.getCode().toLowerCase();
            List<InstanceAuthority> addList = new ArrayList<>();
            List<InstanceAuthority> updateList = new ArrayList<>();

            Map<String, String> authMap = new HashMap<>();
            for (String auth : authList) {
                if (!CmdbUtils.isInclude(InstanceEnum.authority.class, auth)) {
                    return new GeneralResult(false, "新增失败:实例权限(" + auth + ")不存在");
                }
                authMap.put(auth, auth);
            }
            //查询实例已有权限
            List<InstanceAuthority> authorities =
                    instanceAuthorityRepository.listByInstanceId(moduleId, authority.getInstanceIds(), userIdList);
            Map<Long, List<InstanceAuthority>> instanceAuthMap = authorities.stream().collect(Collectors.groupingBy(InstanceAuthority::getInstanceId, Collectors.toList()));
            for (Long instanceId : authority.getInstanceIds()) {
                Instance instance = instanceRepository.query(instanceTable, instanceId);
                if (null == instance) {
                    return new GeneralResult(false, "新增失败:实例(" + instanceId + ")数据不存在");
                }
                List<InstanceAuthority> instanceUserAuthList = instanceAuthMap.get(instanceId);
                Map<Long, InstanceAuthority> userAuthMap = new HashMap<>();
                if (CollectionUtils.isNotEmpty(instanceUserAuthList)) {
                    userAuthMap = instanceUserAuthList.stream().collect(Collectors.toMap(InstanceAuthority::getUserId, Function.identity(), (oldData, newData) -> newData));
                }
                for (Long authUserId : userIdList) {
                    InstanceAuthority userAuthority = userAuthMap.get(authUserId);
                    if (null == userAuthority) {
                        InstanceAuthority instanceAuthority = new InstanceAuthority(moduleId, instanceId, authRoleId,
                                authUserId, authority.getAuths());
                        instanceAuthority.setGmtCreate(new Date());
                        instanceAuthority.setCreatorId(userId);
                        addList.add(instanceAuthority);
                    } else {
                        userAuthority.setAuths(authority.getAuths());
                        userAuthority.setRoleId(authRoleId);
                        userAuthority.setMenderId(userId);
                        userAuthority.setGmtModify(new Date());
                        updateList.add(userAuthority);
                    }
                }
            }
            if (addList.size() > 0) {
                this.instanceAuthorityRepository.batchSave(addList, 100);
            }
            if (updateList.size() > 0) {
                this.instanceAuthorityRepository.batchUpdate(updateList, 100);
            }
            return new GeneralResult(true, "新增成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority create error:", e);
            return new GeneralResult(false, "新增失败");
        }

    }

    @Override
    public GeneralResult<Void> modify(InstanceAuthority authority, Long userId) {
        String path = InstanceAuthority.class.getSimpleName() + "_" + authority.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if(StringUtils.isEmpty(authority.getAuths())){
                return new GeneralResult(false, "编辑失败:关键属性[权限]不能为空");
            }
            InstanceAuthority bean = instanceAuthorityRepository.query(authority.getId());
            if (null == bean) {
                return new GeneralResult(false, "编辑失败:权限关系数据不存在");
            }
            CiModule module = moduleRepository.query(authority.getModuleId());
            if (null == module) {
                return new GeneralResult(false, "编辑失败:模型数据不存在");
            }
            bean.setAuths(authority.getAuths());
            bean.setRoleId(authority.getRoleId());
            bean.setGmtModify(new Date());
            bean.setMenderId(userId);
            instanceAuthorityRepository.update(bean);
            return new GeneralResult(true, "编辑成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority modify fail:", e);
            return new GeneralResult(false, "编辑成功");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = InstanceAuthority.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            InstanceAuthority bean = instanceAuthorityRepository.query(id);
            if (null == bean) {
                return new GeneralResult(false, "删除失败:实例权限关系数据不存在");
            }
            instanceAuthorityRepository.remove(id, userId);
            return new GeneralResult(true, "删除成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority remove error:", e);
            return new GeneralResult(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<Void> removeByRole(InstanceAuthority authority, Long userId) {
        try {
            if (null == authority.getRoleId() || null == authority.getModuleId() || null == authority.getInstanceIds()
                    || authority.getInstanceIds().isEmpty()) {
                return new GeneralResult(false, "删除失败:未选择实例或角色为空");
            }
            instanceAuthorityRepository.batchRemove(authority.getModuleId(), authority.getInstanceIds(), authority.getRoleId(),
                    userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority removeByRole error:", e);
            return new GeneralResult(false, "删除失败");
        }
    }

    @Override
    public GeneralResult listByInstanceId(Long moduleId, Long instanceId) {
        try {
            List<InstanceAuthority> authorities =
                    this.instanceAuthorityRepository.listByInstanceId(moduleId, instanceId);
            for (InstanceAuthority authority : authorities) {
                String roleName = redisTemplate.opsForValue().get("Role_Name_" + authority.getRoleId());
                if (StringUtils.isEmpty(authority.getAuths()) || StringUtils.isEmpty(authority.getUserIds())) {
                    continue;
                }
                String[] auths = authority.getAuths().split(",");
                StringBuilder authNames = new StringBuilder();
                for (String auth : auths) {
                    authNames.append(InstanceEnum.authority.valueOf(auth).getName() + ",");
                }
                authority.setUserName(redisTemplate.opsForValue().get("User_Name_" + authority.getUserId()));
                authority.setRoleName(roleName);
                authority.setAuthNames(authNames.substring(0, authNames.length() - 1));
            }
            return new GeneralResult(true, authorities, "查询成功");
        } catch (Exception e) {
            log.error("Cmdb instance authority list error:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

}
