package com.bocloud.ams.service.instance.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.ams.entity.enums.InstanceEnum;
import com.bocloud.ams.entity.instance.InstanceHistory;
import com.bocloud.ams.repository.instance.InstanceHistoryRepository;
import com.bocloud.ams.service.instance.InstanceHistoryService;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("instanceHistoryService")
public class InstanceHistoryServiceImpl implements InstanceHistoryService {

    @Autowired
    private InstanceHistoryRepository instanceHistoryRepository;

    @Override
    public GeneralResult list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.instanceHistoryRepository.count(pager.getParams());
            List<InstanceHistory> list = this.instanceHistoryRepository.list(pager);
            for (InstanceHistory history : list) {
                if (InstanceEnum.catalog.INSTANCE.name().equals(history.getCatalog())) {
                    history.setCatalog(InstanceEnum.catalog.INSTANCE.getValue());
                } else if (InstanceEnum.catalog.RELATION.name().equals(history.getCatalog())) {
                    history.setCatalog(InstanceEnum.catalog.RELATION.getValue());
                }

                if (InstanceEnum.category.CREATE.name().equals(history.getCategory())) {
                    history.setCategory(InstanceEnum.category.CREATE.getValue());
                } else if (InstanceEnum.category.MODIFY.name().equals(history.getCategory())) {
                    history.setCategory(InstanceEnum.category.MODIFY.getValue());
                } else if (InstanceEnum.category.DEL.name().equals(history.getCategory())) {
                    history.setCategory(InstanceEnum.category.DEL.getValue());
                } else if (InstanceEnum.category.BATCH_DELETE.name().equals(history.getCategory())) {
                    history.setCategory(InstanceEnum.category.BATCH_DELETE.getValue());
                } else if (InstanceEnum.category.BATCH_MODIFY.name().equals(history.getCategory())) {
                    history.setCategory(InstanceEnum.category.BATCH_MODIFY.getValue());
                }
            }
            GridBean<InstanceHistory> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Cmdb instance history list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> create(InstanceHistory history, Long userId) {
        try {
            this.instanceHistoryRepository.save(history);
            return new GeneralResult<>(true, "新增成功");
        } catch (Exception e) {
            log.error("Cmdb instance history create error:", e);
            return new GeneralResult<>(false, "新增失败");
        }
    }

    @Override
    public GeneralResult listByInstanceId(Long moduleId, Long instanceId) {
        try {
            List<InstanceHistory> histories = instanceHistoryRepository.listByInstanceId(moduleId, instanceId);
            return new GeneralResult<>(true, histories, "查询成功");
        } catch (Exception e) {
            log.error("Cmdb instance history list error:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

}
