package com.bocloud.ams.service.instance.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.enums.OperationalAuditType;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceRelationHistoryRecord;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.InstanceRelationHistoryAlterModel;
import com.bocloud.ams.entity.model.InstanceRelationHistoryPreflightModel;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.repository.association.AutoRelationRuleRepository;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.instance.InstanceRelationHistoryRecordRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.bocloud.ams.service.instance.InstanceRelationHistoryRecordService;
import com.bocloud.ams.service.utils.ModuleConstant;
import com.bocloud.ams.service.utils.RelationHistoryConstant;
import com.bocloud.ams.service.utils.RepositoryHandler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 实例自动关联接口实现
 * @Author majiali
 * @CreateTime 2022/6/8 9:10
 */
@Slf4j
@Service("instanceRelationHistoryRecordService")
public class InstanceRelationHistoryRecordServiceImpl implements InstanceRelationHistoryRecordService {

    @Autowired
    private InstanceRelationHistoryRecordRepository instanceRelationHistoryRecordRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private RepositoryHandler handler;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private AutoRelationRuleRepository autoRelationRuleRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private OperationalAuditService operationalAuditService;
    @Autowired
    private OperationalAuditRepository operationalAuditRepository;

    @Override
    public GeneralResult detail(Long id) throws Exception {
        InstanceRelationHistoryRecord record = instanceRelationHistoryRecordRepository.query(id);
        if (null != record) {
            return new GeneralResult<>(false, "查询失败，数据不存在");
        }
        return new GeneralResult<>(true, record, "查询成功");
    }

    @Override
    public GeneralResult list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            pager.getSorter().put("id", Common.ONE);
            int total = instanceRelationHistoryRecordRepository.count(pager.getParams());
            List<InstanceRelationHistoryRecord> list = this.instanceRelationHistoryRecordRepository.list(pager);
            if (CollectionUtils.isNotEmpty(list)) {
                list.forEach(template -> {
                    template
                            .setExecutor(redisTemplate.opsForValue().get("User_Name_" + template.getExecId()));
                    template
                            .setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
                });
            }

            GridBean<InstanceRelationHistoryRecord> bean =
                    new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("CmdbInstanceRelationHistoryRecord list query fail:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult cancel(Long id, RequestContext context) {
        try {
            InstanceRelationHistoryRecord record = instanceRelationHistoryRecordRepository.query(id);
            if (null == record) {
                return new GeneralResult<>(false, "关联历史取消调和失败：数据不存在");
            }

            if (!RelationHistoryConstant.Status.RECONCILED.getValue().equals(record.getStatus())) {
                return new GeneralResult<>(false, "关联历史取消调和失败：数据不是待调和状态");
            }
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.CANCEL_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(true, "关联历史取消调和成功");
        } catch (Exception e) {
            log.error("CmdbInstanceRelationHistoryRecord cancel fail:", e);
            return new GeneralResult<>(false, "关联历史取消调和失败");
        }
    }

    @Override
    public GeneralResult batchCancel(List<Long> ids, RequestContext context) {
        try {
            if (CollectionUtils.isEmpty(ids)) {
                return new GeneralResult(false, "关联历史取消调和失败：参数异常");
            }
            List<InstanceRelationHistoryRecord> records = instanceRelationHistoryRecordRepository.queryByIds(ids);
            if (CollectionUtils.isEmpty(records)) {
                return new GeneralResult<>(false, "关联历史取消调和失败：数据不存在");
            }
            //待调和状态记录
            List<InstanceRelationHistoryRecord> reconciledRecords = new ArrayList<>();
            for (InstanceRelationHistoryRecord record : records) {
                if (RelationHistoryConstant.Status.RECONCILED.getValue().equals(record.getStatus())) {
                    setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.CANCEL_ERROR_MSG);
                    reconciledRecords.add(record);
                }
            }
            if (CollectionUtils.isNotEmpty(reconciledRecords)) {
                instanceRelationHistoryRecordRepository.batchUpdate(reconciledRecords, 100);
                return new GeneralResult<>(true, "关联历史取消调和成功");
            }
            return new GeneralResult<>(false, "关联历史取消调和失败，不存在待调和记录");

        } catch (Exception e) {
            log.error("CmdbInstanceRelationHistoryRecord cancel fail:", e);
            return new GeneralResult<>(false, "关联历史取消调和失败");
        }
    }

    @Override
    public GeneralResult save(InstanceRelation instanceRelation, boolean isSource, String status, String type, String reason,
                              RequestContext context) {
        try {
            Long moduleRelationId = instanceRelation.getModuleRelationId();
            ModuleRelation moduleRelation = moduleRelationRepository.query(moduleRelationId);
            if (null == moduleRelation) {
                return new GeneralResult<>(false, "新增关联历史失败，模型关系不存在");
            }
            AutoRelationRule relationRule = autoRelationRuleRepository.queryByModuleRelationId(moduleRelation.getId());
            InstanceRelationHistoryRecord record =
                    getCanceInstanceRelationHistory(isSource, moduleRelation, relationRule,
                            RelationHistoryConstant.Way.MANUAL.getValue(), status, type, context.getTarget(), instanceRelation);

            record.setReason(reason);
            instanceRelationHistoryRecordRepository.save(record);

            return new GeneralResult<>(true, "新增关联历史成功");
        } catch (Exception e) {
            return new GeneralResult<>(false, "新增关联历史失败");
        }

    }

    @Override
    public GeneralResult reconcile(Long id, RequestContext context) {
        try {
            InstanceRelationHistoryRecord record = instanceRelationHistoryRecordRepository.query(id);
            if (null == record) {
                return new GeneralResult<>(false, "关联历史调和失败：数据不存在");
            }
            if (!RelationHistoryConstant.Status.RECONCILED.getValue().equals(record.getStatus())) {
                return new GeneralResult<>(false, "关联历史调和失败：数据不是待调和状态");
            }
            // 查询当前调和记录中的模型关系，根据最新的约束条件判断当前调和记录是否可以更新
            ModuleRelation moduleRelation = moduleRelationRepository.query(record.getModuleRelationId());
            // 模型关系不存在，取消调和记录
            if (null == moduleRelation) {
                setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_MODULE_ERROR_MSG);
                instanceRelationHistoryRecordRepository.update(record);
                return new GeneralResult<>(false, "关联历史调和失败，模型关系不存在");
            }

            boolean isSource = record.getIsSource();
            Instance sourceInstance;
            Instance targetInstance;
            String sourceInstanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getSourceCode().toLowerCase();
            String targetInstanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getTargetCode().toLowerCase();
            String sourceInstanceValueTable =
                    CmdbUtils.CMDB_INSTANCE_VALUE + moduleRelation.getSourceCode().toLowerCase();
            String targetInstanceValueTable =
                    CmdbUtils.CMDB_INSTANCE_VALUE + moduleRelation.getTargetCode().toLowerCase();
            if (isSource) {
                sourceInstance = instanceRepository.query(sourceInstanceTable, record.getInstanceId());
                if (null != sourceInstance) {
                    List<InstanceValue> sourceInstanceValues =
                            instanceValueRepository.listByInstanceId(record.getInstanceId(), sourceInstanceValueTable);
                    sourceInstance.setInstanceValues(sourceInstanceValues);
                }
                targetInstance = instanceRepository.query(targetInstanceTable, record.getRelationInstanceId());
                if (null != targetInstance) {
                    List<InstanceValue> targetInstanceValues = instanceValueRepository
                            .listByInstanceId(record.getRelationInstanceId(), targetInstanceValueTable);
                    targetInstance.setInstanceValues(targetInstanceValues);
                }
            } else {
                sourceInstance = instanceRepository.query(sourceInstanceTable, record.getRelationInstanceId());
                if (null != sourceInstance) {
                    List<InstanceValue> sourceInstanceValues = instanceValueRepository
                            .listByInstanceId(record.getRelationInstanceId(), sourceInstanceValueTable);
                    sourceInstance.setInstanceValues(sourceInstanceValues);
                }
                targetInstance = instanceRepository.query(targetInstanceTable, record.getInstanceId());
                if (null != targetInstance) {
                    List<InstanceValue> targetInstanceValues =
                            instanceValueRepository.listByInstanceId(record.getInstanceId(), targetInstanceValueTable);
                    targetInstance.setInstanceValues(targetInstanceValues);
                }
            }

            if (null == targetInstance || null == sourceInstance) {
                setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_INSTANCE_ERROR_MSG);
                instanceRelationHistoryRecordRepository.update(record);
                return new GeneralResult<>(false, "关联历史调和失败，实例信息不存在");
            }

            // 取消关联
            if (RelationHistoryConstant.Type.CANCEL.getValue().equals(record.getType())) {
                return cancelRelation(record, moduleRelation, context);
            } else {
                return addRelation(record, moduleRelation, context);
            }
        } catch (Exception e) {
            log.error("CmdbInstanceRelationHistoryRecord cancel fail:", e);
            return new GeneralResult<>(false, "关联历史调和失败");
        }
    }

    // 取消实例关系
    private GeneralResult cancelRelation(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                         RequestContext context) throws Exception {
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        String props = record.getProps();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(props, InstanceRelationHistoryAlterModel.class);
        List<InstanceRelation> relations = alter.getBefore();
        if (CollectionUtils.isEmpty(relations)) {
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_INSTANCE_RELATION_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(false, "关联历史调和失败，实例关系不存在");
        }
        InstanceRelation relation = relations.get(0);
        InstanceRelation tmpRelation = instanceRelationRepository.queryById(instanceRelationTable, relation.getId());
        if (null == tmpRelation) {
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_INSTANCE_RELATION_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(false, "关联历史调和失败，实例关系不存在");
        }
        instanceRelationRepository.remove(relation.getId(), instanceRelationTable, context.getTarget());
        setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.SUCCESS.getValue(), null);
        instanceRelationHistoryRecordRepository.update(record);
        this.addOperationalAudit(Arrays.asList(record),context.getTarget());
        return new GeneralResult<>(true, "关联历史调和成功");
    }

    private GeneralResult addRelation(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                      RequestContext context) throws Exception {
        String sourceType = moduleRelation.getSourceType();
        String targetType = moduleRelation.getTargetType();
        if (ModuleConstant.RELATION_TYPE_N.equals(sourceType) && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
            return dealNNReconcile(record, moduleRelation, context);
        }
        boolean isSource = record.getIsSource();
        if (ModuleConstant.RELATION_TYPE_1.equals(sourceType) && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
            return deal11Reconcile(record, moduleRelation, context);
        } else if (isSource && ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
            return dealN1PositiveReconcile(record, moduleRelation, context);
        } else if (!isSource && ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
            return dealN1PositiveReconcile(record, moduleRelation, context);
        } else {
            return dealSingleReconcile(record, moduleRelation, context);
        }

    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 单条调和更新
     * @Author majiali
     * @CreateTime 2022/6/14 15:16
     */
    private GeneralResult dealNNReconcile(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                          RequestContext context) throws Exception {
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 当前关联的实例
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getInstanceId(), moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), record.getInstanceId());
        }

        //实例关系不存在需要新建实例关系
        if (CollectionUtils.isEmpty(relationList)) {
            // 调和记录中新增关系
            InstanceRelation newRelation = alter.getAfter();
            handler.saveInstanceRelation(newRelation, instanceRelationTable);
            alter.setAfter(newRelation);
            record.setProps(JSONObject.toJSONString(alter));
        }
        // 更新调和记录
        setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.SUCCESS.getValue(), null);
        instanceRelationHistoryRecordRepository.update(record);
        return new GeneralResult<>(true, "任务调和成功");
    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 处理存在多行调和记录冲突的预检
     * @Author majiali
     * @CreateTime 2022/6/14 13:57
     */
    private GeneralResult deal11Reconcile(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                          RequestContext context) throws Exception {
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 调和记录中原来有冲突关系
        List<InstanceRelation> oldRelations = alter.getBefore();
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 当前关联方的实例关系
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), null);
        }
        //数据库中存在的当前调和记录要添加的关系
        List<InstanceRelation> currList = new ArrayList<>();
        //数据库中存在冲突的实例id
        List<Long> conflictList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationList)) {
            if (isSource) {
                currList = relationList.stream().filter(item -> item.getSourceInstanceId().equals(record.getInstanceId()))
                        .collect(Collectors.toList());
                conflictList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                currList = relationList.stream().filter(item -> item.getTargetInstanceId().equals(record.getInstanceId()))
                        .collect(Collectors.toList());
                conflictList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }

        //调和实例中记录的存在冲突的实例id
        List<Long> conflictOldList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(oldRelations)) {
            if (isSource) {
                conflictOldList = oldRelations.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictOldList = oldRelations.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }
        // 数据库中存在有冲突的实例id有不在调和记录中记录的实例id，表示有冲突，手动更新
        if (chargeConflictInfo(conflictList, conflictOldList)) {
            // 待调和记录手动更新失败
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(false, "任务调和失败");
        } else {
            if (CollectionUtils.isEmpty(currList)) {
                // 添加关系
                // 调和记录中新增关系
                InstanceRelation newRelation = alter.getAfter();
                handler.saveInstanceRelation(newRelation, instanceRelationTable);
                alter.setAfter(newRelation);
                record.setProps(JSONObject.toJSONString(alter));
            }
            // 更新当前调和记录
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.SUCCESS.getValue(), null);
            instanceRelationHistoryRecordRepository.update(record);
            if (CollectionUtils.isEmpty(currList)) {
                this.addOperationalAudit(Arrays.asList(record),context.getTarget());
            }

            List<InstanceRelation> delList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(relationList)) {
                if (isSource) {
                    delList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                            .collect(Collectors.toList());
                } else {
                    delList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                            .collect(Collectors.toList());
                }
            }

            // 查询当前关系之外的其他关系
            // 当前关联的实例关系
            List<InstanceRelation> currrelationList;
            if (isSource) {
                currrelationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                        moduleRelation.getSourceId(), record.getInstanceId(), moduleRelation.getTargetId(), null);
                currrelationList = currrelationList.stream()
                        .filter(item -> !item.getTargetInstanceId().equals(record.getRelationInstanceId()))
                        .collect(Collectors.toList());
            } else {
                currrelationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                        moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getInstanceId());
                currrelationList = currrelationList.stream()
                        .filter(item -> !item.getSourceInstanceId().equals(record.getRelationInstanceId()))
                        .collect(Collectors.toList());
            }

            delList.addAll(currrelationList);
            // 删除冲突关系
            if (CollectionUtils.isNotEmpty(delList)) {
                // 取消关系，增加记录
                List<Long> delIds = new ArrayList<>();
                List<InstanceRelationHistoryRecord> historyRecords = new ArrayList<>();
                AutoRelationRule relationRule =
                        autoRelationRuleRepository.queryByModuleRelationId(moduleRelation.getId());
                for (InstanceRelation del : delList) {
                    historyRecords.add(getCanceInstanceRelationHistory(isSource, moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                            RelationHistoryConstant.Type.CANCEL.getValue(), context.getTarget(), del));
                    delIds.add(del.getId());
                }
                instanceRelationRepository.removeByIds(delIds, instanceRelationTable, context.getTarget());
                instanceRelationHistoryRecordRepository.batchSave(historyRecords, 100);
                this.addOperationalAudit(historyRecords,context.getTarget());

            }

            //除了当前待调和记录外的其他待调和记录（同一个模型关系下）标记为失败
            List<InstanceRelationHistoryRecord> recordList =
                    instanceRelationHistoryRecordRepository.queryByCondition(record.getInstanceId(),
                            record.getModuleId(), RelationHistoryConstant.Status.RECONCILED.getValue(), record.getModuleRelationId());
            //当前调和记录已经更新了状态，待调和记录中查询不到了
            //recordList.remove(record);
            if (CollectionUtils.isNotEmpty(recordList)) {
                for (InstanceRelationHistoryRecord record1 : recordList) {
                    setExecInfo(record1, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
                }
                instanceRelationHistoryRecordRepository.batchUpdate(recordList, 100);
            }

            return new GeneralResult<>(true, "任务调和成功");
        }

    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: N-1处理存在多行调和记录冲突的预检
     * @Author majiali
     * @CreateTime 2022/6/14 13:57
     */
    private GeneralResult dealN1PositiveReconcile(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                                  RequestContext context) throws Exception {
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        //数据库中存在的当前调和记录要添加的关系
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getInstanceId(), moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), record.getInstanceId());
        }

        if (CollectionUtils.isEmpty(relationList)) {
            // 添加关系
            // 调和记录中新增关系
            InstanceRelation newRelation = alter.getAfter();
            handler.saveInstanceRelation(newRelation, instanceRelationTable);
            alter.setAfter(newRelation);
            record.setProps(JSONObject.toJSONString(alter));
        }
        // 更新当前调和记录
        setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.SUCCESS.getValue(), null);
        instanceRelationHistoryRecordRepository.update(record);
        if (CollectionUtils.isEmpty(relationList)) {
            this.addOperationalAudit(Arrays.asList(record), context.getTarget());
        }

        // 查询当前关系之外的其他关系（需要删除的关系）
        List<InstanceRelation> currrelationList;
        if (isSource) {
            currrelationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getInstanceId(), moduleRelation.getTargetId(), null);
            currrelationList = currrelationList.stream()
                    .filter(item -> !item.getTargetInstanceId().equals(record.getRelationInstanceId()))
                    .collect(Collectors.toList());
        } else {
            currrelationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getInstanceId());
            currrelationList = currrelationList.stream()
                    .filter(item -> !item.getSourceInstanceId().equals(record.getRelationInstanceId()))
                    .collect(Collectors.toList());
        }
        if (CollectionUtils.isNotEmpty(currrelationList)) {
            // 取消关系，增加记录
            List<Long> delIds = new ArrayList<>();
            List<InstanceRelationHistoryRecord> historyRecords = new ArrayList<>();
            AutoRelationRule relationRule =
                    autoRelationRuleRepository.queryByModuleRelationId(moduleRelation.getId());
            for (InstanceRelation del : currrelationList) {
                historyRecords.add(getCanceInstanceRelationHistory(isSource, moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                        RelationHistoryConstant.Type.CANCEL.getValue(), context.getTarget(), del));
                delIds.add(del.getId());
            }
            instanceRelationRepository.removeByIds(delIds, instanceRelationTable, context.getTarget());
            instanceRelationHistoryRecordRepository.batchSave(historyRecords, 100);
            this.addOperationalAudit(historyRecords,context.getTarget());

        }
        //除了当前待调和记录外的其他待调和记录（同一个模型关系下）标记为失败
        List<InstanceRelationHistoryRecord> recordList =
                instanceRelationHistoryRecordRepository.queryByCondition(record.getInstanceId(),
                        record.getModuleId(), RelationHistoryConstant.Status.RECONCILED.getValue(), record.getModuleRelationId());
        //当前调和记录已经更新了状态，待调和记录中查询不到了
        //recordList.remove(record);
        if (CollectionUtils.isNotEmpty(recordList)) {
            for (InstanceRelationHistoryRecord record1 : recordList) {
                setExecInfo(record1, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
            }
            instanceRelationHistoryRecordRepository.batchUpdate(recordList, 100);
        }

        return new GeneralResult<>(true, "任务调和成功");

    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 单条调和更新
     * @Author majiali
     * @CreateTime 2022/6/14 15:16
     */
    private GeneralResult dealSingleReconcile(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                              RequestContext context) throws Exception {
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 调和记录中原来有冲突关系
        List<InstanceRelation> oldRelations = alter.getBefore();
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 当前关联的实例
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), null);
        }
        //数据库中存在的当前调和记录要添加的关系
        List<InstanceRelation> currList = new ArrayList<>();
        //数据库中存在冲突的实例id
        List<Long> conflictList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationList)) {
            if (isSource) {
                currList = relationList.stream().filter(item -> item.getSourceInstanceId().equals(record.getInstanceId()))
                        .collect(Collectors.toList());
                conflictList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                currList = relationList.stream().filter(item -> item.getTargetInstanceId().equals(record.getInstanceId()))
                        .collect(Collectors.toList());
                conflictList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }

        List<Long> conflictOldList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(oldRelations)) {
            if (isSource) {
                conflictOldList = oldRelations.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictOldList = oldRelations.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }
        // 有冲突，手动更新
        if (chargeConflictInfo(conflictList, conflictOldList)) {
            // 待调和记录手动更新失败
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(false, "任务调和失败");
        } else {
            if (CollectionUtils.isEmpty(currList)) {
                // 添加关系
                // 调和记录中新增关系
                InstanceRelation newRelation = alter.getAfter();
                handler.saveInstanceRelation(newRelation, instanceRelationTable);
                alter.setAfter(newRelation);
                record.setProps(JSONObject.toJSONString(alter));
            }
            // 更新调和记录
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.SUCCESS.getValue(), null);
            instanceRelationHistoryRecordRepository.update(record);
            if (CollectionUtils.isEmpty(currList)) {
                this.addOperationalAudit(Arrays.asList(record), context.getTarget());
            }
            // 删除冲突关系
            if (CollectionUtils.isNotEmpty(relationList)) {
                List<InstanceRelation> delList;
                if (isSource) {
                    delList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                            .collect(Collectors.toList());
                } else {
                    delList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                            .collect(Collectors.toList());
                }
                if (CollectionUtils.isNotEmpty(delList)) {
                    // 取消关系，增加记录
                    List<Long> delIds = new ArrayList<>();
                    List<InstanceRelationHistoryRecord> historyRecords = new ArrayList<>();
                    AutoRelationRule relationRule =
                            autoRelationRuleRepository.queryByModuleRelationId(moduleRelation.getId());
                    for (InstanceRelation del : delList) {
                        historyRecords.add(getCanceInstanceRelationHistory(isSource, moduleRelation, relationRule,
                                RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                                RelationHistoryConstant.Type.CANCEL.getValue(), context.getTarget(), del));
                        delIds.add(del.getId());
                    }
                    instanceRelationHistoryRecordRepository.batchSave(historyRecords, 100);
                    instanceRelationRepository.removeByIds(delIds, instanceRelationTable, context.getTarget());
                    this.addOperationalAudit(historyRecords,context.getTarget());
                }
            }
            return new GeneralResult<>(true, "任务调和成功");
        }
    }

    private InstanceRelationHistoryRecord getCanceInstanceRelationHistory(boolean isPositive,
                                                                          ModuleRelation moduleRelation, AutoRelationRule relationRule, String way, String status, String type,
                                                                          Long userId, InstanceRelation oldRelation) {
        InstanceRelationHistoryRecord record = new InstanceRelationHistoryRecord();
        InstanceRelationHistoryAlterModel alter = new InstanceRelationHistoryAlterModel();
        // 当前实例是源实例
        if (isPositive) {
            record.setName(oldRelation.getSourceInstanceName());
            record.setInstanceId(oldRelation.getSourceInstanceId());
            record.setModuleId(oldRelation.getSourceModuleId());
            record.setRelationInstanceId(oldRelation.getTargetInstanceId());
            record.setModuleName(moduleRelation.getSourceName());

        } else {
            record.setName(oldRelation.getTargetInstanceName());
            record.setInstanceId(oldRelation.getTargetInstanceId());
            record.setModuleId(oldRelation.getTargetModuleId());
            record.setRelationInstanceId(oldRelation.getSourceInstanceId());
            record.setModuleName(moduleRelation.getTargetName());
        }
        // 取消关联
        if (RelationHistoryConstant.Type.CANCEL.getValue().equals(type)) {
            record.setAlterAfter("--");
            record.setAlterBefore(oldRelation.getSourceInstanceName() + "->" + oldRelation.getTargetInstanceName());
            alter.setBefore(Arrays.asList(oldRelation));
        } else {
            record.setAlterBefore("--");
            record.setAlterAfter(oldRelation.getSourceInstanceName() + "->" + oldRelation.getTargetInstanceName());
            alter.setAfter(oldRelation);
        }
        if (null != relationRule) {
            record.setAutoRelationRuleId(relationRule.getId());
        }
        record.setRelationCategoryId(moduleRelation.getRelationCategoryId());
        record.setModuleRelationId(moduleRelation.getId());
        record.setIsSource(isPositive);
        record.setProps(JSONObject.toJSONString(alter));
        record.setDeleted(false);
        record.setCreatorId(userId);
        record.setGmtCreate(new Date());
        record.setWay(way);
        record.setStatus(status);
        record.setType(type);
        record.setExecTime(new Date());
        record.setExecId(userId);
        return record;
    }

    @Override
    public GeneralResult preflight(Long id, RequestContext context) {
        try {
            InstanceRelationHistoryPreflightModel model = new InstanceRelationHistoryPreflightModel();
            InstanceRelationHistoryRecord record = instanceRelationHistoryRecordRepository.query(id);
            if (null == record) {
                return new GeneralResult<>(false, "关联历史预检失败：数据不存在");
            }
            if (!RelationHistoryConstant.Status.RECONCILED.getValue().equals(record.getStatus())) {
                return new GeneralResult<>(false, "关联历史预检失败：数据不是待调和状态");
            }
            // 查询当前调和记录中的模型关系，根据最新的约束条件判断当前调和记录是否可以更新
            ModuleRelation moduleRelation = moduleRelationRepository.query(record.getModuleRelationId());
            // 模型关系不存在，取消调和记录
            if (null == moduleRelation) {
                setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_MODULE_ERROR_MSG);
                instanceRelationHistoryRecordRepository.update(record);
                return new GeneralResult<>(false, "关联历史预检失败，模型关系不存在");
            }
            // 取消关联--调和中的记录应该没有取消关联类型
            if (RelationHistoryConstant.Type.CANCEL.getValue().equals(record.getType())) {
                model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
                return new GeneralResult<>(true, model, "关联历史预检成功");
            }
            boolean isSource = record.getIsSource();
            Instance sourceInstance;
            Instance targetInstance;
            String sourceInstanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getSourceCode().toLowerCase();
            String targetInstanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getTargetCode().toLowerCase();
            String sourceInstanceValueTable =
                    CmdbUtils.CMDB_INSTANCE_VALUE + moduleRelation.getSourceCode().toLowerCase();
            String targetInstanceValueTable =
                    CmdbUtils.CMDB_INSTANCE_VALUE + moduleRelation.getTargetCode().toLowerCase();
            if (isSource) {
                sourceInstance = instanceRepository.query(sourceInstanceTable, record.getInstanceId());
                if (null != sourceInstance) {
                    List<InstanceValue> sourceInstanceValues =
                            instanceValueRepository.listByInstanceId(record.getInstanceId(), sourceInstanceValueTable);
                    sourceInstance.setInstanceValues(sourceInstanceValues);
                }
                targetInstance = instanceRepository.query(targetInstanceTable, record.getRelationInstanceId());
                if (null != targetInstance) {
                    List<InstanceValue> targetInstanceValues = instanceValueRepository
                            .listByInstanceId(record.getRelationInstanceId(), targetInstanceValueTable);
                    targetInstance.setInstanceValues(targetInstanceValues);
                }
            } else {
                sourceInstance = instanceRepository.query(sourceInstanceTable, record.getRelationInstanceId());
                if (null != sourceInstance) {
                    List<InstanceValue> sourceInstanceValues = instanceValueRepository
                            .listByInstanceId(record.getRelationInstanceId(), sourceInstanceValueTable);
                    sourceInstance.setInstanceValues(sourceInstanceValues);
                }
                targetInstance = instanceRepository.query(targetInstanceTable, record.getInstanceId());
                if (null != targetInstance) {
                    List<InstanceValue> targetInstanceValues =
                            instanceValueRepository.listByInstanceId(record.getInstanceId(), targetInstanceValueTable);
                    targetInstance.setInstanceValues(targetInstanceValues);
                }
            }

            if (null == targetInstance || null == sourceInstance) {
                setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.NO_INSTANCE_ERROR_MSG);
                instanceRelationHistoryRecordRepository.update(record);
                return new GeneralResult<>(false, "关联历史预检失败，实例信息不存在");
            }
            String sourceType = moduleRelation.getSourceType();
            String targetType = moduleRelation.getTargetType();
            if (ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
                //N-N关系是可以直接添加关系的。会出现调和记录一般为约束条件1-N或者1-1改为N-N引起的。（调和记录为1-N或1-1的时候产生的）
                model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
                return new GeneralResult<>(true, model, "关联历史预检成功");
            }
            if (ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                return deal11Preflight(record, moduleRelation, context);
            } else if (isSource && ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                return dealN1PositivePreflight(record, moduleRelation, context);
            } else if (!isSource && ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
                return dealN1PositivePreflight(record, moduleRelation, context);
            } else {
                return dealSinglePreflight(record, moduleRelation, context);
            }

        } catch (Exception e) {
            log.error("CmdbInstanceRelationHistoryRecord preflight fail:", e);
            return new GeneralResult<>(false, "关联历史预检失败");
        }
    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 单条记录预检
     * @Author majiali
     * @CreateTime 2022/6/14 13:55
     */
    private GeneralResult dealSinglePreflight(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                              RequestContext context) throws Exception {
        InstanceRelationHistoryPreflightModel model = new InstanceRelationHistoryPreflightModel();
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 调和记录中原来有冲突关系
        List<InstanceRelation> oldRelations = alter.getBefore();
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 当前关联的实例
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), null);
        }
        List<Long> conflictList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationList)) {
            if (isSource) {
                conflictList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }

        List<Long> conflictOldList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(oldRelations)) {
            if (isSource) {
                conflictOldList = oldRelations.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictOldList = oldRelations.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }
        // 有冲突，手动更新
        if (chargeConflictInfo(conflictList, conflictOldList)) {
            model.setStatus(RelationHistoryConstant.Status.FAILED.getValue());
            // 待调和记录手动更新失败
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(false, model, RelationHistoryConstant.RECONCILED_ERROR_MSG);
        } else {
            // 预检成功
            model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        }

    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 1-1关系预检处理存在多行调和记录冲突的预检
     * @Author majiali
     * @CreateTime 2022/6/14 13:57
     */
    private GeneralResult deal11Preflight(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                          RequestContext context) throws Exception {
        InstanceRelationHistoryPreflightModel model = new InstanceRelationHistoryPreflightModel();
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 调和记录中原来有冲突关系
        List<InstanceRelation> oldRelations = alter.getBefore();
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 当前关联的实例
        List<InstanceRelation> relationList;
        if (isSource) {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), record.getRelationInstanceId());
        } else {
            relationList = instanceRelationRepository.queryByCondition(instanceRelationTable,
                    moduleRelation.getSourceId(), record.getRelationInstanceId(), moduleRelation.getTargetId(), null);
        }
        List<Long> conflictList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationList)) {
            if (isSource) {
                conflictList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }

        List<Long> conflictOldList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(oldRelations)) {
            if (isSource) {
                conflictOldList = oldRelations.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictOldList = oldRelations.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }

        // 当前调和记录是否有冲突，手动更新
        if (chargeConflictInfo(conflictList, conflictOldList)) {
            model.setStatus(RelationHistoryConstant.Status.FAILED.getValue());
            // 待调和记录手动更新失败
            setExecInfo(record, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
            instanceRelationHistoryRecordRepository.update(record);
            return new GeneralResult<>(true, model, RelationHistoryConstant.RECONCILED_ERROR_MSG);
        }

        // 除了当前待调和记录外的其他待调和记录（同一个模型关系下）
        List<InstanceRelationHistoryRecord> recordList = instanceRelationHistoryRecordRepository.queryByCondition(
                record.getInstanceId(), record.getModuleId(), RelationHistoryConstant.Status.RECONCILED.getValue(), record.getModuleRelationId());
        recordList.remove(record);

        if (CollectionUtils.isEmpty(recordList)) {
            model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        }

        List<Long> relationIds = recordList.stream().map(InstanceRelationHistoryRecord::getRelationInstanceId).collect(Collectors.toList());
        // 当前关联的实例
        List<InstanceRelation> ralationList;
        if (isSource) {
            ralationList = instanceRelationRepository.queryByInstanceIds(instanceRelationTable,
                    moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), relationIds);
        } else {
            ralationList = instanceRelationRepository.queryByInstanceIds(instanceRelationTable,
                    moduleRelation.getSourceId(), relationIds, moduleRelation.getTargetId(), null);
        }

        Map<Long, List<InstanceRelation>> relationMap;
        if (isSource) {
            relationMap = ralationList.stream().collect(Collectors.groupingBy(InstanceRelation::getTargetInstanceId));
        } else {
            relationMap = ralationList.stream().collect(Collectors.groupingBy(InstanceRelation::getSourceInstanceId));
        }
        List<InstanceRelationHistoryRecord> successList = new ArrayList<>();
        successList.add(record);
        List<InstanceRelationHistoryRecord> failedList = new ArrayList<>();
        for (InstanceRelationHistoryRecord historyRecord : recordList) {
            if (singlePreflight(historyRecord, moduleRelation,
                    relationMap.get(historyRecord.getRelationInstanceId()))) {
                // 待调和记录手动更新失败
                setExecInfo(historyRecord, context.getTarget(), RelationHistoryConstant.Status.FAILED.getValue(), RelationHistoryConstant.RECONCILED_ERROR_MSG);
                failedList.add(historyRecord);
            } else {
                // 预检成功的记录
                successList.add(historyRecord);
            }
        }
        if (CollectionUtils.isNotEmpty(failedList)) {
            instanceRelationHistoryRecordRepository.batchUpdate(failedList, 100);
        }
        if (successList.size() > 1) {
            model.setSourceType(moduleRelation.getSourceType());
            model.setTargetType(moduleRelation.getTargetType());
            model.setHistoryRecordList(successList);
            model.setStatus(RelationHistoryConstant.Status.RECONCILED.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        } else {
            model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        }
    }

    /**
     * @param record
     * @param moduleRelation
     * @param context
     * @Description: 正向N-1处理存在多行调和记录冲突的预检
     * @Author majiali
     * @CreateTime 2022/6/14 13:57
     */
    private GeneralResult dealN1PositivePreflight(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                                  RequestContext context) throws Exception {
        InstanceRelationHistoryPreflightModel model = new InstanceRelationHistoryPreflightModel();

        // 除了当前待调和记录外的其他待调和记录（同一个模型关系下）
        List<InstanceRelationHistoryRecord> recordList = instanceRelationHistoryRecordRepository.queryByCondition(
                record.getInstanceId(), record.getModuleId(), RelationHistoryConstant.Status.RECONCILED.getValue(), record.getModuleRelationId());
        //只有一条调和记录。预检成功
        if (CollectionUtils.isEmpty(recordList)) {
            return new GeneralResult<>(false, "调和任务预检失败，数据异常");
        }
        if (recordList.size() == 1) {
            model.setStatus(RelationHistoryConstant.Status.SUCCESS.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        } else {
            model.setSourceType(moduleRelation.getSourceType());
            model.setTargetType(moduleRelation.getTargetType());
            model.setHistoryRecordList(recordList);
            model.setStatus(RelationHistoryConstant.Status.RECONCILED.getValue());
            return new GeneralResult<>(true, model, "调和任务预检成功");
        }
    }

    private boolean singlePreflight(InstanceRelationHistoryRecord record, ModuleRelation moduleRelation,
                                    List<InstanceRelation> relationList) throws Exception {
        boolean isSource = record.getIsSource();
        InstanceRelationHistoryAlterModel alter = JSONObject.parseObject(record.getProps(), InstanceRelationHistoryAlterModel.class);
        // 调和记录中原来有冲突关系
        List<InstanceRelation> oldRelations = alter.getBefore();
        List<Long> conflictList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(relationList)) {
            if (isSource) {
                conflictList = relationList.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictList = relationList.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }
        List<Long> conflictOldList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(oldRelations)) {
            if (isSource) {
                conflictOldList = oldRelations.stream().filter(item -> !item.getSourceInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getSourceInstanceId).collect(Collectors.toList());
            } else {
                conflictOldList = oldRelations.stream().filter(item -> !item.getTargetInstanceId().equals(record.getInstanceId()))
                        .map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList());
            }
        }
        return chargeConflictInfo(conflictList, conflictOldList);
    }

    /**
     * @param conflictList    现有冲突的关联实例id
     * @param conflictOldList 调和记录中记录的关联实例id
     * @Description: 判断现有冲突的关联实例id与调和记录中记录的关联实例id是否有冲突  conflictList中移除了conflictOldList
     * @Author majiali
     * @CreateTime 2022/6/14 11:05
     */
    private boolean chargeConflictInfo(List<Long> conflictList, List<Long> conflictOldList) {
        if (CollectionUtils.isEmpty(conflictList)) {
            return false;
        }
        // 现有
        conflictList.removeAll(conflictOldList);
        if (CollectionUtils.isEmpty(conflictList)) {
            return false;
        }
        return true;
    }

    /**
     * @param record 关联历史记录
     * @param userId 操作人
     * @Description: 设置关联历史的执行信息
     * @Author majiali
     * @CreateTime 2022/6/16 11:25
     */
    private void setExecInfo(InstanceRelationHistoryRecord record, Long userId, String status, String reason) {
        record.setExecId(userId);
        record.setExecTime(new Date());
        record.setGmtModify(new Date());
        record.setMenderId(userId);
        record.setStatus(status);
        record.setReason(reason);
    }

    //操作审计
    private void addOperationalAudit(List<InstanceRelationHistoryRecord> list,Long userId) {
        try {
            List<OperationalAudit> auditList = new ArrayList<>();
            for (InstanceRelationHistoryRecord historyRecord : list) {
                if(!RelationHistoryConstant.Status.SUCCESS.getValue().equals(historyRecord.getStatus())){
                    continue;
                }
                String props = historyRecord.getProps();
                //关联
                InstanceRelationHistoryAlterModel alter = null;
                if (StringUtils.isNotEmpty(props)) {
                    alter = JSONObject.parseObject(historyRecord.getProps(), InstanceRelationHistoryAlterModel.class);
                }
                //关联
                Map<String, Object> afterMap = null;
                Map<String, Object> beforeMap = null;
                OperationalAudit audit = null;
                if (RelationHistoryConstant.Type.CANCEL.getValue().equals(historyRecord.getType())) {

                    if (null != alter && CollectionUtils.isNotEmpty(alter.getBefore())) {
                        beforeMap = JSON.parseObject(JSON.toJSONString(alter.getBefore().get(0)), new TypeReference<Map<String, Object>>() {
                        });
                    }
                    audit = operationalAuditService.buildLog(OperationalAuditType.ASSOCIATION, historyRecord.getModuleName(),
                            historyRecord.getName(), historyRecord.getModuleCode(), historyRecord.getInstanceId(), userId, beforeMap, afterMap);

                } else {
                    if (null != alter && null != alter.getAfter()) {
                        afterMap =
                                JSON.parseObject(JSON.toJSONString(alter.getAfter()), new TypeReference<Map<String, Object>>() {
                                });
                    }

                    audit = operationalAuditService.buildLog(OperationalAuditType.RELATION, historyRecord.getModuleName(),
                            historyRecord.getName(), historyRecord.getModuleCode(), historyRecord.getInstanceId(), userId, beforeMap, afterMap);
                }

                auditList.add(audit);
            }

            if(!auditList.isEmpty()){
                operationalAuditRepository.batchSave(auditList, 100);
            }

        }catch (Exception e){
            log.error("记录操作审计信息异常",e);
        }

    }
}
