package com.bocloud.ams.service.module.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.entity.association.AutoRelationRuleItemValue;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.model.ModuleLink;
import com.bocloud.ams.entity.model.ModuleTopo;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.Relation;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.repository.association.AutoRelationRuleItemRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleItemValueRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.RelationCategoryRepository;
import com.bocloud.ams.repository.module.RelationRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceService;
import com.bocloud.ams.service.module.ModuleRelationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Description: 模型关系管理
 * @Title: CmdbRelationService
 * @Author: majiali
 * @CreateTime: 2022/5/7 16:59
 */
@Slf4j
@Service("moduleRelationService")
public class ModuleRelationServiceImpl implements ModuleRelationService {

    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private RelationRepository relationRepository;
    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private AutoRelationRuleRepository autoRelationRuleRepository;
    @Autowired
    private AutoRelationRuleItemRepository autoRelationRuleItemRepository;
    @Autowired
    private AutoRelationRuleItemValueRepository autoRelationRuleItemValueRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceService instanceService;

    @Override
    public GeneralResult create(ModuleRelation relation, Long userId) {
        try {
            Long sourceId = relation.getSourceId();
            Long targetId = relation.getTargetId();
            Long relationId = relation.getRelationId();
            CiModule sourceModule = moduleRepository.query(sourceId);
            if (null == sourceModule) {
                return new GeneralResult<>(false, "源模型不存在");
            }
            CiModule targetModule = moduleRepository.query(targetId);
            if (null == targetModule) {
                return new GeneralResult<>(false, "目的模型不存在");
            }
            if (sourceId.equals(targetId)) {
                return new GeneralResult<>(false, "源模型和目的模型不能为同一个模型");
            }
            RelationCategory relationCategory = relationCategoryRepository.query(relation.getRelationCategoryId());
            if (null == relationCategory) {
                return new GeneralResult<>(false, "关联类型分组不存在");
            }
            Relation cmdbRelation = relationRepository.query(relationId);
            if (null == cmdbRelation) {
                return new GeneralResult<>(false, "关联类型不存在");
            }
            // 正向规则
            ModuleRelation moduleRelation =
                moduleRelationRepository.query(relation.getRelationCategoryId(), sourceId, targetId);
            if (null != moduleRelation) {
                return new GeneralResult<>(false, "模型关系已经存在");
            }
            // 反向规则校验
            ModuleRelation everseRelation =
                moduleRelationRepository.query(relation.getRelationCategoryId(), targetId, sourceId);
            if (null != everseRelation) {
                return new GeneralResult<>(false, "模型关系已经存在");
            }
            relation.setSourceCode(sourceModule.getCode());
            relation.setTargetCode(targetModule.getCode());
            relation.setRelationCategoryCode(relationCategory.getCode());
            relation.setDeleted(false);
            relation.setCreatorId(userId);
            relation.setMenderId(userId);
            relation.setGmtModify(new Date());
            relation.setGmtCreate(new Date());
            relation.setName(sourceModule.getName() + cmdbRelation.getName() + targetModule.getName());
            moduleRelationRepository.save(relation);
            List<AutoRelationRuleItem> item = relation.getRuleItemList();
            if (!CollectionUtils.isEmpty(item)) {
                AutoRelationRule rule = new AutoRelationRule();
                rule.setName(sourceModule.getName() + cmdbRelation.getName() + targetModule.getName());
                rule.setItems(item);
                rule.setSourceModuleId(sourceId);
                rule.setTargetModuleId(targetId);
                rule.setRelationCategoryId(relation.getRelationCategoryId());
                rule.setModuleRelationId(relation.getId());
                saveRelationRule(rule, userId);
            }
            return new GeneralResult<>(true, "模型关系新增成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation create fail:", e);
            return new GeneralResult<>(false, "模型关系新增失败");
        }
    }

    private void saveRelationRule(AutoRelationRule rule, Long target) {
        if (CollectionUtils.isEmpty(rule.getItems())) {
            return;
        }
        rule.setCreatorId(target);
        rule.setGmtCreate(new Date());
        rule.setDeleted(false);
        autoRelationRuleRepository.save(rule);
        rule.getItems().forEach(item -> {
            item.setRelationRuleId(rule.getId());
            item.setCreatorId(target);
            item.setGmtCreate(new Date());
            item.setDeleted(false);
            autoRelationRuleItemRepository.save(item);
            List<AutoRelationRuleItemValue> itemValues = item.getValues();
            if (!CollectionUtils.isEmpty(itemValues)) {
                itemValues.forEach(value -> {
                    value.setRelationRuleId(rule.getId());
                    value.setItemId(item.getId());
                    value.setCreatorId(target);
                    value.setGmtCreate(new Date());
                    item.setDeleted(false);
                    autoRelationRuleItemValueRepository.save(value);
                });
            }
        });
    }

    @Override
    public GeneralResult modify(ModuleRelation relation, Long userId) {
        String path = ModuleRelationServiceImpl.class.getSimpleName() + "_" + relation.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CiModule sourceModule = moduleRepository.query(relation.getSourceId());
            if (null == sourceModule) {
                return new GeneralResult<>(false, "源模型不存在");
            }
            CiModule targetModule = moduleRepository.query(relation.getTargetId());
            if (null == targetModule) {
                return new GeneralResult<>(false, "目的模型不存在");
            }
            if (relation.getSourceId().equals(relation.getTargetId())) {
                return new GeneralResult<>(false, "源模型和目的模型不能为同一个模型");
            }
            RelationCategory relationCategory = relationCategoryRepository.query(relation.getRelationCategoryId());
            if (null == relationCategory) {
                return new GeneralResult<>(false, "关联类型分组不存在");
            }
            Relation cmdbRelation = relationRepository.query(relation.getRelationId());
            if (null == cmdbRelation) {
                return new GeneralResult<>(false, "关联类型不存在");
            }
            ModuleRelation temp = moduleRelationRepository.query(relation.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "模型关系不存在");
            }
            LocaleBeanUtils.copyProperties(temp, relation);
            temp.setName(sourceModule.getName() + cmdbRelation.getName() + targetModule.getName());
            temp.setGmtModify(new Date());
            temp.setMenderId(userId);
            moduleRelationRepository.update(temp);
            // 查询规则
            AutoRelationRule rule = autoRelationRuleRepository.queryByModuleRelationId(temp.getId());
            if (null == rule) {
                rule = new AutoRelationRule();
                rule.setName(temp.getName());
                rule.setItems(relation.getRuleItemList());
                rule.setSourceModuleId(temp.getSourceId());
                rule.setTargetModuleId(temp.getTargetId());
                rule.setRelationCategoryId(temp.getRelationCategoryId());
                rule.setModuleRelationId(temp.getId());
                rule.setItems(relation.getRuleItemList());
                saveRelationRule(rule, userId);
            } else {
                rule.setItems(relation.getRuleItemList());
                modifyRelationRule(rule, userId);
            }
            return new GeneralResult<>(true, "模型关系修改成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation modify fail:", e);
            return new GeneralResult<>(false, "模型关系修改失败");
        }
    }

    private void modifyRelationRule(AutoRelationRule rule, Long target) throws Exception {
        // 更新规则明细数据
        autoRelationRuleItemRepository.deleteByRuleId(rule.getId(), target);
        autoRelationRuleItemValueRepository.deleteByRuleId(rule.getId(), target);
        if (CollectionUtils.isEmpty(rule.getItems())) {
            autoRelationRuleRepository.delete(rule.getId(), target);
            return;
        }
        // 更新模型关系规则
        rule.setGmtModify(new Date());
        rule.setMenderId(target);
        autoRelationRuleRepository.update(rule);
        rule.getItems().forEach(item -> {
            item.setRelationRuleId(rule.getId());
            item.setId(null);
            item.setCreatorId(target);
            item.setGmtCreate(new Date());
            autoRelationRuleItemRepository.save(item);
            List<AutoRelationRuleItemValue> itemValues = item.getValues();
            if (!CollectionUtils.isEmpty(itemValues)) {
                itemValues.forEach(value -> {
                    value.setRelationRuleId(rule.getId());
                    value.setId(null);
                    value.setItemId(item.getId());
                    value.setCreatorId(target);
                    value.setGmtCreate(new Date());
                    autoRelationRuleItemValueRepository.save(value);
                });
            }
        });
    }

    @Override
    public GeneralResult detail(Long id) {
        try {
            ModuleRelation relation = moduleRelationRepository.query(id);
            if (null == relation) {
                return new GeneralResult<>(false, "模型关系查询失败，数据不存在");
            }
            // 查询关联规则信息
            AutoRelationRule rule = autoRelationRuleRepository.queryByModuleRelationId(id);
            if (null != rule) {
                List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(rule.getId());
                for (AutoRelationRuleItem item : items) {
                    item.setValues(autoRelationRuleItemValueRepository.queryByItemId(item.getId()));
                }
                relation.setRuleItemList(items);
            }
            return new GeneralResult<>(true, relation, "模型关系查询成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation detail query fail:", e);
            return new GeneralResult<>(false, "模型关系查询失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, Long userId) {
        String path = ModuleRelationServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ModuleRelation temp = moduleRelationRepository.query(id);
            if (null == temp) {
                return new GeneralResult<>(false, "模型关系不存在");
            }
            // 查询实例关系
            String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + temp.getRelationCategoryCode().toLowerCase();
            // 查询该模型关系下的所有实例关系
            List<InstanceRelation> relations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                temp.getSourceId(), null, temp.getTargetId(), null);
            if (CollectionUtils.isNotEmpty(relations)) {
                return new GeneralResult<>(false, "当前模型关系下存在对应的实例关系，请先删除实例关系");
            }
            // 级联删除规则相关信息
            AutoRelationRule relationRule = autoRelationRuleRepository.queryByModuleRelationId(id);
            if (null != relationRule) {
                autoRelationRuleItemValueRepository.deleteByRuleId(relationRule.getId(), userId);
                autoRelationRuleItemRepository.deleteByRuleId(relationRule.getId(), userId);
                autoRelationRuleRepository.delete(relationRule.getId(), userId);
            }
            moduleRelationRepository.delete(id, userId);
            return new GeneralResult<>(true, "模型关系删除成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation remove fail:", e);
            return new GeneralResult<>(false, "模型关系删除失败");
        }
    }

    @Override
    public GeneralResult<ModuleTopo> list(Long moduleId, Long relationCategoryId) {
        try {
            if (null == moduleId || null == relationCategoryId) {
                return GeneralResult.FAILED("模型id或拓扑id不可为空");
            }
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return GeneralResult.FAILED("该模型不存在");
            }
            RelationCategory category = relationCategoryRepository.query(relationCategoryId);
            if (null == category) {
                return GeneralResult.FAILED("该拓扑不存在");
            }
            // 正向关系
            List<ModuleRelation> relations = moduleRelationRepository.list(moduleId, relationCategoryId);
            // 反向关系isReverse
            List<ModuleRelation> reverseRelations = moduleRelationRepository.listReverse(moduleId, relationCategoryId);
            relations.addAll(reverseRelations);
            // 所有模型ids
            Set<Long> moduleSet = new HashSet<>();
            // 放入自身
            moduleSet.add(module.getId());
            List<ModuleLink> links = new ArrayList<>();
            for (ModuleRelation relation : relations) {
                moduleSet.add(relation.getSourceId());
                moduleSet.add(relation.getTargetId());
                // 放入link
                ModuleLink link = new ModuleLink();
                link.setRelationId(relation.getId());
                link.setRelationCategoryId(relation.getRelationCategoryId());
                link.setRelationCode(relation.getRelationCode());
                link.setRelationName(relation.getRelationName());
                link.setIsReversed(relation.getIsReversed());
                // 反向的时候，箭头相反相关信息
                if (relation.getIsReversed()) {
                    link.setSource(relation.getTargetId());
                    link.setTarget(relation.getSourceId());
                    link.setType(relation.getTargetType() + "-" + relation.getSourceType());
                } else {
                    link.setSource(relation.getSourceId());
                    link.setTarget(relation.getTargetId());
                    link.setType(relation.getSourceType() + "-" + relation.getTargetType());
                }
                links.add(link);
            }
            List<CiModule> nodes = new ArrayList<>();
            if (moduleSet.size() > 0) {
                List<Long> ids = new ArrayList<>();
                ids.addAll(moduleSet);
                nodes = moduleRepository.listByIds(ids);
            }
            ModuleTopo topo = new ModuleTopo(nodes, links);
            return new GeneralResult<>(true, topo, "查询模型关系成功!");
        } catch (Exception e) {
            log.error("CmdbModuleRelation list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<ModuleTopo> listByCategoryId(Long relationCategoryId) {
        try {
            if (null == relationCategoryId) {
                return GeneralResult.FAILED("拓扑id不可为空");
            }
            RelationCategory relationCategory = relationCategoryRepository.query(relationCategoryId);
            if (null == relationCategory) {
                return GeneralResult.FAILED("该拓扑不存在");
            }
            // 正向找关系
            List<ModuleRelation> relations = moduleRelationRepository.listByCategoryId(relationCategoryId);
            // 所有模型ids
            Set<Long> moduleSet = new HashSet<>();
            List<ModuleLink> links = new ArrayList<>();
            for (ModuleRelation relation : relations) {
                moduleSet.add(relation.getSourceId());
                moduleSet.add(relation.getTargetId());
                // 放入link
                ModuleLink link = new ModuleLink();
                link.setRelationId(relation.getId());
                link.setRelationCategoryId(relation.getRelationCategoryId());
                link.setRelationCode(relation.getRelationCode());
                link.setRelationName(relation.getRelationName());
                link.setSource(relation.getSourceId());
                link.setTarget(relation.getTargetId());
                link.setType(relation.getSourceType() + "-" + relation.getTargetType());
                links.add(link);
            }
            List<CiModule> nodes = new ArrayList<>();
            if (moduleSet.size() > 0) {
                List<Long> ids = new ArrayList<>();
                ids.addAll(moduleSet);
                nodes = moduleRepository.listByIds(ids);
            }
            // 处理nodes和links
            ModuleTopo topo = new ModuleTopo(nodes, links);
            return new GeneralResult<>(true, topo, "查询模型关系成功!");
        } catch (Exception e) {
            log.error("CmdbModuleRelation list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<ModuleRelation> relevancy(Long relationCategoryId, Long sourceId, Long targetId) {
        try {
            if (null == relationCategoryId) {
                return new GeneralResult<>(false, "缺少拓扑参数");
            }
            if (null == sourceId) {
                return new GeneralResult<>(false, "缺少源模型参数");
            }
            if (null == targetId) {
                return new GeneralResult<>(false, "缺少目标模型参数");
            }
            ModuleRelation relation = moduleRelationRepository.query(relationCategoryId, sourceId, targetId);
            if (null != relation) {
                // 查询关联规则信息
                AutoRelationRule rule = autoRelationRuleRepository.queryByModuleRelationId(relation.getId());
                if (null != rule) {
                    List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(rule.getId());
                    for (AutoRelationRuleItem item : items) {
                        item.setValues(autoRelationRuleItemValueRepository.queryByItemId(item.getId()));
                    }
                    relation.setRuleItemList(items);
                }
            }
            return new GeneralResult<>(true, relation, "查询成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation data query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 关联类型分组id查询模型集合
     *
     * @param relationCategoryId
     * @return
     */
    @Override
    public GeneralResult<List<CiModule>> moduleList(Long relationCategoryId) {
        try {
            List<ModuleRelation> relations = moduleRelationRepository.queryModuleList(relationCategoryId);
            if (CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(true, new ArrayList<CiModule>(), "查询成功");
            }
            List<Long> sourceIds = relations.stream().map(ModuleRelation::getSourceId).collect(Collectors.toList());
            List<Long> targetIds = relations.stream().map(ModuleRelation::getTargetId).collect(Collectors.toList());
            // 去重取
            List<Long> moduleIds = Stream.of(sourceIds, targetIds).flatMap(Collection::stream).filter(Objects::nonNull)
                .distinct().collect(Collectors.toList());
            // 获取模型集合
            List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
            return new GeneralResult<>(true, modules, "查询成功");
        } catch (Exception e) {
            log.error("CmdbModuleRelation modules query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult instances(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            pager.getSorter().put("id", Common.ONE);
            Long id = null;
            Long moduleId = null;
            Long instanceId = null;
            // 实例属性条件查询
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("id")) {
                    if (!StringUtils.isEmpty(paramMap.get("id").toString())) {
                        id = Long.valueOf(paramMap.get("id").toString());
                    }
                    paramMap.remove("id");
                }
                if (paramMap.containsKey("moduleId")) {
                    if (!StringUtils.isEmpty(paramMap.get("moduleId").toString())) {
                        moduleId = Long.valueOf(paramMap.get("moduleId").toString());
                    }
                }
                if (paramMap.containsKey("instanceId")) {
                    if (!StringUtils.isEmpty(paramMap.get("instanceId").toString())) {
                        instanceId = Long.valueOf(paramMap.get("instanceId").toString());
                    }
                    paramMap.remove("instanceId");
                }
            }
            ModuleRelation moduleRelation = moduleRelationRepository.query(id);
            if (null == moduleRelation) {
                return GeneralResult.FAILED("模型关系不存在");
            }
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return GeneralResult.FAILED("模型不存在");
            }
            // 被关联的模型
            String finalModuleCode = null;
            Long finalModuleId = null;
            // 实例类型：目标实例、源实例
            String finalInstanceType = null;
            // 取非当前moduleId对应模型的模型,即若moduleId为源，则查询目标；
            if (moduleId.equals(moduleRelation.getSourceId())) {
                finalModuleCode = moduleRelation.getTargetCode();
                finalModuleId = moduleRelation.getTargetId();
                finalInstanceType = CmdbUtils.INSTANCE_RELATION_TYPE_TARGET;
            } else if (moduleId.equals(moduleRelation.getTargetId())) {
                finalModuleCode = moduleRelation.getSourceCode();
                finalModuleId = moduleRelation.getSourceId();
                finalInstanceType = CmdbUtils.INSTANCE_RELATION_TYPE_SOURCE;
            }

            if (StringUtils.isEmpty(finalModuleCode) || null == finalModuleId
                || StringUtils.isEmpty(moduleRelation.getRelationCategoryCode())) {
                return GeneralResult.FAILED("模型关系不完整");
            }
            // 查询目标模型的实例数据
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("moduleId")) {
                    paramMap.put("moduleId", finalModuleId);
                }
            }
            String instanceTable = CmdbUtils.CMDB_INSTANCE + finalModuleCode.toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + finalModuleCode.toLowerCase();
            String relationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
            // 查询已关联实例
            List<InstanceRelation> relations = null;
            Map<Long, InstanceRelation> relationMap = new HashMap<>();
            if (CmdbUtils.INSTANCE_RELATION_TYPE_SOURCE.equals(finalInstanceType)) {
                relations = instanceRelationRepository.query(relationTable, finalModuleId, null, moduleId, instanceId);
                for (InstanceRelation relation : relations) {
                    relationMap.put(relation.getSourceInstanceId(), relation);
                }
            } else if (CmdbUtils.INSTANCE_RELATION_TYPE_TARGET.equals(finalInstanceType)) {
                relations = instanceRelationRepository.query(relationTable, moduleId, instanceId, finalModuleId, null);
                for (InstanceRelation relation : relations) {
                    relationMap.put(relation.getTargetInstanceId(), relation);
                }
            }

            boolean hasAdminRole = instanceService.checkAdminRole(userId);
            // 查询可被关联的模型实例
            List<Map<String, Object>> list =
                instanceRepository.list(pager, instanceTable, valueTable, userId, hasAdminRole);
            int total = instanceRepository.count(pager.getParams(), instanceTable, valueTable, userId, hasAdminRole);
            instanceService.convertInstanceValue(list, moduleId, userId, hasAdminRole);
            for (Map<String, Object> instance : list) {
                // 判断被关联的模型实例是否已关联
                InstanceRelation existRelation = relationMap.get(instance.get("id"));
                instance.put("isRelation", null != existRelation);
                instance.put("instanceRelation", existRelation);// 存放关联实例的关联关系，配合isRelation一起使用，取消关联需要这个关系的ID
            }
            GridBean<Map<String, Object>> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            return new GeneralResult<>(false, "查询模型实例失败");
        }
    }

}
