package com.bocloud.ams.service.module.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.Relation;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.entity.module.TopologyConfig;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.RelationCategoryRepository;
import com.bocloud.ams.repository.module.RelationRepository;
import com.bocloud.ams.repository.module.TopologyConfigRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceRelationService;
import com.bocloud.ams.service.module.RelationCategoryService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Description: 关联类型分组管理
 * @Title: RelationCategoryServiceImpl
 * @Author: majiali
 * @CreateTime: 2022/5/7 16:59
 */
@Slf4j
@Service("relationCategoryService")
public class RelationCategoryServiceImpl implements RelationCategoryService {

    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private RelationRepository relationRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceRelationService instanceRelationService;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private TopologyConfigRepository topologyConfigRepository;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;

    @Override
    public GeneralResult create(RelationCategory relationCategory, Long userId) {
        try {

            if (judgeNameExist(relationCategory.getName())) {
                return new GeneralResult<>(false, "关联类型分组名称已经存在");
            }
            if (judgeCodeExist(relationCategory.getCode())) {
                return new GeneralResult<>(false, "关联类型分组编码已经存在");
            }
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase();
            if (instanceRelationRepository.countTable(relationTable) > 0) {
                log.error("创建CI实例关系表{}失败", relationTable);
                return new GeneralResult<>(false, "创建失败，" + relationTable + "实例关系表已存");
            }
            relationCategory.setDeleted(false);
            relationCategory.setCreatorId(userId);
            relationCategory.setMenderId(userId);
            relationCategory.setGmtModify(new Date());
            relationCategory.setGmtCreate(new Date());
            relationCategoryRepository.save(relationCategory);
            // 生成CI实例关系表
            GeneralResult<Void> result =
                instanceRelationService.createTable(relationCategory.getCode().toLowerCase(), userId);
            if (result.isFailed()) {
                return result;
            }
            return new GeneralResult<>(true, "关联类型分组新增成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory create fail:", e);
            return new GeneralResult<>(false, "关联类型分组新增失败");
        }
    }

    private boolean judgeCodeExist(String code) throws Exception {
        boolean flag = false;
        RelationCategory temp = relationCategoryRepository.queryByCode(code);
        if (null != temp) {
            flag = true;
        }
        return flag;
    }

    private boolean judgeNameExist(String name) throws Exception {
        boolean flag = false;
        RelationCategory temp = relationCategoryRepository.queryByName(name);
        if (null != temp) {
            flag = true;
        }
        return flag;
    }

    @Override
    public GeneralResult modify(RelationCategory relationCategory, Long userId) {
        String path = RelationCategoryServiceImpl.class.getSimpleName() + "_" + relationCategory.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RelationCategory temp = relationCategoryRepository.query(relationCategory.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "关联类型分组不存在");
            }
            RelationCategory tempByName = relationCategoryRepository.queryByName(relationCategory.getName());
            if(null != tempByName && !tempByName.getId().equals(relationCategory.getId())){
                return new GeneralResult<>(false, "关联类型分组名称已经存在");
            }
            LocaleBeanUtils.copyProperties(temp, relationCategory);
            temp.setGmtModify(new Date());
            temp.setMenderId(userId);
            relationCategoryRepository.update(temp);
            return new GeneralResult<>(true, "关联类型分组修改成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory modify fail:", e);
            return new GeneralResult<>(false, "关联类型分组修改失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, Long userId) {
        String path = RelationCategoryServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RelationCategory temp = relationCategoryRepository.query(id);
            if (null == temp) {
                return new GeneralResult<>(false, "关联类型分组不存在");
            }
            List<Relation> relationList = relationRepository.queryByCategoryId(id);
            if (CollectionUtils.isNotEmpty(relationList)) {
                return new GeneralResult<>(false, "该分组下有内容，不可删除");
            }
            final List<TopologyConfig> topologyConfigs = topologyConfigRepository.queryListBycategoryId(id);
            if (CollectionUtils.isNotEmpty(topologyConfigs)) {
                return new GeneralResult<>(false, "该分组已被实例拓扑使用，不可删除！");
            }
            relationCategoryRepository.delete(id, userId);
            // 删除CI资源关系表
            GeneralResult<Void> result = instanceRelationService.dropTable(temp.getCode().toLowerCase(), userId);
            if (result.isFailed()) {
                return result;
            }
            return new GeneralResult<>(true, "关联类型分组删除成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory remove fail:", e);
            return new GeneralResult<>(false, "关联类型分组删除失败");
        }
    }

    @Override
    public GeneralResult list(Pager pager) {
        try {
            GridBean gridBean;
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            Boolean simple = pager.getSimple();
            if (null == simple) {
                simple = false;
            }
            int total = relationCategoryRepository.count(pager.getParams());
            if (simple) {
                List<RelationCategory> beans = relationCategoryRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<RelationCategory> list = this.relationCategoryRepository.list(pager);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult listAll(List<Param> params, Map<String, String> sorter) {
        try {
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>());
            sorter.put(Common.ID, Common.ONE);
            List<RelationCategory> beans = relationCategoryRepository.list(params, sorter);
            return new GeneralResult<>(true, beans, "查询成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<ModuleRelation>> moduleRelationList(Long id, Long moduleId) {
        try {
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return GeneralResult.FAILED("模型数据不存在");
            }
            RelationCategory category = relationCategoryRepository.query(id);
            if (null == category) {
                return GeneralResult.FAILED("关联类型分组数据不存在");
            }
            // 查询模型关系
            List<ModuleRelation> relations = moduleRelationRepository.query(id, moduleId);
            return new GeneralResult<>(true, relations, "查询模型关系成功!");
        } catch (Exception e) {
            log.error("CmdbModuleRelation moduleRelationList query fail:", e);
            return new GeneralResult<>(false, "查询模型关系失败");
        }
    }

    @Override
    public GeneralResult listByModuleId(Long moduleId) {
        try {
            List<RelationCategory> list = relationCategoryRepository.listByModuleId(moduleId);
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("CmdbRelationCategory list query by moduleId fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
