package com.bocloud.ams.service.module.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.Relation;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.module.RelationCategoryRepository;
import com.bocloud.ams.repository.module.RelationRepository;
import com.bocloud.ams.service.module.RelationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Description: 关联类型管理
 * @Title: RelationServiceImpl
 * @Author: majiali
 * @CreateTime: 2022/5/7 16:59
 */
@Slf4j
@Service("relationService")
public class RelationServiceImpl implements RelationService {

    @Autowired
    private RelationRepository relationRepository;
    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;

    @Override
    public GeneralResult create(Relation relation, Long userId) {
        try {
            RelationCategory category = relationCategoryRepository.query(relation.getCategoryId());
            if (null == category) {
                return new GeneralResult<>(false, "关联类型分组不存在");
            }
            if (judgeCodeExist(relation.getCode(), relation.getCategoryId())) {
                return new GeneralResult<>(false, "关联类型编码已经存在");
            }
            relation.setDeleted(false);
            relation.setCreatorId(userId);
            relation.setMenderId(userId);
            relation.setGmtModify(new Date());
            relation.setGmtCreate(new Date());
            relationRepository.save(relation);
            return new GeneralResult<>(true, "关联类型新增成功");
        } catch (Exception e) {
            log.error("CmdbRelation create fail:", e);
            return new GeneralResult<>(false, "关联类型新增失败");
        }
    }

    private boolean judgeCodeExist(String code, Long categoryId) throws Exception {
        Relation temp = relationRepository.queryByCode(code, categoryId);
        return null != temp;
    }

    @Override
    public GeneralResult modify(Relation relation, Long userId) {
        String path = RelationServiceImpl.class.getSimpleName() + "_" + relation.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Relation temp = relationRepository.query(relation.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "关联类型不存在");
            }
            LocaleBeanUtils.copyProperties(temp, relation);
            temp.setGmtModify(new Date());
            temp.setMenderId(userId);
            relationRepository.update(temp);
            return new GeneralResult<>(true, "关联类型修改成功");
        } catch (Exception e) {
            log.error("CmdbRelation modify fail:", e);
            return new GeneralResult<>(false, "关联类型修改失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, Long userId) {
        String path = RelationServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Relation temp = relationRepository.query(id);
            if (null == temp) {
                return new GeneralResult<>(false, "关联类型不存在");
            }
            // 查询模型关系是否有使用该关系
            List<ModuleRelation> relations = moduleRelationRepository.queryByRelationId(id);
            if (!CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(false, "该关系在模型关系中被使用，不可删除");
            }
            relationRepository.delete(id, userId);
            return new GeneralResult<>(true, "关联类型删除成功");
        } catch (Exception e) {
            log.error("CmdbRelation remove fail:", e);
            return new GeneralResult<>(false, "关联类型删除失败");
        }
    }

    @Override
    public GeneralResult list(Pager pager) {
        try {
            GridBean gridBean;
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            Boolean simple = null == pager.getSimple() ? false : pager.getSimple();
            int total = relationRepository.count(pager.getParams());
            if (simple) {
                List<SimpleBean> beans = relationRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<Relation> list = this.relationRepository.list(pager);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Relation list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
