package com.bocloud.ams.service.module.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.ColumnAndSearchItemModel;
import com.bocloud.ams.entity.model.ColumnModel;
import com.bocloud.ams.entity.model.InstanceModel;
import com.bocloud.ams.entity.model.InstanceNodeModel;
import com.bocloud.ams.entity.model.InstanceRelationModel;
import com.bocloud.ams.entity.model.InstanceTopo;
import com.bocloud.ams.entity.model.PathModel;
import com.bocloud.ams.entity.model.RequestInstanceTopologyModel;
import com.bocloud.ams.entity.model.ResponseInstanceTopologyModel;
import com.bocloud.ams.entity.model.SearchConfigModel;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.PropertyItem;
import com.bocloud.ams.entity.module.PropertyPoolItem;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.entity.module.TopologyConfig;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.PropertyItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolItemRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.module.RelationCategoryRepository;
import com.bocloud.ams.repository.module.TopologyConfigRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.module.TopologyConfigService;
import com.bocloud.ams.service.utils.GraphAlgorithmUtils;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.bocloud.ams.service.utils.paginationUtil;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/12 17:41
 */
@Slf4j
@Service("topologyConfigService")
public class TopologyConfigServiceImpl implements TopologyConfigService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TopologyConfigRepository topologyConfigRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private PropertyItemRepository propertyItemRepository;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private PropertyPoolItemRepository propertyPoolItemRepository;
    @Autowired
    private InstanceRepository instanceRepository;

    @Override
    public GeneralResult<GridBean<TopologyConfig>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = topologyConfigRepository.count(pager.getParams());
            List<TopologyConfig> list = this.topologyConfigRepository.list(pager);
            for (TopologyConfig config : list) {
                RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
                if (null != relationCategory) {
                    config.setCategoryName(relationCategory.getName());
                }
                CiModule startModule = moduleRepository.query(config.getStartModuleId());
                if (null != startModule) {
                    config.setStartModuleName(startModule.getName());
                }
                if (StringUtils.isNotEmpty(config.getWayModuleIds())) {
                    config.setWayModuleId(Arrays.stream(config.getWayModuleIds().split(",")).map(Long::valueOf).collect(Collectors.toList()));
                }
                if (StringUtils.isNotEmpty(config.getIgnoreModuleIds())) {
                    config.setIgnoreModuleId(Arrays.stream(config.getIgnoreModuleIds().split(",")).map(Long::valueOf).collect(Collectors.toList()));
                }
                if (StringUtils.isNotEmpty(config.getTopologyPath())) {
                    List<Long> moduleIds = Arrays.stream(config.getTopologyPath().split(",")).map(Long::valueOf).collect(Collectors.toList());
                    List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
                    String topologyPathName = modules.stream().map(CiModule::getName).collect(Collectors.joining("-"));
                    config.setTopologyPathName(topologyPathName);
                }
            }
            GridBean<TopologyConfig> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> save(TopologyConfig config, Long target) {
        try {
            if (StringUtils.isEmpty(config.getConfigType())) {
                config.setConfigType("PATCH");
            }
            int sameName = topologyConfigRepository.countSameNameOrCode(config.getName(), null, config.getConfigType());
            if (sameName > 0) {
                return new GeneralResult<>(false, "添加失败，该拓扑名称名称已存在，请尝试其他名称");
            }
            if (CollectionUtils.isNotEmpty(config.getIgnoreModuleId())) {
                List<String> ignores = config.getIgnoreModuleId().stream().map(item -> String.valueOf(item)).collect(Collectors.toList());
                config.setIgnoreModuleIds(ignores.stream().collect(Collectors.joining(",")));
            }
            if (CollectionUtils.isNotEmpty(config.getWayModuleId())) {
                List<String> ways = config.getWayModuleId().stream().map(item -> String.valueOf(item)).collect(Collectors.toList());
                config.setWayModuleIds(ways.stream().collect(Collectors.joining(",")));
            }
            config.setGmtCreate(new Date());
            config.setGmtModify(new Date());
            config.setCreatorId(target);
            config.setMenderId(target);
            topologyConfigRepository.save(config);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("topologyConfigService save fail:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TopologyConfig config, Long target) {
        String path = TopologyConfig.class.getSimpleName() + "_" + config.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            int sameName = topologyConfigRepository.countSameNameOrCode(config.getName(), config.getId(), config.getConfigType());
            if (sameName > 0) {
                return new GeneralResult<>(false, "修改失败，该拓扑名称名称已存在，请尝试其他名称");
            }
            if (CollectionUtils.isNotEmpty(config.getIgnoreModuleId())) {
                List<String> ignores = config.getIgnoreModuleId().stream().map(item -> String.valueOf(item)).collect(Collectors.toList());
                config.setIgnoreModuleIds(ignores.stream().collect(Collectors.joining(",")));
            }
            if (CollectionUtils.isNotEmpty(config.getWayModuleId())) {
                List<String> ways = config.getWayModuleId().stream().map(item -> String.valueOf(item)).collect(Collectors.toList());
                config.setWayModuleIds(ways.stream().collect(Collectors.joining(",")));
            }
            config.setGmtModify(new Date());
            config.setMenderId(target);
            topologyConfigRepository.update(config);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("topologyConfigService modify fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long target) {
        String path = TopologyConfig.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "拓扑实例不存在");
            }
            topologyConfigRepository.delete(id, target);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("topologyConfigService remove fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<TopologyConfig> detail(Long id) {
        try {
            TopologyConfig config = topologyConfigRepository.query(id);
            if ("PATCH".equalsIgnoreCase(config.getConfigType())) {
                String message = this.escapeData(config);
                if (null != message) {
                    return new GeneralResult<>(false, message);
                }
            } else {
                RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
                if (null != relationCategory) {
                    config.setCategoryName(relationCategory.getName());
                }
                CiModule startModule = moduleRepository.query(config.getStartModuleId());
                if (null != startModule) {
                    config.setStartModuleName(startModule.getName());
                }
            }
            return new GeneralResult<>(true, config, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 根据起始模型查询最大拓扑层级
     * <p>
     * 本接口实现思路：在categoryId对应的拓扑图中，获取以startModuleId作为头结点或者是终点的所有路径中，长度最大的那条路径的长度。
     * 具体实现：
     * startModuleId作为头结点的路径: 获取以startModuleId为sourceId的数据所以应的树，然后获取这棵树所有路径集合。
     * startModuleId作为终点的路径: 获取以startModuleId为targetId的数据所以应的树，然后获取这棵树所有路径集合。
     * 获取两个路径集合中长度最大的路径的长度，作为为此拓扑模型中以startModuleId为源的最大的拓扑层级。
     *
     * @param categoryId
     * @param startModuleId
     * @return
     */
    @Override
    public GeneralResult<Integer> maxLevel(Long categoryId, Long startModuleId) {
        RequestInstanceTopologyModel topologyModel = new RequestInstanceTopologyModel();
        topologyModel.setCategoryId(categoryId);
        topologyModel.setStartModuleId(startModuleId);
        try {
            //获取起始模型为源id，对应的路径集合。
            GeneralResult<Map<Boolean, List<List<Long>>>> checkData = this.checkData(topologyModel);
            if (checkData.isFailed()) {
                return new GeneralResult<>(false, checkData.getMessage());
            }
            //校验数据
            List<List<Long>> lists = new ArrayList<>();
            Map<Boolean, List<List<Long>>> data = checkData.getData();
            for (Boolean aBoolean : data.keySet()) {
                if (null != data.get(aBoolean)) {
                    lists.addAll(data.get(aBoolean));
                }
            }
            //移除集合中每个元素中所有为null的数据
            if (CollectionUtils.isEmpty(lists)) {
                return new GeneralResult<>(true, 1, "查询最大层级成功");
            }
            Integer maxSize = lists.stream().mapToInt(item -> item.size()).max().getAsInt();
            return new GeneralResult<>(true, maxSize, "查询最大层级成功");
        } catch (Exception e) {
            log.error("topologyConfigService maxLevel fail:", e);
            return new GeneralResult<>(false, "查询最大层级失败");
        }
    }

    /**
     * 获取拓扑路径
     * <p>
     * 接口实现思路：
     *
     * @param categoryId     拓扑图id
     * @param startModuleId  起始模型id
     * @param topologyLevel  拓扑层级
     * @param ignoreModuleId 忽略的拓扑
     * @param wayModuleId
     * @return
     */
    @Override
    public GeneralResult<List<PathModel>> topologyPath(Long categoryId, Long startModuleId, Integer topologyLevel, String ignoreModuleId, String wayModuleId) {
        try {
            RequestInstanceTopologyModel topologyModel = new RequestInstanceTopologyModel();
            topologyModel.setCategoryId(categoryId);
            topologyModel.setStartModuleId(startModuleId);
            topologyModel.setTopologyLevel(topologyLevel);
            if (StringUtils.isNotEmpty(ignoreModuleId)) {
                List<Long> ignoreModuleIds = Arrays.asList(ignoreModuleId.split(",")).stream().map(item -> Long.valueOf(item)).collect(Collectors.toList());
                topologyModel.setIgnoreModuleId(ignoreModuleIds);
            }
            if (StringUtils.isNotEmpty(wayModuleId)) {
                List<Long> wayModuleIds = Arrays.asList(wayModuleId.split(",")).stream().map(item -> Long.valueOf(item)).collect(Collectors.toList());
                topologyModel.setWayModuleId(wayModuleIds);
            }
            List<PathModel> pathModels = new ArrayList<>();
            //获取起始模型为源id，对应的路径集合
            GeneralResult<Map<Boolean, List<List<Long>>>> mapGeneralResult = this.checkData(topologyModel);
            if (mapGeneralResult.isFailed()) {
                return new GeneralResult<>(false, mapGeneralResult.getMessage());
            }
            // 转化成返回的数据
            List<CiModule> moduleList = moduleRepository.all();
            Map<Boolean, List<List<Long>>> data = mapGeneralResult.getData();
            for (Boolean aBoolean : data.keySet()) {
                if (CollectionUtils.isNotEmpty(data.get(aBoolean))) {
                    // 获取拓扑路径（根据 忽略模型、必经模型、路径层级 等条件）
                    this.getPathModelList(data.get(aBoolean), moduleList, topologyLevel, pathModels, aBoolean, topologyModel);
                }
            }
            //去重
            List<PathModel> pathModelList = pathModels.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(PathModel::getTopologyPath))), ArrayList::new));
            return new GeneralResult<>(true, pathModelList, "查询拓扑路径成功");
        } catch (Exception e) {
            log.error("topologyConfigService topologyPath fail:", e);
            return new GeneralResult<>(false, "查询拓扑路径失败");
        }
    }

    /**
     * 获取查询项以及列表项
     * 功能：1：获取搜索项以及搜索项下拉框的值。2：动态获取表格的列数据
     *
     * @param id
     * @return
     */
    @Override
    public GeneralResult<ColumnAndSearchItemModel> searchItemList(Long id) {
        try {
            ColumnAndSearchItemModel itemModel = new ColumnAndSearchItemModel();
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            // 获取拓扑路径模型id
            // 将拓扑路径模型id类型转成Long类型
            List<Long> moduleIds = Arrays.asList(config.getTopologyPath().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
            List<SearchConfigModel> searchConfigs = new ArrayList<>();
            // 根据模型id查询模型信息
            List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
            if (CollectionUtils.isEmpty(modules)) {
                return new GeneralResult<>(false, "没有对应的模型数据");
            }
            // 1：处理搜索条件（搜索框返回的顺序要和拓扑路径中模型的顺序一致）
            for (Long moduleId : moduleIds) {
                CiModule module = modules.stream().filter(item -> item.getId().equals(moduleId)).findFirst().orElse(null);
                if (null == module) {
                    continue;
                }
                searchConfigs.add(new SearchConfigModel(moduleId, module.getName(), module.getCode(), "Select"));
            }
            itemModel.setSearchConfigs(searchConfigs);
            // 2：处理搜索条件的下拉框的值
            GeneralResult<List<List<InstanceModel>>> resourceData = this.getResourceData(config, moduleIds);
            if (resourceData.isFailed()) {
                return new GeneralResult<>(false, resourceData.getMessage());
            }
            //获取拓扑路径对应的资源数据
            List<List<InstanceModel>> levelRelationList = resourceData.getData();
            List<ColumnModel> columns = new ArrayList<>();
            // 获取 模型id 找到 各个模型的实例集合
            for (SearchConfigModel searchConfig : itemModel.getSearchConfigs()) {
                ColumnModel columnModel = new ColumnModel();
                if (CollectionUtils.isNotEmpty(levelRelationList)) {
                    List<Map<String, Object>> data = new ArrayList<>();
                    //获取当前搜索项对应的资源数据并去重得到对应的下拉框数据
                    levelRelationList.stream().flatMap(instanceModels -> instanceModels.stream()).filter(item -> item.getModuleId().equals(searchConfig.getModuleId())).collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(InstanceModel::getInstanceId))), ArrayList::new)).forEach(item -> {
                        Map<String, Object> map = new HashMap<>();
                        map.put("id", item.getInstanceId());
                        map.put("name", item.getInstanceName());
                        data.add(map);
                    });
                    searchConfig.setData(data);
                }
                columnModel.setLabel(searchConfig.getLabel());
                columnModel.setProp(searchConfig.getValue());
                columns.add(columnModel);
            }
            // 3：处理列表集合
            List<Property> properties = propertyRepository.listIsTitleByModuleId(moduleIds.get(moduleIds.size() - 1));
            List<Property> finalProperties = properties.stream().filter(item -> !item.getCode().equals("name")).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(finalProperties)) {
                for (Property property : finalProperties) {
                    columns.add(new ColumnModel(property.getName(), property.getCode()));
                }
            }
            itemModel.setColumns(columns);
            return new GeneralResult<>(true, itemModel, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService searchItemList fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 查询资源列表
     *
     * @param pager
     * @return
     */
    @Override
    public GeneralResult<GridBean<Map<String, Object>>> resourcelist(Pager pager) {
        try {
            GridBean<Map<String, Object>> gridBean = null;
            // 编辑返回数据格式
            List<Map<String, Object>> lists = new ArrayList<>();
            if (CollectionUtils.isEmpty(pager.getParams()) || MapUtils.isEmpty(pager.getParams().get(0).getParam()) || null == pager.getParams().get(0).getParam().get("id")) {
                return new GeneralResult<>(false, "id 不能为空");
            }
            String id = pager.getParams().get(0).getParam().get("id").toString();
            TopologyConfig config = topologyConfigRepository.query(Long.valueOf(id));
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            // 将拓扑路径模型id类型转成Long类型
            List<Long> moduleIds = Arrays.asList(config.getTopologyPath().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
            GeneralResult<List<List<InstanceModel>>> resourceData = this.getResourceData(config, moduleIds);
            if (resourceData.isFailed()) {
                return new GeneralResult<>(false, resourceData.getMessage());
            } else if (resourceData.isSuccess()) {
                if (CollectionUtils.isEmpty(resourceData.getData())) {
                    gridBean = new GridBean<>(pager.getPage(), pager.getRows(), 0, lists);
                    return new GeneralResult<>(true, gridBean, "查询成功");
                }
            }
            //获取拓扑路径对应的资源数据
            List<List<InstanceModel>> levelRelationList = resourceData.getData();
            // 通过pager传过来的搜索参数过滤数据
            List<List<InstanceModel>> relationModelList = this.pagerFilter(levelRelationList, pager);
            if (CollectionUtils.isEmpty(relationModelList)) {
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), 0, lists);
                return new GeneralResult<>(true, gridBean, "查询成功");
            }
            // 此时先做分页
            Map<String, Object> pagination = paginationUtil.pagination(relationModelList, pager.getPage(), pager.getRows());
            // 获取分页后的数据
            List<List<InstanceModel>> paginations = (List<List<InstanceModel>>) pagination.get("list");
            // 获取每行数据中最后那个实例的实例id的集合。
            List<Long> lastInstanceIds = new ArrayList<>();
            for (List<InstanceModel> links : paginations) {
                InstanceModel relationModel = links.get(links.size() - 1);
                lastInstanceIds.add(relationModel.getInstanceId());
            }
            // 获取模型code 从而才能找到对应的实例表
            String lastModuleCode = paginations.get(0).get(paginations.get(0).size() - 1).getModuleCode();
            // 拓扑路径最后一个模型的实例表是
            String lastInstanceTable = CmdbUtils.CMDB_INSTANCE_VALUE + lastModuleCode.toLowerCase();
            // 根据lastInstanceIds以及模型属性为列表项的propertyIds 查询 lastInstanceTable表中的数据
            List<Property> properties = propertyRepository.listIsTitleByModuleId(moduleIds.get(moduleIds.size() - 1));
            //获取需要转义的属性数据
            List<Property> propertyList = properties.stream().filter(item -> item.getType().equals(PropertyConstant.PropertyType.SINGLESELECT) || item.getType().equals(PropertyConstant.PropertyType.MULTISELECT) || item.getType().equals(PropertyConstant.PropertyType.RADIO) || item.getType().equals(PropertyConstant.PropertyType.CHECKBOX)).collect(Collectors.toList());
            List<Long> propertyIds = properties.stream().filter(item -> !item.getCode().equals("name")).map(Property::getId).collect(Collectors.toList());

            // 获取指定实例的指定属性的数据列表
            List<InstanceValue> instanceValues = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(propertyIds)) {
                instanceValues = instanceValueRepository.queryByInstanceIdsAndPropertyIds(lastInstanceTable, lastInstanceIds, propertyIds);
            }
            // 对每行数据进行循环
            for (List<InstanceModel> links : paginations) {
                Map<String, Object> map = new HashMap<>();
                for (InstanceModel link : links) {
                    // 对于模型name使用对应的模型code作为key值
                    map.put(link.getModuleCode(), link.getInstanceName());
                    map.put(link.getModuleCode() + "_instanceId", link.getInstanceId());
                    map.put(link.getModuleCode() + "_moduleId", link.getModuleId());
                }
                // 获取每行中剩余属性数据
                if (CollectionUtils.isNotEmpty(instanceValues)) {
                    List<InstanceValue> values = instanceValues.stream().filter(item -> item.getInstanceId().equals(links.get(links.size() - 1).getInstanceId())).collect(Collectors.toList());
                    for (InstanceValue value : values) {
                        Property property = propertyList.stream().filter(item -> item.getId().equals(value.getPropertyId())).findFirst().orElse(null);
                        //if内是转义
                        if (null != property) {
                            List<String> listString = Arrays.asList(value.getPropertyValue().replace("[", "").replace("]", "").replace("\"", "").split(","));
                            if (CollectionUtils.isEmpty(listString)) {
                                continue;
                            }
                            List<String> list = new ArrayList<>();
                            List<PropertyItem> propertyItems = new ArrayList<>();
                            if (null != property.getPropertyPoolId()) {
                                List<PropertyPoolItem> propertyPoolItems = propertyPoolItemRepository.listByPropertyPoolId(property.getPropertyPoolId());
                                BeanUtils.copyProperties(propertyPoolItems, propertyItems);
                            } else {
                                propertyItems = propertyItemRepository.queryByPropertyId(property.getId());
                            }
                            if (CollectionUtils.isNotEmpty(propertyItems)) {
                                for (String s : listString) {
                                    PropertyItem propertyItem = propertyItems.stream().filter(item -> item.getValue().equals(s)).findFirst().orElse(null);
                                    if (null != propertyItem) {
                                        list.add(propertyItem.getName());
                                    }
                                }
                            }
                            map.put(value.getPropertyCode(), StringUtils.join(list, ','));
                        } else {
                            map.put(value.getPropertyCode(), value.getPropertyValue());
                        }
                    }
                }
                lists.add(map);
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), (int) pagination.get("totalCount"), lists);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService resourcelist fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 获取拓扑路径下的资源列表
     *
     * @param config
     * @param moduleIds
     * @return
     */
    private GeneralResult<List<List<InstanceModel>>> getResourceData(TopologyConfig config, List<Long> moduleIds) {
        try {
            RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
            if (null == relationCategory) {
                return new GeneralResult<>(false, "没有对应的拓扑分组");
            }
            List<InstanceRelationModel> mapList = new ArrayList<>();
            for (int i = 0; i < moduleIds.size() - 1; i++) {
                InstanceRelationModel instanceRelationModel = new InstanceRelationModel();
                instanceRelationModel.setSourceModuleId(moduleIds.get(i));
                instanceRelationModel.setTargetModuleId(moduleIds.get(i + 1));
                mapList.add(instanceRelationModel);
            }
            // 得到要查询的表名
            String relationTable = "ci_instance_relation_" + relationCategory.getCode().toLowerCase();
            // 通过 in（通过拓扑路径中的模型id）这样才能过滤出当前拓扑路径上的所有有关系的数据
            List<InstanceRelation> allRelations = new ArrayList<>();
            for (InstanceRelationModel instanceRelationModel : mapList) {
                List<InstanceRelation> instanceRelationsForward = instanceRelationRepository.queryByModuleIds(relationTable, instanceRelationModel.getSourceModuleId(), instanceRelationModel.getTargetModuleId());
                if (CollectionUtils.isNotEmpty(instanceRelationsForward)) {
                    allRelations.addAll(instanceRelationsForward);
                }
                List<InstanceRelation> instanceRelationsReverse = instanceRelationRepository.queryByModuleIds(relationTable, instanceRelationModel.getTargetModuleId(), instanceRelationModel.getSourceModuleId());
                if (CollectionUtils.isNotEmpty(instanceRelationsReverse)) {
                    allRelations.addAll(instanceRelationsReverse);
                }
            }
            if (CollectionUtils.isEmpty(allRelations)) {
                return new GeneralResult<>(true, null, "查询成功");
            }
            // startRelations 是 根节点的数据集合。如果是正向的就取sourceid并去重，如果是反向就取targetid并去重 。后面生成的树就是以这些节点为根节点生成树的
            List<InstanceRelation> startRelations = new ArrayList<>();
            if (config.getPathDirection()) {
                // 拓扑路径和模型关系是相反的的即反向操作--获取树的根节点
                startRelations = allRelations.stream().filter(item -> config.getStartModuleId().equals(item.getTargetModuleId())).collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(InstanceRelation::getTargetInstanceId))), ArrayList::new));
            } else {
                // 拓扑路径和模型关系是一致的即正向操作--获取树的根节点
                startRelations = allRelations.stream().filter(item -> config.getStartModuleId().equals(item.getSourceModuleId())).collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(InstanceRelation::getSourceInstanceId))), ArrayList::new));
            }
            // 生成森林
            List<InstanceRelation> trees = this.getTrees(startRelations, allRelations, moduleIds, config.getPathDirection());
            // 根据森林生成路径
            List<List<InstanceModel>> instancePaths = new ArrayList<>();
            for (InstanceRelation instanceRelation : trees) {
                List<List<InstanceModel>> paths = GraphAlgorithmUtils.resourceInstanceAlgorithm(instanceRelation, config.getPathDirection());
                instancePaths.addAll(paths);
            }
            // 保证路径中节点数为拓扑指定的拓扑层级
            List<List<InstanceModel>> levelRelationList = instancePaths.stream().filter(item -> item.size() >= config.getTopologyLevel()).collect(Collectors.toList());
            return new GeneralResult<>(true, levelRelationList, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService getResourceData fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private List<InstanceRelation> getTrees(List<InstanceRelation> startRelations, List<InstanceRelation> allRelations, List<Long> moduleIds, Boolean pathDirection) {
        List<InstanceRelation> relations = new ArrayList<>();
        // 循环的是根节点
        for (InstanceRelation startRelation : startRelations) {
            InstanceRelation instanceRelation = new InstanceRelation();
            int i = 0;
            List<InstanceRelation> nextRelations = new ArrayList<>();
            if (pathDirection) {
                // 拓扑路径和模型关系是相反的即反向操作---判断根节点下有没有对应的子节点
                nextRelations = allRelations.stream().filter(item -> item.getTargetModuleId().equals(moduleIds.get(i)) && item.getTargetInstanceId().equals(startRelation.getTargetInstanceId())).collect(Collectors.toList());
            } else {
                // 拓扑路径和模型关系是一致的即正向操作--判断根节点下有没有对应的子节点
                nextRelations = allRelations.stream().filter(item -> item.getSourceModuleId().equals(moduleIds.get(i)) && item.getSourceInstanceId().equals(startRelation.getSourceInstanceId())).collect(Collectors.toList());
            }
            if (CollectionUtils.isNotEmpty(nextRelations)) {
                // 从这到this.getTreeByRecursion方法之前是设置的根节点的信息
                if (pathDirection) {
                    // 拓扑路径和模型关系是相反的即反向操作
                    instanceRelation.setSourceInstanceId(startRelation.getTargetInstanceId());
                    instanceRelation.setSourceInstanceName(startRelation.getTargetInstanceName());
                    instanceRelation.setSourceModuleCode(startRelation.getTargetModuleCode());
                    instanceRelation.setSourceModuleId(startRelation.getTargetModuleId());
                    instanceRelation.setTargetInstanceId(startRelation.getSourceInstanceId());
                    instanceRelation.setTargetInstanceName(startRelation.getSourceInstanceName());
                    instanceRelation.setTargetModuleCode(startRelation.getSourceModuleCode());
                    instanceRelation.setTargetModuleId(startRelation.getSourceModuleId());
                } else {
                    // 拓扑路径和模型关系是一致的即正向操作
                    instanceRelation.setSourceInstanceId(startRelation.getSourceInstanceId());
                    instanceRelation.setSourceInstanceName(startRelation.getSourceInstanceName());
                    instanceRelation.setSourceModuleCode(startRelation.getSourceModuleCode());
                    instanceRelation.setSourceModuleId(startRelation.getSourceModuleId());
                    instanceRelation.setTargetInstanceId(startRelation.getTargetInstanceId());
                    instanceRelation.setTargetInstanceName(startRelation.getTargetInstanceName());
                    instanceRelation.setTargetModuleCode(startRelation.getTargetModuleCode());
                    instanceRelation.setTargetModuleId(startRelation.getTargetModuleId());
                }
                // 从这个开始是递归每个根节点的信息并生成树
                this.getTreeByRecursion(nextRelations, allRelations, moduleIds, i + 1, pathDirection);
                instanceRelation.setCmdbInstanceRelations(nextRelations);
            }
            relations.add(instanceRelation);
        }
        return relations;
    }

    private void getTreeByRecursion(List<InstanceRelation> startRelations, List<InstanceRelation> allRelations, List<Long> moduleIds, int i, Boolean pathDirection) {
        for (InstanceRelation startRelation : startRelations) {
            List<InstanceRelation> nextRelations = new ArrayList<>();
            if (pathDirection) {
                // 拓扑路径和模型关系是相反的即反向操作
                nextRelations = allRelations.stream().filter(item -> item.getTargetModuleId().equals(moduleIds.get(i)) && item.getTargetInstanceId().equals(startRelation.getSourceInstanceId())).collect(Collectors.toList());
            } else {
                // 拓扑路径和模型关系是一致的即正向操作
                nextRelations = allRelations.stream().filter(item -> item.getSourceModuleId().equals(moduleIds.get(i)) && item.getSourceInstanceId().equals(startRelation.getTargetInstanceId())).collect(Collectors.toList());
            }
            if (CollectionUtils.isEmpty(nextRelations)) {
                continue;
            }
            startRelation.setCmdbInstanceRelations(nextRelations);
            int j = i + 1;
            if (j < moduleIds.size()) {
                this.getTreeByRecursion(nextRelations, allRelations, moduleIds, j, pathDirection);
            }
        }
    }

    /**
     * @param id
     * @return
     */
    @Override
    public GeneralResult<List<ResponseInstanceTopologyModel>> startModuleInstanceList(Long id) {
        try {
            List<ResponseInstanceTopologyModel> topologyModels = new ArrayList<>();
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            CiModule module = moduleRepository.query(config.getStartModuleId());
            if (null == module) {
                return new GeneralResult<>(false, "没有对应的起始模型！");
            }
            // 将拓扑路径模型id类型转成Long类型
            List<Long> moduleIds = Arrays.asList(config.getTopologyPath().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
            GeneralResult<List<List<InstanceModel>>> resourceData = this.getResourceData(config, moduleIds);
            if (resourceData.isFailed()) {
                return new GeneralResult<>(false, resourceData.getMessage());
            }
            //获取拓扑路径对应的资源数据
            List<List<InstanceModel>> levelRelationList = resourceData.getData();
            if (CollectionUtils.isNotEmpty(levelRelationList)) {
                //第一个就是起始模型数据
                levelRelationList.stream().flatMap(item -> item.stream()).filter(instanceModel -> instanceModel.getModuleId().equals(config.getStartModuleId())).collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(InstanceModel::getInstanceId))), ArrayList::new)).forEach(item -> {
                    ResponseInstanceTopologyModel topologyModel = new ResponseInstanceTopologyModel();
                    topologyModel.setCategoryId(config.getCategoryId());
                    topologyModel.setStartModuleId(item.getModuleId());
                    topologyModel.setStartModuleCode(item.getModuleCode());
                    topologyModel.setInstanceId(item.getInstanceId());
                    topologyModel.setInstanceName(item.getInstanceName());
                    topologyModels.add(topologyModel);
                });
            }
            return new GeneralResult<>(true, topologyModels, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService startModuleInstanceList fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 文件下载
     * <p>
     * 1. 创建excel对应的实体对象
     * <p>
     * 2. 设置返回的 参数
     * <p>
     * 3. 直接写，这里注意，finish的时候会自动关闭OutputStream,当然你外面再关闭流问题不大
     */
    @Override
    public GeneralResult<Void> download(Long id, HttpServletResponse response) {
        try {
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            String fileName = URLEncoder.encode(config.getName(), "UTF-8").replaceAll("\\+", "%20") + System.currentTimeMillis();
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            GeneralResult<ColumnAndSearchItemModel> itemResult = this.searchItemList(id);
            if (itemResult.isFailed()) {
                return new GeneralResult<>(false, "下载失败");
            }
            ColumnAndSearchItemModel data = itemResult.getData();
            List<ColumnModel> columns = data.getColumns();
            // 获取列名
            List<List<String>> columnTest = this.columnData(columns);
            Pager pager = new Pager();
            List<Param> params = new ArrayList<>();
            Param param = new Param();
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("id", id);
            param.setSign(Sign.EQ);
            param.setParam(paramMap);
            params.add(param);
            pager.setParams(params);
            pager.setPage(1);
            pager.setRows(100000);
            GeneralResult<GridBean<Map<String, Object>>> resourceResult = this.resourcelist(pager);
            if (resourceResult.isFailed()) {
                return new GeneralResult<>(false, "下载失败");
            }
            List<Map<String, Object>> rows = resourceResult.getData().getRows();
            // 获取数据
            List<List<Object>> lists = tableData(columns, rows);
            EasyExcel.write(response.getOutputStream()).head(columnTest).sheet("模板").doWrite(lists);
            return null;
        } catch (Exception e) {
            log.error("download is fail:", e);
            return new GeneralResult<>(false, "下载失败");
        }
    }

    /**
     * 实例拓扑-全局拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    @Override
    public GeneralResult<InstanceTopo> topoList(Long id, Long instanceId) {
        try {
            InstanceTopo topo = new InstanceTopo();
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            CiModule startModule = moduleRepository.query(config.getStartModuleId());
            if (null == startModule) {
                return new GeneralResult<>(false, "没有对应的起始模型数据");
            }
            RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
            if (null == relationCategory) {
                return new GeneralResult<>(false, "没有对应的关系分组数据");
            }
            List<ModuleRelation> moduleRelations = moduleRelationRepository.queryModuleList(config.getCategoryId());
            if (CollectionUtils.isEmpty(moduleRelations)) {
                return new GeneralResult<>(false, "对应的拓扑图内没有模型关系数据");
            }
            Param param = new Param();
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("id", id);
            paramMap.put(startModule.getCode(), instanceId);
            param.setParam(paramMap);
            param.setSign(Sign.EQ);
            List<Param> params = new ArrayList<>();
            params.add(param);
            Pager pager = new Pager(1, Integer.MAX_VALUE, params, null);
            // 获取到实例数据
            GeneralResult<GridBean<Map<String, Object>>> result = this.resourcelist(pager);
            if (result.isFailed()) {
                return new GeneralResult<>(false, "查询失败");
            }
            if (CollectionUtils.isEmpty(result.getData().getRows())) {
                return new GeneralResult<>(true, topo, "查询成功");
            }
            // 获取实例数据
            List<Map<String, Object>> datas = result.getData().getRows();
            // 将拓扑路径模型id类型转成Long类型
            List<Long> moduleIds = Arrays.asList(config.getTopologyPath().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
            // 根据模型id查询模型信息
            List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
            if (CollectionUtils.isEmpty(modules)) {
                return new GeneralResult<>(false, "没有对应的模型数据");
            }
            // instancenodes中每条数据实际上在instance_relation_{code}表中都有一条数据与之对应
            List<InstanceNodeModel> instancenodes = new ArrayList<>();
            Set<Long> sourceInstanceIds = new HashSet<>();
            Set<Long> targetInstanceIds = new HashSet<>();
            // 循环每行数据
            for (Map<String, Object> map : datas) {
                // 循环拓扑路径
                for (int i = 0; i < modules.size() - 1; i++) {
                    InstanceNodeModel nodeModel = new InstanceNodeModel();
                    nodeModel.setSourceModuleId(Long.valueOf(String.valueOf(map.get(modules.get(i).getCode() + "_moduleId"))));
                    nodeModel.setTargetModuleId(Long.valueOf(String.valueOf(map.get(modules.get(i + 1).getCode() + "_moduleId"))));
                    nodeModel.setSourceInstanceId(Long.valueOf(String.valueOf(map.get(modules.get(i).getCode() + "_instanceId"))));
                    nodeModel.setTargetInstanceId(Long.valueOf(String.valueOf(map.get(modules.get(i + 1).getCode() + "_instanceId"))));
                    // 去重-instancenodes集合中不存在才添加进去
                    List<InstanceNodeModel> nodeModels = instancenodes.stream().filter(item -> item.getSourceInstanceId().equals(nodeModel.getSourceInstanceId()) && item.getTargetInstanceId().equals(nodeModel.getTargetInstanceId()) && item.getSourceModuleId().equals(nodeModel.getSourceModuleId()) && item.getTargetModuleId().equals(nodeModel.getTargetModuleId())).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(nodeModels)) {
                        instancenodes.add(nodeModel);
                        sourceInstanceIds.add(nodeModel.getSourceInstanceId());
                        targetInstanceIds.add(nodeModel.getTargetInstanceId());
                    }
                }
            }
            // 得到要查询的模型实例关系表名
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase();
            // 通过源实例id的集合和目的实例id的集合以及拓扑路径中模型id的集合 查询出大概范围的数据
            List<InstanceRelation> instanceRelations = new ArrayList<>();
            if (config.getPathDirection()) {
                instanceRelations = instanceRelationRepository.listReverse(relationTable, config.getCategoryId(), new ArrayList<>(targetInstanceIds), new ArrayList<>(sourceInstanceIds));
            } else {
                instanceRelations = instanceRelationRepository.listForward(relationTable, config.getCategoryId(), new ArrayList<>(sourceInstanceIds), new ArrayList<>(targetInstanceIds));
            }
            List<InstanceRelationModel> links = new ArrayList<>();
            List<InstanceModel> nodes = new ArrayList<>();
            // java 处理精准匹配
            for (InstanceNodeModel nodeModel : instancenodes) {
                InstanceRelation instanceRelation = new InstanceRelation();
                if (config.getPathDirection()) {
                    instanceRelation = instanceRelations.stream().filter(item -> item.getSourceInstanceId().equals(nodeModel.getTargetInstanceId()) && item.getTargetInstanceId().equals(nodeModel.getSourceInstanceId()) && item.getSourceModuleId().equals(nodeModel.getTargetModuleId()) && item.getTargetModuleId().equals(nodeModel.getSourceModuleId())).findFirst().orElse(null);
                } else {
                    instanceRelation = instanceRelations.stream().filter(item -> item.getSourceInstanceId().equals(nodeModel.getSourceInstanceId()) && item.getTargetInstanceId().equals(nodeModel.getTargetInstanceId()) && item.getSourceModuleId().equals(nodeModel.getSourceModuleId()) && item.getTargetModuleId().equals(nodeModel.getTargetModuleId())).findFirst().orElse(null);
                }
                if (null != instanceRelation) {
                    InstanceRelationModel link = new InstanceRelationModel();
                    Long moduleRelationId = instanceRelation.getModuleRelationId();
                    ModuleRelation moduleRelation = moduleRelations.stream().filter(item -> item.getId().equals(moduleRelationId)).findFirst().orElse(null);
                    this.buildInstanceRelationModel(instanceRelation, link, moduleRelation);
                    links.add(link);
                    this.buildNodes(instanceRelation, nodes, modules);
                }
            }
            topo.setNodes(nodes);
            topo.setLinks(links);
            return new GeneralResult<>(true, topo, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService topoList fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<InstanceTopo> topology(Long id, Long instanceId) {
        try {
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            CiModule startModule = moduleRepository.query(config.getStartModuleId());
            if (null == startModule) {
                return new GeneralResult<>(false, "没有对应的起始模型数据");
            }
            RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
            if (null == relationCategory) {
                return new GeneralResult<>(false, "没有对应的关系分组数据");
            }
            List<ModuleRelation> moduleRelations = moduleRelationRepository.queryModuleRealtionAndModuleImage(config.getCategoryId());
            if (CollectionUtils.isEmpty(moduleRelations)) {
                return new GeneralResult<>(false, "对应的拓扑图内没有模型关系数据");
            }
            //根据模型id去重
            ArrayList<ModuleRelation> moduleRelationList = moduleRelations.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ModuleRelation::getModuleId))), ArrayList::new));
            //整理模型图标
            Map<Long, ModuleRelation> moduleRelationMap = moduleRelationList.stream().collect(Collectors.toMap(ModuleRelation::getModuleId, Function.identity()));
            //获取实例数据
            Instance instance = instanceRepository.query(CmdbUtils.CMDB_INSTANCE + startModule.getCode().toLowerCase(), instanceId);
            if (null == instance) {
                return new GeneralResult<>(false, "资源实例数据不存在");
            }
            List<InstanceModel> routeNodes = new ArrayList<>();
            //生产拓扑关系
            List<InstanceModel> nodes = Lists.newArrayList();
            //将当前节点放入
            InstanceModel instanceModel = new InstanceModel();
            instanceModel.setModuleId(startModule.getId());
            instanceModel.setModuleCode(startModule.getCode());
            instanceModel.setInstanceId(instanceId);
            instanceModel.setInstanceName(instance.getName());
            instanceModel.setIconImage(startModule.getIconImage());
            nodes.add(instanceModel);
            routeNodes.add(instanceModel);
            List<InstanceRelationModel> links = Lists.newArrayList();
            buildRelation(routeNodes, nodes, links, config, startModule.getId(), relationCategory, instance.getId(), 1, moduleRelationMap, config.getStartType());
            InstanceTopo topo = new InstanceTopo();
            topo.setNodes(nodes);
            topo.setLinks(links);
            return new GeneralResult<>(true, topo, "资源拓扑数据查询成功");
        } catch (Exception e) {
            log.error("failed to query topology data:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void buildRelation(List<InstanceModel> routeNodes, List<InstanceModel> nodes, List<InstanceRelationModel> links, TopologyConfig config, Long startModuleId, RelationCategory relationCategory, Long instanceId, int hierarchy, Map<Long, ModuleRelation> moduleRelationMap, String startType) throws Exception {
        //当前当前实例节点所有的关系数据
        List<InstanceRelation> instanceRelationsUp = Lists.newArrayList();
        List<InstanceRelation> instanceRelationsDown = Lists.newArrayList();
        if ("UP".equalsIgnoreCase(startType)) {
            List<InstanceRelation> instanceRelations = instanceRelationRepository.queryTargets(CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase(), startModuleId, instanceId);
            instanceRelationsUp.addAll(instanceRelations);
        } else if ("DOWN".equalsIgnoreCase(startType)) {
            List<InstanceRelation> instanceRelations = instanceRelationRepository.querySources(CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase(), startModuleId, instanceId);
            instanceRelationsDown.addAll(instanceRelations);
        } else {
            log.info("表名:{},源模型id:{},源实例id:{}", relationCategory.getCode().toLowerCase(), startModuleId, instanceId);
            List<InstanceRelation> up = instanceRelationRepository.queryTargets(CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase(), startModuleId, instanceId);
            List<InstanceRelation> down = instanceRelationRepository.querySources(CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase(), startModuleId, instanceId);
            instanceRelationsUp.addAll(up);
            instanceRelationsDown.addAll(down);
        }
        log.info("查询到的上层关系数据:{}", JSON.toJSONString(instanceRelationsUp));
        log.info("查询到的下层关系数据:{}", JSON.toJSONString(instanceRelationsDown));
        if (instanceRelationsUp.isEmpty() && instanceRelationsDown.isEmpty()) {
            log.info("向上或向下查询第" + hierarchy + "层结束");
            return;
        }
        if ("UP".equalsIgnoreCase(startType) && instanceRelationsUp.isEmpty()) {
            log.info("向上查询第" + hierarchy + "层结束");
            return;
        }
        if ("DOWN".equalsIgnoreCase(startType) && instanceRelationsDown.isEmpty()) {
            log.info("向下查询第" + hierarchy + "层结束");
            return;
        }
        log.info("当前模型[" + startModuleId + "],实例:[" + instanceId + "],向上查询到：[" + instanceRelationsUp.size() + "],目前在第" + hierarchy + "层");
        for (InstanceRelation instanceRelation : instanceRelationsUp) {
            long count = routeNodes.stream().filter(item -> item.getModuleId().equals(instanceRelation.getSourceModuleId()) && item.getInstanceId().equals(instanceRelation.getSourceInstanceId())).count();
            //实例关系
            buildInstanceRelation(links, instanceRelation, relationCategory);
            if (count > 0) {
                continue;
            }
            //实例关系另一端的节点
            InstanceModel instanceModel = new InstanceModel();
            instanceModel.setModuleId(instanceRelation.getSourceModuleId());
            instanceModel.setModuleCode(instanceRelation.getSourceModuleCode());
            instanceModel.setInstanceId(instanceRelation.getSourceInstanceId());
            instanceModel.setInstanceName(instanceRelation.getSourceInstanceName());
            if (null == moduleRelationMap.get(instanceRelation.getSourceModuleId())) {
                log.error("模型关系：{},实例关系{}", JSONObject.toJSON(moduleRelationMap), JSONObject.toJSON(instanceRelation));
            }
            instanceModel.setIconImage(moduleRelationMap.get(instanceRelation.getSourceModuleId()).getIconImage());
            nodes.add(instanceModel);
            routeNodes.add(instanceModel);
            //向上查询
            if (hierarchy < config.getTopologyLevelUp()) {
                buildRelation(routeNodes, nodes, links, config, instanceRelation.getSourceModuleId(), relationCategory, instanceRelation.getSourceInstanceId(), hierarchy + 1, moduleRelationMap, "UP");
            }
        }
        for (InstanceRelation instanceRelation : instanceRelationsDown) {
            long count = routeNodes.stream().filter(item -> item.getModuleId().equals(instanceRelation.getTargetModuleId()) && item.getInstanceId().equals(instanceRelation.getTargetInstanceId())).count();
            //实例关系
            buildInstanceRelation(links, instanceRelation, relationCategory);
            if (count > 0) {
                continue;
            }
            //实例关系另一端的节点
            InstanceModel instanceModel = new InstanceModel();
            instanceModel.setModuleId(instanceRelation.getTargetModuleId());
            instanceModel.setModuleCode(instanceRelation.getTargetModuleCode());
            instanceModel.setInstanceId(instanceRelation.getTargetInstanceId());
            instanceModel.setInstanceName(instanceRelation.getTargetInstanceName());
            instanceModel.setIconImage(moduleRelationMap.get(instanceRelation.getTargetModuleId()).getIconImage());
            nodes.add(instanceModel);
            routeNodes.add(instanceModel);
            //向下查询
            if (hierarchy < config.getTopologyLevelDown()) {
                buildRelation(routeNodes, nodes, links, config, instanceRelation.getTargetModuleId(), relationCategory, instanceRelation.getTargetInstanceId(), hierarchy + 1, moduleRelationMap, "DOWN");
            }
        }
    }

    private void buildInstanceRelation(List<InstanceRelationModel> links, InstanceRelation instanceRelation, RelationCategory relationCategory) {
        InstanceRelationModel instanceRelationModel = new InstanceRelationModel();
        instanceRelationModel.setId(instanceRelation.getId());
        instanceRelationModel.setRelationCategoryId(relationCategory.getId());
        instanceRelationModel.setRelationCategoryCode(relationCategory.getCode());
        instanceRelationModel.setRelationId(instanceRelation.getRelationId());
        instanceRelationModel.setRelationName(instanceRelation.getRelationName());
        instanceRelationModel.setSourceModuleId(instanceRelation.getSourceModuleId());
        instanceRelationModel.setSourceModuleCode(instanceRelation.getSourceModuleCode());
        instanceRelationModel.setSourceInstanceId(instanceRelation.getSourceInstanceId());
        instanceRelationModel.setSourceInstanceName(instanceRelation.getSourceInstanceName());
        instanceRelationModel.setTargetModuleId(instanceRelation.getTargetModuleId());
        instanceRelationModel.setTargetModuleCode(instanceRelation.getTargetModuleCode());
        instanceRelationModel.setTargetInstanceId(instanceRelation.getTargetInstanceId());
        instanceRelationModel.setTargetInstanceName(instanceRelation.getTargetInstanceName());
        instanceRelationModel.setType(instanceRelation.getSourceType() + "-" + instanceRelation.getTargetType());
        links.add(instanceRelationModel);
    }


    /**
     * 实例拓扑-根据实例id查找下一层级实例拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    @Override
    public GeneralResult<InstanceTopo> nextTopoList(Long id, Long moduleId, Long instanceId) {
        try {
            InstanceTopo topo = new InstanceTopo();
            TopologyConfig config = topologyConfigRepository.query(id);
            if (null == config) {
                return new GeneralResult<>(false, "没有对应的拓扑实例");
            }
            List<ModuleRelation> moduleRelations = moduleRelationRepository.queryModuleList(config.getCategoryId());
            if (CollectionUtils.isEmpty(moduleRelations)) {
                return new GeneralResult<>(false, "对应的拓扑图内没有模型关系数据");
            }
            RelationCategory category = relationCategoryRepository.query(config.getCategoryId());
            if (null == category) {
                return new GeneralResult<>(false, "没有对应的关系分组数据");
            }
            // 得到要查询的模型实例关系表名
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + category.getCode().toLowerCase();
            List<InstanceRelation> instanceRelations = new ArrayList<>();
            List<Long> moduleIds = new ArrayList<>();
            if (null != config.getPathDirection()) {
                if (config.getPathDirection()) {
                    instanceRelations = instanceRelationRepository.listReverseByInstanceIds(relationTable, config.getCategoryId(), moduleId, instanceId);
                    moduleIds = instanceRelations.stream().map(InstanceRelation::getSourceModuleId).collect(Collectors.toList());
                } else {
                    instanceRelations = instanceRelationRepository.listForwardByInstanceIds(relationTable, config.getCategoryId(), moduleId, instanceId);
                    moduleIds = instanceRelations.stream().map(InstanceRelation::getTargetModuleId).collect(Collectors.toList());
                }
            }
            if (CollectionUtils.isEmpty(instanceRelations)) {
                return new GeneralResult<>(true, topo, "查询成功");
            }
            moduleIds.add(moduleId);
            // 根据模型id查询模型信息
            List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
            if (CollectionUtils.isEmpty(modules)) {
                return new GeneralResult<>(false, "没有对应的模型数据");
            }
            List<InstanceRelationModel> links = new ArrayList<>();
            List<InstanceModel> nodes = new ArrayList<>();
            for (InstanceRelation instanceRelation : instanceRelations) {
                InstanceRelationModel link = new InstanceRelationModel();
                Long moduleRelationId = instanceRelation.getModuleRelationId();
                ModuleRelation moduleRelation = moduleRelations.stream().filter(item -> item.getId().equals(moduleRelationId)).findFirst().orElse(null);
                this.buildInstanceRelationModel(instanceRelation, link, moduleRelation);
                links.add(link);
                this.buildNodes(instanceRelation, nodes, modules);
            }
            topo.setNodes(nodes);
            topo.setLinks(links);
            return new GeneralResult<>(true, topo, "查询成功");
        } catch (Exception e) {
            log.error("topologyConfigService nextTopoList fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void buildInstanceRelationModel(InstanceRelation instanceRelation, InstanceRelationModel link, ModuleRelation moduleRelation) {
        if (instanceRelation.getIsReversed()) {
            link.setRelationCategoryId(instanceRelation.getRelationCategoryId());
            link.setRelationCategoryCode(instanceRelation.getRelationCategoryCode());
            link.setRelationId(instanceRelation.getModuleRelationId());
            link.setRelationName(instanceRelation.getModuleRelationName());
            link.setSourceModuleId(instanceRelation.getTargetModuleId());
            link.setSourceModuleCode(instanceRelation.getTargetModuleCode());
            link.setSourceInstanceId(instanceRelation.getTargetInstanceId());
            link.setSourceInstanceName(instanceRelation.getTargetInstanceName());
            link.setTargetModuleId(instanceRelation.getSourceModuleId());
            link.setTargetModuleCode(instanceRelation.getSourceModuleCode());
            link.setTargetInstanceId(instanceRelation.getSourceInstanceId());
            link.setTargetInstanceName(instanceRelation.getSourceInstanceName());
            link.setType(moduleRelation.getTargetType() + "-" + moduleRelation.getSourceType());
        } else {
            BeanUtils.copyProperties(instanceRelation, link);
            link.setRelationId(instanceRelation.getModuleRelationId());
            link.setRelationName(instanceRelation.getModuleRelationName());
            link.setType(moduleRelation.getSourceType() + "-" + moduleRelation.getTargetType());
        }
    }

    private void buildNodes(InstanceRelation instanceRelation, List<InstanceModel> nodes, List<CiModule> modules) {
        // 获取节点数据
        // 去重-因为不同的模型对应的实例的id有可能重复。所以要对实例去重的话必须用模型id和实例id一起去重
        List<InstanceModel> instanceModelList = nodes.stream().filter(item -> item.getModuleId().equals(instanceRelation.getSourceModuleId()) && item.getInstanceId().equals(instanceRelation.getSourceInstanceId())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(instanceModelList)) {
            InstanceModel source = new InstanceModel();
            source.setModuleId(instanceRelation.getSourceModuleId());
            source.setModuleCode(instanceRelation.getSourceModuleCode());
            source.setInstanceId(instanceRelation.getSourceInstanceId());
            source.setInstanceName(instanceRelation.getSourceInstanceName());
            CiModule module = modules.stream().filter(item -> item.getId().equals(source.getModuleId())).findFirst().orElse(null);
            source.setIconImage(module.getIconImage());
            nodes.add(source);
        }
        List<InstanceModel> instanceModels = nodes.stream().filter(item -> item.getModuleId().equals(instanceRelation.getTargetModuleId()) && item.getInstanceId().equals(instanceRelation.getTargetInstanceId())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(instanceModels)) {
            InstanceModel target = new InstanceModel();
            target.setModuleId(instanceRelation.getTargetModuleId());
            target.setModuleCode(instanceRelation.getTargetModuleCode());
            target.setInstanceId(instanceRelation.getTargetInstanceId());
            target.setInstanceName(instanceRelation.getTargetInstanceName());
            CiModule module = modules.stream().filter(item -> item.getId().equals(target.getModuleId())).findFirst().orElse(null);
            target.setIconImage(module.getIconImage());
            nodes.add(target);
        }
    }

    /**
     * 获取起始模型为源id，对应的路径。
     *
     * @param topologyModel
     * @return
     */
    private GeneralResult<Map<Boolean, List<List<Long>>>> checkData(RequestInstanceTopologyModel topologyModel) {
        try {
            if (null == topologyModel.getCategoryId()) {
                return new GeneralResult<>(false, "categoryId参数不能为空");
            }
            if (null == topologyModel.getStartModuleId()) {
                return new GeneralResult<>(false, "startModuleId参数不能为空");
            }
            // 根据拓扑图分组id查询分组内模型数据
            List<ModuleRelation> relations = moduleRelationRepository.queryModuleList(topologyModel.getCategoryId());
            if (CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(false, "拓扑图内无拓扑模型！");
            }
            // 情况一：以startModuleId为源id获取对应的树 并 获取对应路径集合
            List<List<Long>> outDegreePath = this.getTreePath(topologyModel, relations, true);
            // 情况二：以startModuleId为目的id获取对应的树 并 获取对应路径集合
            List<List<Long>> penetrationPath = this.getTreePath(topologyModel, relations, false);
            Map<Boolean, List<List<Long>>> map = new HashMap();
            map.put(true, penetrationPath);
            map.put(false, outDegreePath);
            return new GeneralResult<>(true, map, "拓扑图内无拓扑模型！");
        } catch (Exception e) {
            e.printStackTrace();
            return new GeneralResult<>(false, "参数校验失败");
        }
    }

    private String escapeData(TopologyConfig config) {
        try {
            // 获取拓扑路径模型id
            // 将拓扑路径模型id类型转成Long类型
            List<Long> moduleIds = Arrays.asList(config.getTopologyPath().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
            // 根据模型id查询模型信息
            List<CiModule> modules = moduleRepository.queryModuleListByIds(moduleIds);
            if (CollectionUtils.isEmpty(modules)) {
                return "没有对应的模型数据";
            }
            String pathName = modules.stream().map(CiModule::getName).collect(Collectors.joining("-"));
            config.setTopologyPathName(pathName);
            RelationCategory relationCategory = relationCategoryRepository.query(config.getCategoryId());
            if (null == relationCategory) {
                return "没有对应的关联分组数据";
            }
            config.setCategoryName(relationCategory.getName());
            CiModule startModule = moduleRepository.query(config.getStartModuleId());
            if (null == startModule) {
                return "没有对应的起始模型数据";
            }
            if (null != config.getIgnoreModuleIds()) {
                List<Long> ignoreIds = Arrays.asList(config.getIgnoreModuleIds().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
                List<CiModule> ignores = moduleRepository.queryModuleListByIds(ignoreIds);
                for (Long moduleId : ignoreIds) {
                    CiModule module = ignores.stream().filter(item -> item.getId().equals(moduleId)).findFirst().orElse(null);
                    if (null == module) {
                        return "没有对应的忽略模型数据";
                    }
                }
                config.setIgnoreModuleId(ignoreIds);
            }
            if (null != config.getWayModuleIds()) {
                List<Long> wayIds = Arrays.asList(config.getWayModuleIds().split(",")).stream().map(moduleId -> Long.parseLong(moduleId)).collect(Collectors.toList());
                List<CiModule> ways = moduleRepository.queryModuleListByIds(wayIds);
                for (Long moduleId : wayIds) {
                    CiModule module = ways.stream().filter(item -> item.getId().equals(moduleId)).findFirst().orElse(null);
                    if (null == module) {
                        return "没有对应的忽略模型数据";
                    }
                }
                config.setWayModuleId(wayIds);
            }
            config.setStartModuleName(startModule.getName());
            return null;
        } catch (Exception e) {
            return "数据库查询出错";
        }
    }

    /**
     * 根据predicate生成出度树或者入度树并得到起路径：
     * <p>
     * 生成出度树 并 获取对应路径
     * 实现逻辑：这里是以sourceId为key进行分组的并用startModuleId匹配sourceId生成对应的树的。
     * 这里就是以startModuleId作为源id（sourceId）获取对应的树。
     * <p>
     * 生成入度树 并 获取对应路径
     * 实现逻辑：这里是以targetId为key进行分组的并用startModuleId匹配targetId生成对应的树的。
     * 在数据结构上看就是以startModuleId为目的id（targetId）获取相应的额树。
     *
     * @param topologyModel
     * @param relations
     * @return
     */
    private List<List<Long>> getTreePath(RequestInstanceTopologyModel topologyModel, List<ModuleRelation> relations, Boolean predicate) {
        ModuleRelation treePath = new ModuleRelation();
        Map<Long, List<ModuleRelation>> relationMap = relations.stream().collect(Collectors.groupingBy(predicate ? ModuleRelation::getSourceId : ModuleRelation::getTargetId));
        List<ModuleRelation> moduleRelations = relationMap.get(topologyModel.getStartModuleId());
        if (CollectionUtils.isNotEmpty(moduleRelations)) {
            if (predicate) {
                treePath.setSourceId(topologyModel.getStartModuleId());
            } else {
                treePath.setTargetId(topologyModel.getStartModuleId());
            }
            //把遍历过的路径存入到集合中
            List<Map<Long, Long>> lineList = new ArrayList<>();
            for (ModuleRelation moduleRelation : moduleRelations) {
                Map<Long, Long> map = new HashMap<>();
                if (predicate) {
                    map.put(moduleRelation.getSourceId(), moduleRelation.getTargetId());
                } else {
                    map.put(moduleRelation.getTargetId(), moduleRelation.getSourceId());
                }
                lineList.add(map);
            }
            //递归生成对应的树
            this.recursionTree(moduleRelations, relationMap, lineList, predicate);
            treePath.setCmdbModuleRelationList(moduleRelations);
        } else {
            return new ArrayList<>();
        }
        if (predicate) {
            //根据生成树获取树的所有路径
            return GraphAlgorithmUtils.outDegreeGraphAlgorithm(treePath);
        } else {
            //根据生成树获取对应路径
            return GraphAlgorithmUtils.penetrationGraphAlgorithm(treePath);
        }
    }

    /**
     * 此处就是递归生成对应的树
     *
     * @param relations
     * @param relationMap
     */
    private void recursionTree(List<ModuleRelation> relations, Map<Long, List<ModuleRelation>> relationMap, List<Map<Long, Long>> lineList, Boolean predicate) {
        for (ModuleRelation moduleRelation : relations) {
            List<ModuleRelation> moduleRelations = relationMap.get(predicate ? moduleRelation.getTargetId() : moduleRelation.getSourceId());
            if (CollectionUtils.isEmpty(moduleRelations)) {
                continue;
            }
            List<ModuleRelation> moduleRelationList = new ArrayList<>();
            for (ModuleRelation relation : moduleRelations) {
                //1：判断这个路径是是否已经遍历过了（路径就是线段，是不区分弧头和弧尾的。所以说 需要用 头结点-尾结点 以及 尾结点-头结点 去判断这条路径是否存在）
                Map<Long, Long> map = lineList.stream().filter(item -> relation.getTargetId().equals(item.get(relation.getSourceId())) || relation.getSourceId().equals(item.get(relation.getTargetId()))).findFirst().orElse(null);
                //2：判断如果尾结点是走过路径的头结点也是不行的。
                Map<Long, Long> longMap = lineList.stream().filter(item -> item.containsKey(predicate ? relation.getTargetId() : relation.getSourceId())).findFirst().orElse(null);
                if (MapUtils.isEmpty(map) && MapUtils.isEmpty(longMap)) {
                    moduleRelationList.add(relation);
                    Map<Long, Long> hashMap = new HashMap<>();
                    hashMap.put(predicate ? relation.getSourceId() : relation.getTargetId(), predicate ? relation.getTargetId() : relation.getSourceId());
                    lineList.add(hashMap);
                }
            }
            if (CollectionUtils.isNotEmpty(moduleRelationList)) {
                moduleRelation.setCmdbModuleRelationList(moduleRelationList);
                this.recursionTree(moduleRelationList, relationMap, lineList, predicate);
            }
        }
    }

    /**
     * 获取路径
     *
     * @param allList
     * @param modules
     * @param topologyLevel 拓扑层级
     * @param pathModels
     * @param pathDirection
     */
    private void getPathModelList(List<List<Long>> allList, List<CiModule> modules, Integer topologyLevel, List<PathModel> pathModels, Boolean pathDirection, RequestInstanceTopologyModel topologyModel) {
        List<List<Long>> treePath = allList.stream().filter(item -> item.size() >= topologyLevel).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(treePath)) {
            log.info("treePath:{}", JSON.toJSONString(treePath));
            List<List<CiModule>> moduleList = new ArrayList<>();
            for (List<Long> ids : treePath) {
                ids.removeAll(Collections.singleton(null));
                // 应该是 一个一个id 去找对应的模型 并 add对应的模型分组中 (一个一个的匹配是为了保证节点的顺序性不被打乱)
                List<CiModule> list = new ArrayList<>();
                for (Long id : ids) {
                    CiModule module = modules.stream().filter(item -> id.equals(item.getId())).findFirst().orElse(null);
                    if (null != module) {
                        list.add(module);
                    }
                }
                moduleList.add(list);
            }
            for (List<CiModule> list : moduleList) {
                if (list.size() >= topologyLevel) {
                    //截取前topologyLevel个数据
                    List<CiModule> modulesData = list.subList(0, topologyLevel);
                    //过滤忽略模型
                    if (CollectionUtils.isNotEmpty(topologyModel.getIgnoreModuleId())) {
                        List<CiModule> modulesDataList = modulesData.stream().filter(item -> topologyModel.getIgnoreModuleId().contains(item.getId())).collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(modulesDataList)) {
                            continue;
                        }
                    }
                    if (CollectionUtils.isNotEmpty(topologyModel.getWayModuleId())) {
                        //判断是否存在必经模型.containsAll
                        boolean containsAll = modulesData.stream().map(CiModule::getId).collect(Collectors.toList()).containsAll(topologyModel.getWayModuleId());
                        if (!containsAll) {
                            continue;
                        }
                    }
                    PathModel pathModel = new PathModel();
                    String pathName = modulesData.stream().map(CiModule::getName).collect(Collectors.joining("-"));
                    String path = modulesData.stream().map(CiModule::getId).map(item -> String.valueOf(item)).collect(Collectors.joining(","));
                    pathModel.setTopologyPathName(pathName);
                    pathModel.setTopologyPath(path);
                    pathModel.setPathDirection(pathDirection);
                    pathModels.add(pathModel);
                }
            }
        }
    }

    private List<List<InstanceModel>> pagerFilter(List<List<InstanceModel>> relationList, Pager pager) {
        List<List<InstanceModel>> instanceIdRelations = new ArrayList<>();
        // 如果传过来的pager参数为空就不需要过滤了 直接返回
        if (CollectionUtils.isEmpty(pager.getParams()) || MapUtils.isEmpty(pager.getParams().get(0).getParam())) {
            return relationList;
        }
        Map<String, Object> param = pager.getParams().get(0).getParam();
        param.remove("id");
        // 外循环是循环每一行数据
        for (List<InstanceModel> relationModels : relationList) {
            boolean flag = true;
            // 内循环是循环每一行数据中的每个对象
            for (InstanceModel relationModel : relationModels) {
                if (param.containsKey(relationModel.getModuleCode())) {
                    if (!relationModel.getInstanceId().equals(Long.valueOf(String.valueOf(param.get(relationModel.getModuleCode()))))) {
                        flag = false;
                        break;
                    }
                }
            }
            if (flag) {
                instanceIdRelations.add(relationModels);
            }
        }
        return instanceIdRelations;
    }

    /**
     * 获取动态表头
     *
     * @param columns
     * @return
     */
    private List<List<String>> columnData(List<ColumnModel> columns) {
        List<List<String>> labelList = new ArrayList<>();
        for (ColumnModel column : columns) {
            List<String> list = new ArrayList<>();
            list.add(column.getLabel());
            labelList.add(list);
        }
        return labelList;
    }

    /**
     * 根据动态表头匹配对应列数据
     *
     * @param columns
     * @param rows
     * @return
     */
    private List<List<Object>> tableData(List<ColumnModel> columns, List<Map<String, Object>> rows) {
        // 对于每一行数据找匹配到对应的列上
        List<List<Object>> list = ListUtils.newArrayList();
        for (Map<String, Object> map : rows) {
            List<Object> data = ListUtils.newArrayList();
            // 对于每行数据匹配到对应的列上
            for (ColumnModel column : columns) {
                String value = column.getProp();
                Object object = map.get(value);
                data.add(object);
            }
            list.add(data);
        }
        return list;
    }
}
