package com.bocloud.ams.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.PropertyItem;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.module.PropertyItemRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @ClassName: InstanceExportHelper
 * @Description
 * @Author yangzhenrong
 * @Date 2022/5/25 14:26
 * @Version 1.0
 **/
@Slf4j
@Component
public class InstanceExcelHelper {

    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private PropertyItemRepository propertyItemRepository;

    /***
     * 导出数据
     *
     * @param wb
     * @param properties
     * @param instances
     * @throws Exception
     */
    public void generateDate(Workbook wb, List<Property> properties, List<Map<String, Object>> instances)
            throws Exception {
        DataFormat format = wb.createDataFormat();
        CellStyle cellStyle = wb.createCellStyle();
//        cellStyle.setWrapText(true);// 自动换行
        // 设置文本格式
        cellStyle.setDataFormat(format.getFormat("@"));
        Sheet sheet = wb.getSheetAt(0);

        Map<String, Integer> header = ExcelHelper.readHeader4NameKey(sheet.getRow(0), 0);
        Map<String, Property> propertyMap = Maps.newHashMap();
        Map<String, Integer> propertyIndex = Maps.newHashMap();
        for (Property property : properties) {
            Integer index = header.get(property.getName());
            propertyIndex.put(property.getCode(), index);
            propertyMap.put(property.getCode(), property);
        }
        // 从第2行开始
        int rowNum = 1;
        for (Map<String, Object> instance : instances) {
            generateCellData(sheet, cellStyle, rowNum++, propertyIndex, propertyMap, instance);
        }
    }

    /***
     * 生成数据单元格
     *
     * @param sheet
     * @param cellStyle
     * @param rowNum
     * @param propertyIndex
     * @param propertyMap
     * @param instance
     * @throws Exception
     */
    protected void generateCellData(Sheet sheet, CellStyle cellStyle, int rowNum, Map<String, Integer> propertyIndex,
                                    Map<String, Property> propertyMap, Map<String, Object> instance) throws Exception {
        // 生成行
        Row row = sheet.createRow(rowNum);
        // 生成属性值
        generateInstanceValue(cellStyle, propertyIndex, propertyMap, instance, row);
    }

    /***
     * 生成属性值
     *
     * @param cellStyle
     * @param propertyIndex
     * @param propertyMap
     * @param instance
     * @param row
     */
    private void generateInstanceValue(CellStyle cellStyle, Map<String, Integer> propertyIndex,
                                       Map<String, Property> propertyMap, Map<String, Object> instance, Row row) {
        Cell cell;
        for (Map.Entry<String, Object> entry : instance.entrySet()) {
            Integer index = propertyIndex.get(entry.getKey());
            if (index != null) {
                // 生成列
                cell = row.createCell(index);
                if (propertyMap.containsKey(entry.getKey())) {
                    Property property = propertyMap.get(entry.getKey());
                    String propertyType = property.getType();
                    cell = row.createCell(index);
                    cell.setCellStyle(cellStyle);
                    Object value = entry.getValue();
                    List<PropertyItem> propertyItems = property.getItemList();
                    if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                            || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                        for (PropertyItem propertyItem : propertyItems) {
                            if (propertyItem.getValue().equals(entry.getValue())) {
                                value = propertyItem.getName();
                                break;
                            }
                        }
                    } else if (null != value && (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType))) {
                        Map<String, PropertyItem> propertyItemMap = new HashMap<>();
                        for (PropertyItem propertyItem : propertyItems) {
                            propertyItemMap.put(propertyItem.getValue(), propertyItem);
                        }
                        String newValue = "";
                        List<String> values = JSONObject.parseArray(value.toString(), String.class);
                        for (String val : values) {
                            if (propertyItemMap.containsKey(val)) {
                                newValue += propertyItemMap.get(val).getName() + ",";
                            }
                        }
                        if(StringUtils.isNotEmpty(newValue)){
                            value = newValue.substring(0, newValue.length() - 1);
                        }else {
                            value = newValue;
                        }

                    } else if (null != value && (PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType))) {
                        JSONArray result = dealTableData(propertyItems, value);
                        value = result.toJSONString();
                    }
                    cell.setCellValue(new ExcelFactory(row).newRichTextString((String) value));
                }

            }
        }
    }

    private JSONArray dealTableData(List<PropertyItem> propertyItems, Object value) {

        JSONArray targetArray = new JSONArray();
        if(null == value){
            return targetArray;
        }

        Map<String, PropertyItem> propertyItemMap = new HashMap<>();
        for (PropertyItem propertyItem : propertyItems) {
            propertyItemMap.put(propertyItem.getCode(), propertyItem);
        }
        try {
            JSONArray array = JSONArray.parseArray(value.toString());

            for (Object a : array) {
                JSONObject obj = (JSONObject) JSON.toJSON(a);
                JSONObject targetObj = new JSONObject();

                for (String key : obj.keySet()) {
                    PropertyItem property = propertyItemMap.get(key);
                    String propertyType = property.getType();
                    if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                            || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {

                        for (PropertyItem propertyItem : property.getItemList()) {
                            if (propertyItem.getValue().equals(obj.getString(key))) {
                                targetObj.put(property.getName(), propertyItem.getName());
                                break;
                            }
                        }
                    } else if (null != value && (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType))) {
                        Map<String, PropertyItem> propertyItemValueMap = new HashMap<>();
                        for (PropertyItem propertyItem : property.getItemList()) {
                            propertyItemValueMap.put(propertyItem.getValue(), propertyItem);
                        }
                        String newValue = "";
                        List<String> values = JSONObject.parseArray(obj.getString(key), String.class);
                        for (String val : values) {
                            if (propertyItemValueMap.containsKey(val)) {
                                newValue += propertyItemValueMap.get(val).getName() + ",";
                            }
                        }

                        if (StringUtils.isNotEmpty(newValue)) {
                            targetObj.put(property.getName(), newValue.substring(0, newValue.length() - 1));
                        } else {
                            targetObj.put(property.getName(), newValue);
                        }

                    } else {
                        targetObj.put(property.getName(), obj.getString(key));
                    }
                }

                targetArray.add(targetObj);

            }
        } catch (Exception e) {
            log.error("表格数据格式异常", value);
        }

        return targetArray;

    }

    /**
     * 生成模板
     *
     * @param wb
     * @throws Exception
     */
    public void generateSheet(Workbook wb, CiModule module, List<Property> properties) throws Exception {
        String sheetName = module.getName() + "(" + module.getCode() + ")";
        Sheet sheet = wb.createSheet(sheetName);
        // 第一行 属性名称
        Row row0 = sheet.createRow(0);
        // 定义颜色
        CellStyle cellStyle = wb.createCellStyle();
        cellStyle.setFillForegroundColor(HSSFColor.HSSFColorPredefined.LIGHT_BLUE.getIndex());
        // 填充模式
        cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);
        XSSFFont font = (XSSFFont) wb.createFont();
        font.setFontName("仿宋_GB2312");
        font.setBold(true);// 粗体显示
        font.setFontHeightInPoints((short) 12);
        cellStyle.setFont(font);
        // 生成表头
        generateHeader(sheet, cellStyle, row0, properties);
        // 设置文本格式以及调整列宽,使之能完整显示字段名称
        ExcelHelper.setTextformatAndColumnwidth(wb);
    }

    protected void generateHeader(Sheet sheet, CellStyle cellStyle, Row row0, List<Property> properties)
            throws Exception {
        // 导出属性
        Cell cell0 = null;
        // 列位置索引
        int index = 0;
        for (Property property : properties) {
            cell0 = row0.createCell(index);
            cell0.setCellValue(new ExcelFactory(row0).newRichTextString(property.getName()));
            cell0.setCellStyle(cellStyle);
            sheet.setColumnWidth(index, 20 * 256);
            index++;
        }
    }

    public void generateTemplateSheet(Workbook wb, CiModule module, List<Property> properties) throws Exception {
        String sheetName = module.getName() + "(" + module.getCode() + ")";
        Sheet sheet = wb.createSheet(sheetName);
        // 第一行 属性名称
        Row row0 = sheet.createRow(0);
        // 第二行 属性说明
        Row row1 = sheet.createRow(1);
        // 第三行 正文提示
        Row row2 = sheet.createRow(2);
        // 定义颜色
        CellStyle cellStyle = wb.createCellStyle();
        cellStyle.setFillForegroundColor(HSSFColor.HSSFColorPredefined.ROYAL_BLUE.getIndex());
        // 填充模式
        cellStyle.setFillPattern(FillPatternType.DIAMONDS);
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);

        CellStyle cellStyle2 = wb.createCellStyle();
        cellStyle.setFillForegroundColor(HSSFColor.HSSFColorPredefined.LIGHT_YELLOW.getIndex());
        // 填充模式
        cellStyle2.setFillPattern(FillPatternType.DIAMONDS);
        // 居中
        cellStyle2.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle2.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle2.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle2.setBorderRight(BorderStyle.THIN);

        XSSFFont font = (XSSFFont) wb.createFont();
        font.setFontName("仿宋_GB2312");
        font.setBold(true);// 粗体显示
        font.setFontHeightInPoints((short) 12);
        cellStyle.setFont(font);
        cellStyle2.setFont(font);
        // 生成表头
        generateTemplateHeader(sheet, cellStyle, cellStyle2, row0, row1, row2, properties);
        // 设置文本格式以及调整列宽,使之能完整显示字段名称
        ExcelHelper.setTextformatAndColumnwidth(wb);
    }

    protected void generateTemplateHeader(Sheet sheet, CellStyle cellStyle, CellStyle cellStyle2, Row row0, Row row1,
                                          Row row2, List<Property> properties) throws Exception {
        // 导出属性
        // 第一行 属性名称
        Cell cell0 = null;
        // 第二行 属性说明
        Cell cell1 = null;
        // 第三行 正文提示
        Cell cell2 = null;

        // 第一列给出提示start...
        cell0 = row0.createCell(0);
        cell0.setCellValue(new ExcelFactory(row0).newRichTextString("属性名称-->"));
        cell0.setCellStyle(cellStyle);

        cell1 = row1.createCell(0);
        cell1.setCellValue(new ExcelFactory(row1).newRichTextString("填写要求-->"));
        cell1.setCellStyle(cellStyle);

        cell2 = row2.createCell(0);
        cell2.setCellValue(new ExcelFactory(row2).newRichTextString("此行开始填写数据-->"));
        cell2.setCellStyle(cellStyle);
        sheet.setColumnWidth(0, "此行开始填写数据".getBytes().length * 256);
        // 第一列给出提示end...

        // 列位置索引
        int index = 1;
        for (Property property : properties) {
            String propertyType = property.getType();
            if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                    || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                    || PropertyConstant.PropertyType.RADIO.equals(propertyType)
                    || PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                    || PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                    || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                List<PropertyItem> items = property.getItemList();
                property.setItemList(items);
            }
            cell0 = row0.createCell(index);
            cell0.setCellValue(new ExcelFactory(row0).newRichTextString(property.getName()));
            cell0.setCellStyle(cellStyle2);

            cell1 = row1.createCell(index);
            cell1.setCellValue(new ExcelFactory(row1).newRichTextString(buildNoticeMsg(property)));
            cell1.setCellStyle(cellStyle2);
            // 多选下拉框暂不支持，以页面提示输入，以英文逗号分割
            if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                    || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                propertyItemCell(sheet, index, property);
            }
            // if (PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
            // || PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)) {
            // propertyItemCell(sheet, index, property);
            // }
            index++;
        }
    }

    private void propertyItemCell(Sheet sheet, int index, Property property) throws Exception {
        // 获取所有下拉值
        List<PropertyItem> items = property.getItemList();
        if (CollectionUtils.isNotEmpty(items)) {
            // 是否存在逗号
            Boolean flag = false;
            String[] textlist = new String[items.size()];
            // 数据有效性的长度
            int veluesLength = 0;
            int itemIndex = 0;
            for (PropertyItem item : items) {
                if (item.getName().contains(",")) {
                    flag = true;
                }
                textlist[itemIndex++] = item.getName();
                // 因为会追加逗号，所以+1
                veluesLength += item.getName().length() + 1;
            }
            if (veluesLength > 0 && veluesLength <= 255 && !flag) {
                ExcelHelper.setValidation(sheet, textlist, 2, 65535, index, index, false);
            } else if (veluesLength > 255 || flag) {
                // 用新页
                ExcelHelper.setValidation(sheet, textlist, 2, 65535, index, index, true);
            }
        }
    }

    /***
     * 构建属性提示语
     *
     * @param property
     * @return
     */
    private String buildNoticeMsg(Property property) {
        StringBuilder msg = new StringBuilder();
        try {
            if (property.getIsRequired()) {
                msg.append("必填;");
            } else {
                msg.append("非必填;");
            }
            if (null != property.getIsUniqued() && property.getIsUniqued()) {
                msg.append("唯一性;");
            }
            msg.append("格式[");
            String propertyType = property.getType();
            if (PropertyConstant.PropertyType.TEXTFIELD.equals(propertyType)) {
                msg.append("单行文本];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.TEXTAREA.equals(propertyType)) {
                msg.append("多行文本];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                    || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                msg.append("单选];");
                msg.append("可选值[");
                List<PropertyItem> items = property.getItemList();
                if (CollectionUtils.isNotEmpty(items)) {
                    for (int i = 0; i < items.size(); i++) {
                        msg.append(items.get(i).getName());
                        if (i < items.size() - 1) {
                            msg.append(",");
                        }
                    }
                    msg.append("];");
                }
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    try {
                        if (CollectionUtils.isNotEmpty(items)) {
                            for (PropertyItem item : items) {
                                if (property.getDefaultValue().equals(item.getValue())) {
                                    msg.append("默认值[" + item.getName() + "];");
                                }
                            }
                        }
                    } catch (Exception e) {
                        log.error("设置默认值失败", e);
                    }
                }
            } else if (PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                    || PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)) {
                msg.append("多选],");
                msg.append("可选值[");
                List<PropertyItem> items = property.getItemList();
                if (CollectionUtils.isNotEmpty(items)) {
                    for (int i = 0; i < items.size(); i++) {
                        msg.append(items.get(i).getName());
                        if (i < items.size() - 1) {
                            msg.append(",");
                        }
                    }
                    msg.append("];以英文逗号分割;");
                }
            } else if (PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                    || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                msg.append((PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType) ? "单行" : "多行")
                        + "表格];填写JSON格式;");
                List<PropertyItem> items = property.getItemList();
                String propertyIds = property.getPropertyIds();
                Long linkModuleId = property.getLinkModuleId();
                StringBuilder tipMsg = new StringBuilder();
                if (StringUtils.isEmpty(propertyIds)) {
                    if (CollectionUtils.isNotEmpty(items)) {
                        msg.append("示例[{");
                        tipMsg.append("示例说明[{");
                        for (int i = 0; i < items.size(); i++) {
                            if (PropertyConstant.PropertyType.SINGLESELECT.equals(items.get(i).getType())) {
                                tipMsg.append("(单选)");
                                List<JSONObject> itemValObj =
                                        JSONArray.parseArray(items.get(i).getItemValue(), JSONObject.class);
                                List<String> newItemVals = new ArrayList<>();
                                List<String> newItemValTips = new ArrayList<>();
                                for (JSONObject obj : itemValObj) {
                                    String newObj2Tip =
                                            "\"" + obj.getString("value") + "(" + obj.getString("name") + ")\"";
                                    String newObj = "\"" + obj.getString("value") + "\"";
                                    newItemValTips.add(newObj2Tip);
                                    newItemVals.add(newObj);
                                }
                                tipMsg.append("\"" + items.get(i).getCode() + "(" + items.get(i).getName() + ")\":"
                                        + newItemValTips);
                                if (CollectionUtils.isNotEmpty(newItemVals)) {
                                    msg.append("\"" + items.get(i).getCode() + "\":" + newItemVals.get(0));
                                } else {
                                    msg.append("\"" + items.get(i).getCode() + "\":\"\"");
                                }

                            } else if (PropertyConstant.PropertyType.MULTISELECT.equals(items.get(i).getType())) {
                                tipMsg.append("(多选)");
                                List<JSONObject> itemValObj =
                                        JSONArray.parseArray(items.get(i).getItemValue(), JSONObject.class);
                                List<String> newItemVals = new ArrayList<>();
                                List<String> newItemValTips = new ArrayList<>();
                                for (JSONObject obj : itemValObj) {
                                    String newObj2Tip =
                                            "\"" + obj.getString("value") + "(" + obj.getString("name") + ")\"";
                                    String newObj = "\"" + obj.getString("value") + "\"";
                                    newItemValTips.add(newObj2Tip);
                                    newItemVals.add(newObj);
                                }
                                tipMsg.append("\"" + items.get(i).getCode() + "(" + items.get(i).getName() + ")\":"
                                        + newItemValTips);
                                msg.append("\"" + items.get(i).getCode() + "\":" + newItemVals);
                            } else {
                                tipMsg
                                        .append("\"" + items.get(i).getCode() + "(" + items.get(i).getName() + ")\":\"\"");
                                msg.append("\"" + items.get(i).getCode() + "\":\"\"");
                            }
                            if (i < items.size() - 1) {
                                tipMsg.append(",");
                                msg.append(",");
                            }
                        }
                        tipMsg.append("}];");
                        msg.append("}];");
                        msg.append(tipMsg);
                    }
                } else {
                    msg.append("示例[{");
                    tipMsg.append("示例说明[{");
                    String[] propertyIdsplit = propertyIds.split(",");
                    for (int i = 0; i < propertyIdsplit.length; i++) {
                        String propertyId = propertyIdsplit[i];
                        Property object = propertyRepository.query(Long.valueOf(propertyId));
                        if (null != object) {
                            if (PropertyConstant.PropertyType.SINGLESELECT.equals(object.getType())
                                    || PropertyConstant.PropertyType.MULTISELECT.equals(object.getType())
                                    || PropertyConstant.PropertyType.RADIO.equals(object.getType())
                                    || PropertyConstant.PropertyType.CHECKBOX.equals(object.getType())) {
                                List<PropertyItem> propertyItems = propertyItemRepository.queryByModuleId(linkModuleId);
                                if (CollectionUtils.isNotEmpty(propertyItems)) {
                                    if (PropertyConstant.PropertyType.SINGLESELECT.equals(object.getType())
                                            || PropertyConstant.PropertyType.RADIO.equals(object.getType())) {
                                        msg.append("(单选)");
                                    }
                                    List<String> newItemVals = new ArrayList<>();
                                    List<String> newItemValTips = new ArrayList<>();
                                    for (PropertyItem propertyItem : propertyItems) {
                                        String newObjTip =
                                                "\"" + propertyItem.getCode() + "(" + propertyItem.getName() + ")\"";
                                        String newObj = "\"" + propertyItem.getCode() + "\"";
                                        newItemValTips.add(newObjTip);
                                        newItemVals.add(newObj);
                                    }
                                    tipMsg.append(
                                            "\"" + object.getCode() + "(" + object.getName() + ")\":" + newItemValTips);
                                    msg.append("\"" + object.getCode() + "\":" + newItemVals);
                                }
                            } else {
                                tipMsg.append("\"" + object.getCode() + "(" + object.getName() + ")\":\"\"");
                                msg.append("\"" + object.getCode() + "\":\"\"");
                            }
                        }
                        if (i < propertyIdsplit.length - 1) {
                            tipMsg.append(",");
                            msg.append(",");
                        }
                    }
                    tipMsg.append("}];");
                    msg.append("}];");
                    msg.append(tipMsg);
                }

                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.INTEGER.equals(propertyType)) {
                msg.append("整数];");
                if (null != property.getMinValue()) {
                    msg.append("最小值[" + property.getMinValue() + "];");
                }
                if (null != property.getMaxValue()) {
                    msg.append("最大值[" + property.getMaxValue() + "];");
                }
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.FLOAT.equals(propertyType)) {
                msg.append("数字];");
                if (null != property.getDecimalNum()) {
                    msg.append("小数位数[" + property.getDecimalNum() + "];");
                }
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.BOOLEAN.equals(propertyType)) {
                msg.append("布尔值];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.JSON.equals(propertyType)) {
                msg.append("JSON格式];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.DATE.equals(propertyType)) {
                msg.append("日期];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.TIME.equals(propertyType)) {
                msg.append("时间];");
                if (StringUtils.isNotEmpty(property.getDefaultValue())) {
                    msg.append("默认值[" + property.getDefaultValue() + "];");
                }
            } else if (PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {
                msg.append("加密,仅创建人能看到属性值];");
            }

            if (StringUtils.isNotEmpty(property.getUnit())) {
                msg.append("单位[" + property.getUnit() + "];");
            }
            if (null != property.getLength() && property.getLength() > 0) {
                msg.append("长度限制[" + property.getLength() + "]字符;");
            }
            return msg.toString();
        } catch (Exception e) {
            msg.append(e.getMessage());
        }
        return msg.toString();

    }


}
