package com.bocloud.ams.service.utils;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;

/**
 * @ClassName: RepositoryHandler
 * @Description 动态修改类Table注解值，及保存或修改数据
 * @Author yangzhenrong
 * @Date 2022/5/27 18:48
 * @Version 1.0
 **/
@Component
public class RepositoryHandler {
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;

    public void saveInstance(Instance instance, String tableName) {
        synchronized (Instance.class) {
            CmdbUtils.changeTableAnnotationValue(Instance.class, tableName);
            instanceRepository.save(instance);
        }
    }

    public void saveInstanceValue(InstanceValue instanceValue, String tableName) {
        synchronized (InstanceValue.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceValue.class, tableName);
            instanceValueRepository.save(instanceValue);
        }

    }

    public void saveInstanceRelation(InstanceRelation instanceRelation, String tableName) {
        synchronized (InstanceRelation.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceRelation.class, tableName);
            instanceRelationRepository.save(instanceRelation);
        }

    }

    public void batchSaveInstance(List<Instance> instances, String tableName) {
        synchronized (Instance.class) {
            CmdbUtils.changeTableAnnotationValue(Instance.class, tableName);
            instanceRepository.batchSave(instances, 100);
        }
    }

    public void batchSaveInstanceValue(List<InstanceValue> instanceValues, String tableName) {
        synchronized (InstanceValue.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceValue.class, tableName);
            instanceValueRepository.batchSave(instanceValues, 100);
        }

    }

    public void batchSaveInstanceRelation(List<InstanceRelation> instanceRelations, String tableName) {
        synchronized (InstanceRelation.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceRelation.class, tableName);
            instanceRelationRepository.batchSave(instanceRelations, 100);
        }

    }

    public void updateInstance(Instance instance, String tableName) {
        synchronized (Instance.class) {
            CmdbUtils.changeTableAnnotationValue(Instance.class, tableName);
            instanceRepository.update(instance);
        }
    }

    public void updateInstanceValue(InstanceValue instanceValue, String tableName) {
        synchronized (InstanceValue.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceValue.class, tableName);
            instanceValueRepository.update(instanceValue);
        }

    }

    public void updateInstanceRelation(InstanceRelation instanceRelation, String tableName) {
        synchronized (InstanceRelation.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceRelation.class, tableName);
            instanceRelationRepository.update(instanceRelation);
        }

    }

    public void batchUpdateInstance(List<Instance> instances, String tableName) {
        synchronized (Instance.class) {
            CmdbUtils.changeTableAnnotationValue(Instance.class, tableName);
            instanceRepository.batchUpdate(instances, 100);
        }
    }

    public void batchUpdateInstanceValue(List<InstanceValue> instanceValues, String tableName) {
        synchronized (InstanceValue.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceValue.class, tableName);
            instanceValueRepository.batchUpdate(instanceValues, 100);
        }

    }

    public void batchUpdateInstanceRelation(List<InstanceRelation> instanceRelations, String tableName) {
        synchronized (InstanceRelation.class) {
            CmdbUtils.changeTableAnnotationValue(InstanceRelation.class, tableName);
            instanceRelationRepository.batchUpdate(instanceRelations, 100);
        }

    }
}
