package com.bocloud.ams.task.listener;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.ams.task.exception.NodeFormatException;
import com.bocloud.ams.task.model.Job;
import com.bocloud.ams.task.model.JobResult;
import com.bocloud.ams.task.utils.ParamBuilder;
import com.bocloud.ams.task.worker.AbstractWorker;
import com.bocloud.ams.task.worker.WorkerFactory;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

/**
 * 运维类任务监听器
 */
@Slf4j
@Component
public class MaintainTaskListener extends AbstractTaskListener {
    private final WorkerFactory workerFactory;
    private ParamBuilder paramBuilder;

    @Autowired
    public MaintainTaskListener(WorkerFactory workerFactory) {
        this.workerFactory = workerFactory;
        this.paramBuilder = new ParamBuilder();
    }

    @Override
    public JobResult onTask(Job job) {
        String code = job.getParam(CODE);
        String name = job.getParam(NAME);
        String data = job.getParam(DATA);
        String time = job.getParam(TIME);
        log.info("receive a maintain task! name is: {} ,code is :{}", name, code);
        try {
            AbstractWorker worker = workerFactory.buildWorker(code, buildTimeout(time));
            Object param = paramBuilder.buildParam(code, data);
            if (null == worker) {
                log.error("maintain worker [{}] instance build error!", code);
                throw new NodeFormatException("worker instance build error!");
            }
            if (null == param) {
                log.error("maintain worker [{}] param build error!", code);
                throw new NodeFormatException("worker param build error!");
            }
            return buildResult(worker.execute(param), job);
        } catch (NodeFormatException e) {
            log.error("maintain worker param {} is illegal", code);
            return buildResult(GeneralResult.FAILED(e.getLocalizedMessage()), job);
        } catch (Exception e) {
            log.error("maintain worker occur unknown exception!", e);
            return buildResult(GeneralResult.FAILED(e.getMessage()), job);
        }
    }
}