package com.bocloud.ams.task.worker;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.exception.NodeFormatException;
import com.megatron.common.model.GeneralResult;

import com.megatron.common.model.Result;
import com.megatron.framework.http.core.ServiceFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 14:49 2018/12/27 @ Modified
 * By：
 */
@Slf4j
public abstract class AbstractWorker {
    String code;
    RabbitAdmin rabbitAdmin;
    ServiceFactory serviceFactory;
    private Integer timeout = Integer.valueOf(10 * 60);

    public AbstractWorker(String code, RabbitAdmin rabbitAdmin, ServiceFactory serviceFactory, Integer timeout) {
        this.code = code;
        if (null != timeout && timeout > 0) {
            this.timeout = timeout;
        }
        this.rabbitAdmin = rabbitAdmin;
        this.serviceFactory = serviceFactory;
    }

    public GeneralResult<?> execute(Object param) {
        GeneralResult<?> result = null;
        try {
            init(param);
            result = call();
        } catch (NodeFormatException e) {
            log.error("worker param illegal!", e);
            result = GeneralResult.FAILED("worker param illegal!!");
        }
        return result;
    }

    protected abstract void init(Object param) throws NodeFormatException;

    protected abstract GeneralResult<?> call();

    public Result buildResult(GeneralResult<?> result) {
        String message = JSONObject.toJSONString(result);
        return new Result(result.isSuccess(), message);
    }

    public Integer getTimeout() {
        return timeout;
    }

    public void setTimeout(Integer timeout) {
        this.timeout = timeout;
    }

    public RabbitAdmin getRabbitAdmin() {
        return rabbitAdmin;
    }

    public ServiceFactory getServiceFactory() {
        return serviceFactory;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }
}