package com.bocloud.ams.task.worker;

import com.bocloud.ams.task.exception.NodeFormatException;
import com.bocloud.ams.task.executor.WorkerDispatcher;
import com.bocloud.ams.task.input.Target;
import com.bocloud.ams.task.model.TargetGroup;
import com.bocloud.ams.task.model.WorkerHost;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.http.core.ServiceFactory;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 14:52 2018/12/27 @ Modified
 * By：
 */
@Slf4j
public abstract class MaintainWorker extends AbstractWorker {
    /**
     * 目标主机数量经验值
     */
    int expNum = 100;
    WorkerDispatcher dispatcher;
    List<List<WorkerHost>> targets;
    List<Map<String, Object>> paramList;

    public MaintainWorker(String code, RabbitAdmin rabbitAdmin, ServiceFactory serviceFactory, Integer timeout) {
        super(code, rabbitAdmin, serviceFactory, timeout);
        this.targets = new ArrayList<>();
        this.dispatcher = new WorkerDispatcher(serviceFactory, rabbitAdmin);
        paramList = Lists.newArrayList();
    }

    abstract void buildParam() throws NodeFormatException;

    @Override
    protected GeneralResult<?> call() {
        GeneralResult<?> result;
        try {
            // 任务分发并获取整体结果
            result = dispatcher.execute(paramList, getTimeout());
        } catch (NodeFormatException e) {
            result = GeneralResult.FAILED(e.getMessage());
        }
        if (log.isDebugEnabled()) {
            log.debug("task result is {}", result);
        }
        return result;
    }

    /**
     * 主机分片
     *
     * @param target
     */
    public void burst(Target target) throws NodeFormatException {
        GeneralResult<TargetGroup> targetGroupResult = groupTarget(target.getData());
        if (targetGroupResult.isSuccess()) {
            TargetGroup targetGroup = targetGroupResult.getData();
            this.targets.addAll(splitTarget(targetGroup.getLinuxs()));
            this.targets.addAll(splitTarget(targetGroup.getWindows()));
        }
    }

    /**
     * 将目标机器按照操作系统类型分为Windows和Linux两组
     *
     * @param targets
     * @return
     */
    private GeneralResult<TargetGroup> groupTarget(List<WorkerHost> targets) throws NodeFormatException {
        List<WorkerHost> linuxs = Lists.newArrayList();
        List<WorkerHost> windows = Lists.newArrayList();
        if (ListTool.isEmpty(targets)) {
            throw new NodeFormatException("目标机器为空!");
        }
        for (WorkerHost target : targets) {
            if (OSCategory.LINUX.name().equalsIgnoreCase(target.getCategory())) {
                linuxs.add(target);
            } else if (OSCategory.WINDOWS.name().equalsIgnoreCase(target.getCategory())) {
                windows.add(target);
            } else {
                log.error("未知操作系统，放置到Linux操作系统中");
                linuxs.add(target);
            }
        }
        return new GeneralResult<TargetGroup>(true, new TargetGroup(windows, linuxs), "success");
    }

    /**
     * 根据每个worker处理目标机器数的经验值划分每个worker处理的目标机器IP列表
     *
     * @param targets
     * @return
     */
    private List<List<WorkerHost>> splitTarget(List<WorkerHost> targets) {
        List<List<WorkerHost>> targetList = new ArrayList<>(16);
        int step;
        if (this.getCode().equalsIgnoreCase("task.maintain.recovery")) {
            step = 1;
        } else {
            step = expNum;
        }
        for (int i = 0; i < targets.size(); i = i + step) {
            List<WorkerHost> targetArray = new ArrayList<>(16);
            for (int j = i; j < i + step && j < targets.size(); j++) {
                WorkerHost workerHost = targets.get(j);
                workerHost.setPasd(workerHost.getPasd());
                targetArray.add(workerHost);
            }
            targetList.add(targetArray);
        }
        return targetList;
    }

    public static void main(String[] args) {
        AESEncryptor aesEncryptor = new AESEncryptor();
        System.out.println(aesEncryptor.decrypt("NVN3gZaJ1DETbj/TCezSU8orgmZbJOtpn0nR7vHL86I=", null));
        System.out.println(aesEncryptor.encrypt("sa@123", null));
        Encryptor encryptor = new AESEncryptor();
        System.out.println(encryptor.decrypt("NDVfYMofbBCFj/5abVN4xA==", null));
    }

    public int getExpNum() {
        return expNum;
    }

    public void setExpNum(int expNum) {
        this.expNum = expNum;
    }

    public List<List<WorkerHost>> getTargets() {
        return targets;
    }
}