package com.bocloud.ams.task.worker;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.annotation.Worker;
import com.bocloud.ams.task.input.HttpInput;
import com.bocloud.ams.task.model.HttpFront;
import com.megatron.common.enums.HttpRequestMethod;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.framework.http.core.ServiceFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * @ Description：Http接口调用worker执行器 @ Author ：丁明威 @ Date ：Created in 14:54
 * 2018/12/27 @ Modified By：
 */
@Worker(name = "task.maintain.method")
public class MethodWorker extends AbstractWorker {
    /**
     * 请求URL
     */
    private String url;
    /**
     * 超时时间，默认超时1分钟
     */
    private int timeout = 60 * 1000;
    /**
     * 请求方法
     */
    private HttpRequestMethod method;
    /**
     * 请求参数数据格式
     */
    private PostDataFormat postDataFormat;

    private Map<String, Object> header;
    private Map<String, Object> params;

    private HttpInput input;

    public MethodWorker(String code, RabbitAdmin rabbitAdmin, ServiceFactory serviceFactory, Integer timeout) {
        super(code, rabbitAdmin, serviceFactory, timeout);
        this.postDataFormat = PostDataFormat.FORM_DATA;
        if (null != timeout && timeout > 0) {
            this.timeout = timeout * 1000;
        }
    }

    @Override
    protected GeneralResult<?> call() {
        Result result;
        HttpClient httpClient = new HttpClient(timeout, postDataFormat);
        if (null != postDataFormat){
            switch (postDataFormat) {
                case RAW:
                    covertParam();
                    break;
                case BINARY:
                    covertParam();
                    break;
                default:
                    break;
            }
        }
        switch (method) {
            case GET:
                result = httpClient.get(header, params, url);
                break;
            case POST:
                result = httpClient.post(header, params, url);
                break;
            case PATCH:
                result = httpClient.patch(header, params, url);
                break;
            case PUT:
                result = httpClient.patch(header, params, url);
                break;
            case DELETE:
                result = httpClient.delete(header, params, url);
                break;
            case TRACE:
                result = httpClient.trace(header, params, url);
                break;
            default:
                // 默认采用get请求
                result = httpClient.get(header, params, url);
                break;
        }
        return new GeneralResult<>(result.isSuccess(), JSONObject.parseObject(result.getMessage()), "success");
    }

    @Override
    protected void init(Object param) {
        if (param instanceof HttpInput) {
            this.input = (HttpInput) param;
            HttpFront front = input.getFronts();
            this.url = front.getUrl();
            this.method = front.getMethod();
            this.params = front.getParams();
            this.header = front.getHeaders();
            this.postDataFormat = front.getFormat();
        }
    }

    private void covertParam() {
        if (StringUtils.hasText(this.input.getFronts().getBody())) {
            @SuppressWarnings("unchecked")
            Map<String, Object> body = JSONObject.parseObject(this.input.getFronts().getBody(), HashMap.class);
            params.putAll(body);
        }
    }
}